@@ -1,15 +1,20 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
 extern "C" {
+#include <afsconfig.h>
 #include <afs/param.h>
 #include <afs/stds.h>
+#include <roken.h>
 }
 
 #include <WINNT/afsclass.h>
@@ -760,7 +765,7 @@ void AfsClass_SkipRefresh (int idSection)
 
 DWORD WINAPI SERVER::CanTalkToServer_ThreadProc (PVOID lp)
 {
-   int idSection = (int)lp;
+   int idSection = PtrToInt(lp);
 
    // Until we post a notification saying that we've entered
    // a section, we don't need to worry about the aRefSec[] entry
@@ -781,10 +786,10 @@ DWORD WINAPI SERVER::CanTalkToServer_ThreadProc (PVOID lp)
    // has canceled the operation--and if no one has, there is a thread
    // hanging around which holds the library's critsec, which ensures
    // the lpServer pointer won't have been freed.)
-   // 
+   //
    PVOID hCell;
-   PVOID hBOS;
-   PVOID hVOS;
+   PVOID hBOS = NULL;
+   PVOID hVOS = NULL;
 
    TCHAR szServer[ cchNAME ];
 
@@ -984,7 +989,7 @@ BOOL SERVER::CanTalkToServer (ULONG *pStatus)
    //
    DWORD dwThreadID;
    HANDLE hThread;
-   if ((hThread = CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE)CanTalkToServer_ThreadProc, (PVOID)idSection, 0, &dwThreadID)) == NULL)
+   if ((hThread = CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE)CanTalkToServer_ThreadProc, IntToPtr(idSection), 0, &dwThreadID)) == NULL)
       {
       EnterCriticalSection (pcsRefSec);
       aRefSec[ idSection ].fInUse = FALSE;
@@ -1083,7 +1088,7 @@ BOOL SERVER::RefreshAll (ULONG *pStatus, double dInit, double dFactor)
                ++nAggregates;
                lpAggregate->Close();
                }
-               
+
             if (nAggregates)
                {
                for (lpAggregate = AggregateFindFirst (&hEnum); lpAggregate; lpAggregate = AggregateFindNext (&hEnum))
