@@ -7,6 +7,10 @@
 ////////////////////////////////////////////////////////////////////
 
 
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include <windows.h>
 #include <stdarg.h>
 #include <string.h>
@@ -212,7 +216,14 @@ LogEvent(WORD wEventType, DWORD dwEventID, ...)
         break;
     case MSG_SERVICE_RUNNING:
         wNumArgs = 1;
-        lpArgs[0] = "SMB interface";
+        if (smb_Enabled && RDR_Initialized)
+            lpArgs[0] = "SMB and RDR interfaces";
+        else if (smb_Enabled)
+            lpArgs[0] = "SMB interface";
+        else if (RDR_Initialized)
+            lpArgs[0] = "RDR interface";
+        else
+            lpArgs[0] = "No active interface";
         break;
     case MSG_FLUSH_BAD_SHARE_NAME:
     case MSG_FLUSH_OPEN_ENUM_ERROR:
