@@ -1,23 +1,29 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include <windows.h>
 #include <winsock2.h>
+#include <shlwapi.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <strsafe.h>
 
 #include "afsd.h"
 #include <WINNT\afssw.h>
 #include <WINNT\afsreg.h>
 
-#include <afs/param.h>
 #include <afs/stds.h>
 #include <afs/cellconfig.h>
 
@@ -30,7 +36,7 @@ static long cm_ParsePair(char *lineBufferp, char *leftp, char *rightp)
     char tc;
     int sawEquals;
     int sawBracket;
-        
+
     sawEquals = 0;
     sawBracket = 0;
     for(tp = lineBufferp; *tp; tp++) {
@@ -43,17 +49,17 @@ static long cm_ParsePair(char *lineBufferp, char *leftp, char *rightp)
 	}
 
 	/* comment or line end */
-        if (tc == '#' || tc == '\r' || tc == '\n') 
+        if (tc == '#' || tc == '\r' || tc == '\n')
             break;
 
 	/* square bracket comment -- look for closing delim */
 	if (tc == '[') {
-            sawBracket = 1; 
+            sawBracket = 1;
             continue;
-        }	
+        }
 
 	/* space or tab */
-        if (tc == ' ' || tc == '\t') 
+        if (tc == ' ' || tc == '\t')
             continue;
 
         if (tc == '=') {
@@ -64,8 +70,8 @@ static long cm_ParsePair(char *lineBufferp, char *leftp, char *rightp)
         /* now we have a real character, put it in the appropriate bucket */
         if (sawEquals == 0) {
 	    *leftp++ = tc;
-        }	
-        else {	
+        }
+        else {
 	    *rightp++ = tc;
         }
     }
@@ -91,7 +97,7 @@ IsWindowsModule(const char * name)
     p = strrchr(name, '.');
     if (p) {
         /* as of 2009-09-04 these are not valid ICANN ccTLDs */
-	if (i == 1 && 
+	if (i == 1 &&
 	    (!cm_stricmp_utf8N(p,".dll") ||
 	     !cm_stricmp_utf8N(p,".exe") ||
 	     !cm_stricmp_utf8N(p,".ini") ||
@@ -137,6 +143,7 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
     int foundCell = 0;
     long code;
     int tracking = 1, partial = 0;
+    size_t len;
 
     if ( IsWindowsModule(cellNamep) )
 	return -3;
@@ -145,13 +152,13 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
     if (*wdir)
         tfilep = fopen(wdir, "r");
 
-    if (!tfilep) 
+    if (!tfilep)
         return -2;
 
     bestp = fopen(wdir, "r");
-    
+
 #ifdef CELLSERV_DEBUG
-    osi_Log2(afsd_logp,"cm_searchfile fopen handle[%p], wdir[%s]", bestp, 
+    osi_Log2(afsd_logp,"cm_searchfile fopen handle[%p], wdir[%s]", bestp,
 	     osi_LogSaveString(afsd_logp,wdir));
 #endif
     /* have we seen the cell line for the guy we're looking for? */
@@ -181,7 +188,7 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
 		    return (foundCell? 0 : -3);
 		}
 	    }
-        }	
+        }
 
         /* turn trailing cr or lf into null */
         tp = strrchr(lineBuffer, '\r');
@@ -204,7 +211,7 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
                 return(foundCell ? 0 : -6);
             }
 
-            /* 
+            /*
              * terminate the cellname at the first white space
              * leaving 'tp' pointing to the next string if any
              */
@@ -215,7 +222,7 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
                 if (*tp != '#') {
                     linkp = tp;
                     for (; tp && !isspace(*tp); tp++);
-                    if (tp) 
+                    if (tp)
                         *tp = '\0';
                 }
             }
@@ -224,18 +231,24 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
             if (stricmp(lineBuffer+1, cellNamep) == 0) {
 		/* found the cell we're looking for */
 		if (newCellNamep) {
-		    strncpy(newCellNamep, lineBuffer+1,CELL_MAXNAMELEN);
-                    newCellNamep[CELL_MAXNAMELEN-1] = '\0';
+                    if (FAILED(StringCchCopy(newCellNamep, CELL_MAXNAMELEN, lineBuffer+1))) {
+                        fclose(tfilep);
+                        fclose(bestp);
+                        return(-1);
+                    }
                     strlwr(newCellNamep);
                 }
                 if (linkedNamep) {
-                    strncpy(linkedNamep, linkp ? linkp : "", CELL_MAXNAMELEN);
-                    linkedNamep[CELL_MAXNAMELEN-1] = '\0';
+                    if (FAILED(StringCchCopy(linkedNamep, CELL_MAXNAMELEN, linkp ? linkp : ""))) {
+                        fclose(tfilep);
+                        fclose(bestp);
+                        return(-1);
+                    }
                     strlwr(linkedNamep);
                 }
                 inRightCell = 1;
 		tracking = 0;
-#ifdef CELLSERV_DEBUG                
+#ifdef CELLSERV_DEBUG
 		osi_Log2(afsd_logp, "cm_searchfile is cell inRightCell[%p], linebuffer[%s]",
 			 inRightCell, osi_LogSaveString(afsd_logp,lineBuffer));
 #endif
@@ -248,13 +261,19 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
 		    return -5;
 		}
 		if (newCellNamep) {
-		    strncpy(newCellNamep, lineBuffer+1,CELL_MAXNAMELEN);
-                    newCellNamep[CELL_MAXNAMELEN-1] = '\0';
+                    if (FAILED(StringCchCopy(newCellNamep, CELL_MAXNAMELEN, lineBuffer+1))) {
+                        fclose(tfilep);
+                        fclose(bestp);
+                        return(-1);
+                    }
                     strlwr(newCellNamep);
                 }
                 if (linkedNamep) {
-                    strncpy(linkedNamep, linkp ? linkp : "", CELL_MAXNAMELEN);
-                    linkedNamep[CELL_MAXNAMELEN-1] = '\0';
+                    if (FAILED(StringCchCopy(linkedNamep, CELL_MAXNAMELEN, linkp ? linkp : ""))) {
+                        fclose(tfilep);
+                        fclose(bestp);
+                        return(-1);
+                    }
                     strlwr(linkedNamep);
                 }
 		inRightCell = 0;
@@ -273,11 +292,21 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
             valuep++;	/* skip the "#" */
 
             valuep += strspn(valuep, " \t"); /* skip SP & TAB */
-            /* strip spaces and tabs in the end. They should not be there according to CellServDB format
-             * so do this just in case                        
+            /*
+             * strip spaces and tabs in the end. They should not be there according to CellServDB format
+             * so do this just in case
 	     */
-            while (valuep[strlen(valuep) - 1] == ' ' || valuep[strlen(valuep) - 1] == '\t') 
-                valuep[strlen(valuep) - 1] = '\0';
+            if(FAILED(StringCchLength(valuep, sizeof(lineBuffer), &len))) {
+                fclose(tfilep);
+                fclose(bestp);
+                return(-1);
+            } else {
+                len--;
+                while(isspace(valuep[len]))
+                    valuep[len--] = '\0';
+                while(isspace(*valuep))
+                    valuep++;
+            }
 
 	    if (inRightCell) {
                 char hostname[256];
@@ -318,8 +347,8 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
                 if (!thp) {
                     afs_uint32 ip_addr;
 		    unsigned int c1, c2, c3, c4;
-                    
-                    /* Since there is no gethostbyname() data 
+
+                    /* Since there is no gethostbyname() data
 		     * available we will read the IP address
 		     * stored in the CellServDB file
                      */
@@ -353,17 +382,80 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
     return (foundCell) ? 0 : -11;
 }
 
+long
+cm_EnumerateCellFile(afs_uint32 client, cm_enumCellProc_t *procp, void *rockp)
+{
+    char wdir[MAX_PATH]="";
+    FILE *tfilep = NULL;
+    char *tp;
+    char lineBuffer[257];
+
+    if (!procp)
+        return 0;
+
+    cm_GetCellServDB(wdir, sizeof(wdir));
+    if (*wdir)
+        tfilep = fopen(wdir, "r");
+
+    if (!tfilep)
+        return -2;
+
+#ifdef CELLSERV_DEBUG
+    osi_Log2(afsd_logp,"cm_enumfile fopen handle[%p], wdir[%s]", tfilep,
+	     osi_LogSaveString(afsd_logp,wdir));
+#endif
+    while (1) {
+        tp = fgets(lineBuffer, sizeof(lineBuffer), tfilep);
+        if (tp == NULL && feof(tfilep)) {
+            /* hit EOF */
+            fclose(tfilep);
+            return (0);
+        }
+
+        /* turn trailing cr or lf into null */
+        tp = strrchr(lineBuffer, '\r');
+        if (tp) *tp = 0;
+        tp = strrchr(lineBuffer, '\n');
+        if (tp) *tp = 0;
+
+	/* skip blank lines */
+        if (lineBuffer[0] == 0)
+            continue;
+
+        /*
+         * The format is:
+         *   >[cell] [linked-cell] #[Description]
+         * where linked-cell and Description are optional
+         * but we are only going to use the initial cell name
+         */
+        if (lineBuffer[0] == '>') {
+            /*
+             * terminate the cellname at the first white space
+             * leaving 'tp' pointing to the next string if any
+             */
+            for (tp = &lineBuffer[1]; tp && !isspace(*tp); tp++);
+            if (tp)
+                *tp = '\0';
+
+            /* Now process the cell */
+            (*procp)(rockp, &lineBuffer[1]);
+        }
+    }		/* while loop processing all lines */
+
+    return 0;
+}
+
 /*
  * The CellServDB registry schema is as follows:
  *
  * HKLM\SOFTWARE\OpenAFS\Client\CellServDB\[cellname]\
- *   "LinkedCell"    REG_SZ "[cellname]" 
+ *   "LinkedCell"    REG_SZ "[cellname]"
  *   "Description"   REG_SZ "[comment]"
  *   "ForceDNS"      DWORD  {0,1}
  *
  * HKLM\SOFTWARE\OpenAFS\Client\CellServDB\[cellname]\[servername]\
- *   "HostName"      REG_SZ "[hostname]" 
- *   "IPv4Address"   REG_SZ "[address]" 
+ *   "HostName"      REG_SZ "[hostname]"
+ *   "IPv4Address"   REG_SZ "[address]"
  *   "IPv6Address"   REG_SZ "[address]"   <future>
  *   "Comment"       REG_SZ "[comment]"
  *   "Rank"          DWORD  "0..65535"
@@ -377,7 +469,7 @@ long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
  * to a valid server configuration, the return code is success.
  * This prevents failover to the CellServDB file or DNS.
  */
-long cm_SearchCellRegistry(afs_uint32 client, 
+long cm_SearchCellRegistry(afs_uint32 client,
                            char *cellNamep, char *newCellNamep,
                            char *linkedNamep,
                            cm_configProc_t *procp, void *rockp)
@@ -397,6 +489,7 @@ long cm_SearchCellRegistry(afs_uint32 client,
     struct hostent *thp;
     struct sockaddr_in vlSockAddr;
     char * s;
+    size_t len;
 
     if ( IsWindowsModule(cellNamep) )
 	return -1;
@@ -408,14 +501,14 @@ long cm_SearchCellRegistry(afs_uint32 client,
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                       AFSREG_CLT_OPENAFS_SUBKEY "\\CellServDB",
                       0,
-                      KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                      KEY_READ|KEY_QUERY_VALUE,
                       &hkCellServDB) != ERROR_SUCCESS)
         return CM_ERROR_NOSUCHCELL;
 
-    if (RegOpenKeyEx( hkCellServDB, 
+    if (RegOpenKeyEx( hkCellServDB,
                       cellNamep,
                       0,
-                      KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                      KEY_READ|KEY_QUERY_VALUE,
                       &hkCellName) != ERROR_SUCCESS) {
         BOOL bFound = 0;
 
@@ -437,13 +530,13 @@ long cm_SearchCellRegistry(afs_uint32 client,
         if (code != ERROR_SUCCESS)
             dwCells = 0;
 
-        /* 
+        /*
          * We search the entire list to ensure that there is only
          * one prefix match.  If there is more than one, we return none.
          */
         for ( dwIndex = 0; dwIndex < dwCells; dwIndex++ ) {
             dwSize = CELL_MAXNAMELEN;
-            code = RegEnumKeyEx( hkCellServDB, dwIndex, szCellName, &dwSize, NULL, 
+            code = RegEnumKeyEx( hkCellServDB, dwIndex, szCellName, &dwSize, NULL,
                                  NULL, NULL, &ftLastWriteTime);
             if (code != ERROR_SUCCESS)
                 continue;
@@ -451,7 +544,10 @@ long cm_SearchCellRegistry(afs_uint32 client,
             strlwr(szCellName);
 
             /* if not a prefix match, try the next key */
-            if (strncmp(cellNamep, szCellName, strlen(cellNamep)))
+            if (FAILED(StringCchLength(cellNamep, CELL_MAXNAMELEN, &len)))
+                continue;
+
+            if (strncmp(cellNamep, szCellName, len))
                 continue;
 
             /* If we have a prefix match and we already found another
@@ -462,16 +558,16 @@ long cm_SearchCellRegistry(afs_uint32 client,
                 break;
             }
 
-            if (RegOpenKeyEx( hkCellServDB, 
+            if (RegOpenKeyEx( hkCellServDB,
                               szCellName,
                               0,
-                              KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                              KEY_READ|KEY_QUERY_VALUE,
                               &hkCellName) != ERROR_SUCCESS)
                 continue;
 
             if (newCellNamep) {
-                strncpy(newCellNamep, szCellName, CELL_MAXNAMELEN);
-                newCellNamep[CELL_MAXNAMELEN-1] = '\0';
+                if (FAILED(StringCchCopy(newCellNamep, CELL_MAXNAMELEN, cellNamep)))
+                    goto done;
             }
             bFound = 1;
         }
@@ -509,7 +605,7 @@ long cm_SearchCellRegistry(afs_uint32 client,
         dwForceDNS = 0;
     }
 
-    /* 
+    /*
      * Using the defined server list.  Enumerate and populate
      * the server list for the cell.
      */
@@ -531,22 +627,22 @@ long cm_SearchCellRegistry(afs_uint32 client,
 
     for ( dwIndex = 0; dwIndex < dwServers; dwIndex++ ) {
         dwSize = MAXHOSTCHARS;
-        code = RegEnumKeyEx( hkCellName, dwIndex, szServerName, &dwSize, NULL, 
+        code = RegEnumKeyEx( hkCellName, dwIndex, szServerName, &dwSize, NULL,
                              NULL, NULL, &ftLastWriteTime);
         if (code != ERROR_SUCCESS)
             continue;
 
         szServerName[MAXHOSTCHARS-1] = '\0';
-        if (RegOpenKeyEx( hkCellName, 
+        if (RegOpenKeyEx( hkCellName,
                           szServerName,
                           0,
-                          KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                          KEY_READ|KEY_QUERY_VALUE,
                           &hkServerName) != ERROR_SUCCESS)
             continue;
 
-        /* We have a handle to a valid server key.  Now we need 
+        /* We have a handle to a valid server key.  Now we need
          * to add the server to the cell */
-        
+
         /* First, see if there is an alternate hostname specified */
         dwSize = MAXHOSTCHARS;
         code = RegQueryValueEx(hkServerName, "HostName", NULL, &dwType,
@@ -599,7 +695,7 @@ long cm_SearchCellRegistry(afs_uint32 client,
             afs_uint32 ip_addr;
             unsigned int c1, c2, c3, c4;
 
-            /* Since there is no gethostbyname() data 
+            /* Since there is no gethostbyname() data
              * available we will read the IP address
              * stored in the CellServDB file
              */
@@ -631,7 +727,113 @@ long cm_SearchCellRegistry(afs_uint32 client,
     return ((dwForceDNS || dwServers == 0) ? CM_ERROR_FORCE_DNS_LOOKUP : 0);
 }
 
-long cm_EnumerateCellRegistry(afs_uint32 client, cm_enumCellRegistryProc_t *procp, void *rockp)
+/*
+ * Following the registry schema listed above, cm_AddCellToRegistry
+ * will either create or update the registry configuration data for
+ * the specified cellname.
+ */
+long cm_AddCellToRegistry( char * cellname,
+                           char * linked_cellname,
+                           unsigned short vlport,
+                           afs_uint32 host_count,
+                           char *hostname[],
+                           afs_uint32 flags)
+{
+    HKEY hkCellServDB = 0, hkCellName = 0, hkServerName = 0;
+    DWORD dwPort, dwDisposition;
+    long code;
+    unsigned int i;
+
+    if (RegCreateKeyEx( HKEY_LOCAL_MACHINE,
+                        AFSREG_CLT_OPENAFS_SUBKEY "\\CellServDB",
+                        0,
+                        NULL,
+                        REG_OPTION_NON_VOLATILE,
+                        KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                        NULL,
+                        &hkCellServDB,
+                        &dwDisposition) != ERROR_SUCCESS) {
+        code = CM_ERROR_NOACCESS;
+        goto done;
+    }
+
+    /* Perform a recursive deletion of the cellname key */
+    SHDeleteKey( hkCellServDB, cellname);
+
+    if (RegCreateKeyEx( hkCellServDB,
+                        cellname,
+                        0,
+                        NULL,
+                        REG_OPTION_NON_VOLATILE,
+                        KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                        NULL,
+                        &hkCellName,
+                        &dwDisposition) != ERROR_SUCCESS ||
+        (dwDisposition == REG_OPENED_EXISTING_KEY) ) {
+        code = CM_ERROR_NOACCESS;
+        goto done;
+    }
+
+    /* If we get here, hkCellName is a handle to an empty key */
+
+    if (linked_cellname && linked_cellname[0]) {
+        code = RegSetValueEx( hkCellName, "LinkedCell",
+                              0, REG_SZ,
+                              (BYTE *) linked_cellname, CELL_MAXNAMELEN);
+    }
+
+    if (flags & CM_CELLFLAG_DNS) {
+        DWORD dwForceDNS = 1;
+        code = RegSetValueEx( hkCellName, "ForceDNS",
+                              0, REG_DWORD,
+                              (BYTE *) &dwForceDNS, sizeof(DWORD));
+    }
+
+    for ( i = 0; i < host_count; i++ ) {
+        if (RegCreateKeyEx( hkCellName,
+                            hostname[i],
+                            0,
+                            NULL,
+                            REG_OPTION_NON_VOLATILE,
+                            KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
+                            NULL,
+                            &hkServerName,
+                            &dwDisposition) != ERROR_SUCCESS ||
+             (dwDisposition == REG_OPENED_EXISTING_KEY)) {
+            code = CM_ERROR_NOACCESS;
+            goto done;
+        }
+
+        /* We have a handle to a valid server key.  Now we need
+         * to add the server to the cell */
+
+        /* First, see if there is an alternate hostname specified */
+        code = RegSetValueEx( hkServerName, "HostName",
+                              0, REG_SZ,
+                              (BYTE *) hostname[i], (DWORD)strlen(hostname[i]) + 1);
+
+        if (vlport) {
+            dwPort = vlport;
+            code = RegSetValueEx( hkServerName, "vlserver",
+                                  0, REG_DWORD,
+                                  (BYTE *) &dwPort, (DWORD)sizeof(DWORD));
+        }
+        RegCloseKey( hkServerName);
+        hkServerName = 0;
+    }
+
+  done:
+    if (hkServerName)
+        RegCloseKey(hkServerName);
+    if (hkCellName)
+        RegCloseKey(hkCellName);
+    if (hkCellServDB)
+        RegCloseKey(hkCellServDB);
+
+    return code;
+}
+
+long cm_EnumerateCellRegistry(afs_uint32 client, cm_enumCellProc_t *procp, void *rockp)
 {
     HKEY hkCellServDB = 0;
     DWORD dwSize;
@@ -668,12 +870,12 @@ long cm_EnumerateCellRegistry(afs_uint32 client, cm_enumCellRegistryProc_t *proc
     if (code != ERROR_SUCCESS)
         dwCells = 0;
 
-    /* 
-     * Enumerate each Cell and 
+    /*
+     * Enumerate each Cell and
      */
     for ( dwIndex = 0; dwIndex < dwCells; dwIndex++ ) {
         dwSize = CELL_MAXNAMELEN;
-        code = RegEnumKeyEx( hkCellServDB, dwIndex, szCellName, &dwSize, NULL, 
+        code = RegEnumKeyEx( hkCellServDB, dwIndex, szCellName, &dwSize, NULL,
                              NULL, NULL, &ftLastWriteTime);
         if (code != ERROR_SUCCESS)
             continue;
@@ -699,6 +901,7 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
     int numServers;
     int i;
     struct sockaddr_in vlSockAddr;
+
 #ifdef CELLSERV_DEBUG
     osi_Log1(afsd_logp,"SearchCellDNS-Doing search for [%s]", osi_LogSaveString(afsd_logp,cellNamep));
 #endif
@@ -711,7 +914,7 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
      */
     if ( IsWindowsModule(cellNamep) ||
          cm_FsStrChr(cellNamep, '.') == NULL ||
-         strncasecmp(cellNamep, CM_IOCTL_FILENAME_NOSLASH, strlen(CM_IOCTL_FILENAME_NOSLASH)) == 0)
+         strncasecmp(cellNamep, CM_IOCTL_FILENAME_NOSLASH, sizeof(CM_IOCTL_FILENAME_NOSLASH)-1) == 0)
 	return -1;
 
     rc = getAFSServer("afs3-vlserver", "udp", cellNamep, htons(7003),
@@ -726,8 +929,8 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
                 (*procp)(rockp, &vlSockAddr, cellHostNames[i], ipRanks[i]);
         }
         if (newCellNamep) {
-            strncpy(newCellNamep,cellNamep,CELL_MAXNAMELEN);
-            newCellNamep[CELL_MAXNAMELEN-1] = '\0';
+            if(FAILED(StringCchCopy(newCellNamep, CELL_MAXNAMELEN, cellNamep)))
+                return -1;
             strlwr(newCellNamep);
         }
         return 0;   /* found cell */
@@ -736,32 +939,34 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
        return -1;  /* not found */
 }
 
-/* use cm_GetConfigDir() plus AFS_CELLSERVDB to 
- * generate the fully qualified name of the CellServDB 
+/* use cm_GetConfigDir() plus AFS_CELLSERVDB to
+ * generate the fully qualified name of the CellServDB
  * file.
  */
 long cm_GetCellServDB(char *cellNamep, afs_uint32 len)
 {
     size_t tlen;
-    
+
     cm_GetConfigDir(cellNamep, len);
 
     /* add trailing backslash, if required */
-    tlen = (int)strlen(cellNamep);
+    if (FAILED(StringCchLength(cellNamep, len, &tlen)))
+        return(-1L);
+
     if (tlen) {
         if (cellNamep[tlen-1] != '\\') {
-            strncat(cellNamep, "\\", len);
-            cellNamep[len-1] = '\0';
+            if (FAILED(StringCchCat(cellNamep, len, "\\")))
+                return(-1L);
         }
-        
-        strncat(cellNamep, AFS_CELLSERVDB, len);
-        cellNamep[len-1] = '\0';
+
+        if (FAILED(StringCchCat(cellNamep, len, AFS_CELLSERVDB)))
+            return(-1L);
     }
     return 0;
 }
 
-/* look up the root cell's name in the Registry 
- * Input buffer must be at least CELL_MAXNAMELEN 
+/* look up the root cell's name in the Registry
+ * Input buffer must be at least CELL_MAXNAMELEN
  * in size.  (Defined in cm_cell.h)
  */
 long cm_GetRootCellName(char *cellNamep)
@@ -791,18 +996,19 @@ cm_configFile_t *cm_CommonOpen(char *namep, char *rwp)
 
     cm_GetConfigDir(wdir, sizeof(wdir));
     if (*wdir) {
-        strncat(wdir, namep, sizeof(wdir));
-        wdir[sizeof(wdir)-1] = '\0';
-        
+        if (FAILED(StringCchCat(wdir, MAX_PATH, namep)))
+            return NULL;
+
         tfilep = fopen(wdir, rwp);
     }
-    return ((cm_configFile_t *) tfilep);        
-}	
+    return ((cm_configFile_t *) tfilep);
+}
 
 long cm_WriteConfigString(char *labelp, char *valuep)
 {
     DWORD code, dummyDisp;
     HKEY parmKey;
+    size_t len;
 
     code = RegCreateKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
 			   0, "container", 0, KEY_SET_VALUE, NULL,
@@ -810,13 +1016,15 @@ long cm_WriteConfigString(char *labelp, char *valuep)
     if (code != ERROR_SUCCESS)
 	return -1;
 
-    code = RegSetValueEx(parmKey, labelp, 0, REG_SZ,
-			  valuep, (DWORD)strlen(valuep) + 1);
+    if (FAILED(StringCchLength(valuep, CM_CONFIGDEFAULT_CELLS, &len)))
+        return -1;
+
+    code = RegSetValueEx(parmKey, labelp, 0, REG_SZ, valuep, (DWORD)(len + 1));
     RegCloseKey (parmKey);
     if (code != ERROR_SUCCESS)
-	return (long)-1;
+	return -1;
 
-    return (long)0;
+    return 0;
 }
 
 long cm_WriteConfigInt(char *labelp, long value)
@@ -857,7 +1065,7 @@ long cm_AppendPrunedCellList(cm_configFile_t *ofp, char *cellNamep)
     int foundCell;
 
     tfilep = cm_CommonOpen(AFS_CELLSERVDB, "r");
-    if (!tfilep) 
+    if (!tfilep)
         return -1;
 
     foundCell = 0;
@@ -879,7 +1087,7 @@ long cm_AppendPrunedCellList(cm_configFile_t *ofp, char *cellNamep)
         if (tp) *tp = 0;
         tp = strchr(lineBuffer, '\n');
         if (tp) *tp = 0;
-                
+
         /* skip blank lines */
         if (lineBuffer[0] == 0) {
             fprintf((FILE *)ofp, "%s\n", lineBuffer);
@@ -914,7 +1122,7 @@ long cm_AppendPrunedCellList(cm_configFile_t *ofp, char *cellNamep)
             }
         }	/* a vldb line */
     }		/* while loop processing all lines */
-}       
+}
 
 long cm_AppendNewCell(cm_configFile_t *filep, char *cellNamep)
 {
@@ -933,42 +1141,42 @@ long cm_CloseCellFile(cm_configFile_t *filep)
     char wdir[MAX_PATH];
     char sdir[MAX_PATH];
     long code;
-    long closeCode;
-    closeCode = fclose((FILE *)filep);
+    long closeCode = fclose((FILE *)filep);
 
     cm_GetConfigDir(wdir, sizeof(wdir));
-    strcpy(sdir, wdir);
+    if (FAILED(StringCchCopy(sdir, sizeof(sdir), wdir)))
+        return -1;
 
     if (closeCode != 0) {
         /* something went wrong, preserve original database */
-        strncat(wdir, AFS_CELLSERVDB ".new", sizeof(wdir));
-        wdir[sizeof(wdir)-1] = '\0';
+        if (FAILED(StringCchCat(wdir, sizeof(wdir), AFS_CELLSERVDB ".new")))
+            return -1;
         unlink(wdir);
         return closeCode;
     }
 
-    strncat(wdir, AFS_CELLSERVDB, sizeof(wdir));
-    wdir[sizeof(wdir)-1] = '\0';
-    strncat(sdir, AFS_CELLSERVDB ".new", sizeof(sdir));/* new file */
-    sdir[sizeof(sdir)-1] = '\0';
+    if (FAILED(StringCchCat(wdir, sizeof(wdir), AFS_CELLSERVDB)))
+        return -1;
+    if (FAILED(StringCchCat(sdir, sizeof(sdir), AFS_CELLSERVDB ".new"))) /* new file */
+        return -1;
 
     unlink(sdir);			/* delete old file */
 
     code = rename(sdir, wdir);	/* do the rename */
 
-    if (code) 
+    if (code)
         code = errno;
 
     return code;
-}   
+}
 
 void cm_GetConfigDir(char *dir, afs_uint32 len)
 {
     char * dirp = NULL;
 
     if (!afssw_GetClientCellServDBDir(&dirp)) {
-        strncpy(dir, dirp, len);
-        dir[len-1] = '\0';
+        if (FAILED(StringCchCopy(dir, len, dirp)))
+            return;
         free(dirp);
     } else {
         dir[0] = '\0';
