@@ -142,7 +142,7 @@ cm_PingServer(cm_server_t *tsp)
     afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
     lock_ReleaseMutex(&tsp->mx);
 
-    code = cm_ConnByServer(tsp, cm_rootUserp, &connp);
+    code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &connp);
     if (code == 0) {
 	/* now call the appropriate ping call.  Drop the timeout if
 	* the server is known to be down, so that we don't waste a
@@ -172,13 +172,13 @@ cm_PingServer(cm_server_t *tsp)
     }	/* got an unauthenticated connection to this server */
 
     lock_ObtainMutex(&tsp->mx);
-    if (code >= 0 || code == RXGEN_OPCODE || code == CM_RX_RETRY_BUSY_CALL) {
+    if (code >= 0 || code == RXGEN_OPCODE || code == RX_CALL_BUSY) {
 	/* mark server as up */
 	_InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
         tsp->downTime = 0;
 
 	/* we currently handle 32-bits of capabilities */
-	if (code != RXGEN_OPCODE && code != CM_RX_RETRY_BUSY_CALL &&
+	if (code != RXGEN_OPCODE && code != RX_CALL_BUSY &&
             caps.Capabilities_len > 0) {
 	    tsp->capabilities = caps.Capabilities_val[0];
 	    xdr_free((xdrproc_t) xdr_Capabilities, &caps);
@@ -418,7 +418,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ReleaseMutex(&tsp->mx);
 
             serversp[nconns] = tsp;
-            code = cm_ConnByServer(tsp, cm_rootUserp, &conns[nconns]);
+            code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &conns[nconns]);
             if (code) {
                 lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
@@ -457,13 +457,13 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
             if (results[i] >= 0 || results[i] == RXGEN_OPCODE ||
-                results[i] == CM_RX_RETRY_BUSY_CALL)  {
+                results[i] == RX_CALL_BUSY)  {
                 /* mark server as up */
                 _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
                 tsp->downTime = 0;
 
                 /* we currently handle 32-bits of capabilities */
-                if (results[i] != RXGEN_OPCODE && results[i] != CM_RX_RETRY_BUSY_CALL &&
+                if (results[i] != RXGEN_OPCODE && results[i] != RX_CALL_BUSY &&
                     caps[i].Capabilities_len > 0) {
                     tsp->capabilities = caps[i].Capabilities_val[0];
                     xdr_free((xdrproc_t) xdr_Capabilities, &caps[i]);
@@ -584,7 +584,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ReleaseMutex(&tsp->mx);
 
             serversp[nconns] = tsp;
-            code = cm_ConnByServer(tsp, cm_rootUserp, &conns[nconns]);
+            code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &conns[nconns]);
             if (code) {
                 lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
@@ -623,7 +623,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if (results[i] >= 0 || results[i] == CM_RX_RETRY_BUSY_CALL)  {
+            if (results[i] >= 0 || results[i] == RX_CALL_BUSY)  {
                 /* mark server as up */
                 _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
                 tsp->downTime = 0;
