@@ -7,9 +7,12 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
 #include <afs/stds.h>
 
+#include <roken.h>
+
 #include <windows.h>
 #include <winsock2.h>
 #include <nb30.h>
@@ -73,6 +76,7 @@ cm_ShutdownVolume(void)
                 cm_VolumeStatusNotification(volp, volp->vol[volType].ID, volp->vol[volType].state, vl_alldown);
         }
         volp->cbExpiresRO = 0;
+        volp->cbIssuedRO = 0;
         volp->cbServerpRO = NULL;
         lock_FinalizeRWLock(&volp->rw);
     }
@@ -113,6 +117,7 @@ void cm_InitVolume(int newFile, long maxVols)
                         cm_VolumeStatusNotification(volp, volp->vol[volType].ID, vl_unknown, volp->vol[volType].state);
                 }
                 volp->cbExpiresRO = 0;
+                volp->cbIssuedRO = 0;
                 volp->cbServerpRO = NULL;
             }
         }
@@ -271,9 +276,9 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
     }
 
 #ifdef AFS_FREELANCE_CLIENT
-    if (cellp->cellID == AFS_FAKE_ROOT_CELL_ID)
+    if ( cellp->cellID == AFS_FAKE_ROOT_CELL_ID && volp->vol[RWVOL].ID == AFS_FAKE_ROOT_VOL_ID )
     {
-        freelance = 1;
+	freelance = 1;
         memset(&vldbEntry, 0, sizeof(vldbEntry));
         vldbEntry.flags |= VLF_RWEXISTS;
         vldbEntry.volumeId[0] = AFS_FAKE_ROOT_VOL_ID;
@@ -307,56 +312,54 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         }
 
         _InterlockedOr(&volp->flags, CM_VOLUMEFLAG_UPDATING_VL);
-
-        /* Do not hold the volume lock across the RPC calls */
         lock_ReleaseWrite(&volp->rw);
 
         if (cellp->flags & CM_CELLFLAG_VLSERVER_INVALID)
-            cm_UpdateCell(cellp, 0);
+             cm_UpdateCell(cellp, 0);
 
         /* now we have volume structure locked and held; make RPC to fill it */
         code = cm_GetEntryByName(cellp, volp->namep, &vldbEntry, &nvldbEntry,
                                  &uvldbEntry,
                                  &method, userp, reqp);
-
-        /* We can end up here with code == CM_ERROR_NOSUCHVOLUME if the base volume name
-         * does not exist and is not a numeric string but there might exist a .readonly volume.
-         * If the base name doesn't exist we will not care about the .backup that might be left
-         * behind since there should be no method to access it.
-         */
-        if (code == CM_ERROR_NOSUCHVOLUME &&
-             _atoi64(volp->namep) == 0 &&
-             volp->vol[RWVOL].ID == 0 &&
-             strlen(volp->namep) < (VL_MAXNAMELEN - 9)) {
-            char name[VL_MAXNAMELEN];
-
-            snprintf(name, VL_MAXNAMELEN, "%s.readonly", volp->namep);
-
-            /* now we have volume structure locked and held; make RPC to fill it */
-            code = cm_GetEntryByName(cellp, name, &vldbEntry, &nvldbEntry,
-                                     &uvldbEntry,
-                                     &method, userp, reqp);
-        }
-
-        /*
-         * What if there was a volume rename?  The volume name no longer exists but the
-         * volume id might.  Try to refresh the volume location information based one
-         * of the readwrite or readonly volume id.
-         */
-        if (code == CM_ERROR_NOSUCHVOLUME) {
-            if (volp->vol[RWVOL].ID != 0) {
-                code = cm_GetEntryByID(cellp, volp->vol[RWVOL].ID, &vldbEntry, &nvldbEntry,
-                                       &uvldbEntry,
-                                       &method, userp, reqp);
-            } else if (volp->vol[ROVOL].ID != 0) {
-                code = cm_GetEntryByID(cellp, volp->vol[ROVOL].ID, &vldbEntry, &nvldbEntry,
-                                       &uvldbEntry,
-                                       &method, userp, reqp);
-            }
-        }
-        lock_ObtainWrite(&volp->rw);
     }
 
+    /* We can end up here with code == CM_ERROR_NOSUCHVOLUME if the base volume name
+     * does not exist and is not a numeric string but there might exist a .readonly volume.
+     * If the base name doesn't exist we will not care about the .backup that might be left
+     * behind since there should be no method to access it.
+     */
+    if (code == CM_ERROR_NOSUCHVOLUME &&
+        _atoi64(volp->namep) == 0 &&
+        volp->vol[RWVOL].ID == 0 &&
+        strlen(volp->namep) < (VL_MAXNAMELEN - 9)) {
+        char name[VL_MAXNAMELEN];
+
+        snprintf(name, VL_MAXNAMELEN, "%s.readonly", volp->namep);
+
+        /* now we have volume structure locked and held; make RPC to fill it */
+        code = cm_GetEntryByName(cellp, name, &vldbEntry, &nvldbEntry,
+                                 &uvldbEntry,
+                                 &method, userp, reqp);
+    }
+
+    /*
+     * What if there was a volume rename?  The volume name no longer exists but the
+     * volume id might.  Try to refresh the volume location information based one
+     * of the readwrite or readonly volume id.
+     */
+    if (code == CM_ERROR_NOSUCHVOLUME) {
+        if (volp->vol[RWVOL].ID != 0) {
+            code = cm_GetEntryByID(cellp, volp->vol[RWVOL].ID, &vldbEntry, &nvldbEntry,
+                                    &uvldbEntry,
+                                    &method, userp, reqp);
+        } else if (volp->vol[ROVOL].ID != 0) {
+            code = cm_GetEntryByID(cellp, volp->vol[ROVOL].ID, &vldbEntry, &nvldbEntry,
+                                    &uvldbEntry,
+                                    &method, userp, reqp);
+        }
+    }
+
+    lock_ObtainWrite(&volp->rw);
     if (code == 0) {
         afs_int32 flags;
         afs_int32 nServers;
@@ -442,7 +445,6 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                         if (code)
                             continue;
 
-			xdr_free((xdrproc_t) xdr_bulkaddrs, &addrs);
                         rxconnp = cm_GetRxConn(connp);
                         code = VL_GetAddrsU(rxconnp, &attrs, &uuid, &unique, &nentries, &addrs);
                         rx_PutConnection(rxconnp);
@@ -456,8 +458,6 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                     }
                     osi_Log1(afsd_logp, "CALL VL_GetAddrsU serverNumber %u SUCCESS", i);
 
-		    nentries = min(nentries, addrs.bulkaddrs_len);
-
                     addrp = addrs.bulkaddrs_val;
                     for (k = 0; k < nentries && j < NMAXNSERVERS; j++, k++) {
                         serverFlags[j] = uvldbEntry.serverFlags[i];
@@ -975,6 +975,7 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
             volp->vol[volType].flags = 0;
         }
         volp->cbExpiresRO = 0;
+        volp->cbIssuedRO = 0;
         volp->cbServerpRO = NULL;
         volp->creationDateRO = 0;
         cm_AddVolumeToNameHashTable(volp);
@@ -1231,12 +1232,22 @@ cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32
     cm_conn_t *connp;
     long code;
     AFSFetchVolumeStatus volStat;
+    char *Name;
+    char *OfflineMsg;
+    char *MOTD;
     cm_req_t req;
     struct rx_connection * rxconnp;
+    char volName[32];
+    char offLineMsg[256];
+    char motd[256];
     long alldown, alldeleted;
     cm_serverRef_t *serversp;
     cm_fid_t fid;
 
+    Name = volName;
+    OfflineMsg = offLineMsg;
+    MOTD = motd;
+
     if (statep->ID != 0 && (!volID || volID == statep->ID)) {
         /* create fid for volume root so that VNOVOL and VMOVED errors can be processed */
         cm_SetFid(&fid, volp->cellp->cellID, statep->ID, 1, 1);
@@ -1278,10 +1289,6 @@ cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32
 
                 lock_ReleaseWrite(&volp->rw);
                 do {
-		    char *Name = NULL;
-		    char *OfflineMsg = NULL;
-		    char *MOTD = NULL;
-
                     code = cm_ConnFromVolume(volp, statep->ID, cm_rootUserp, &req, &connp);
                     if (code)
                         continue;
@@ -1290,11 +1297,6 @@ cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32
                     code = RXAFS_GetVolumeStatus(rxconnp, statep->ID,
                                                  &volStat, &Name, &OfflineMsg, &MOTD);
                     rx_PutConnection(rxconnp);
-
-		    xdr_free((xdrproc_t) xdr_string, &Name);
-		    xdr_free((xdrproc_t) xdr_string, &OfflineMsg);
-		    xdr_free((xdrproc_t) xdr_string, &MOTD);
-
                 } while (cm_Analyze(connp, cm_rootUserp, &req, &fid, 0, NULL, NULL, NULL, code));
                 code = cm_MapRPCError(code, &req);
 
@@ -1552,7 +1554,9 @@ int cm_DumpVolumes(FILE *outputFile, char *cookie, int lock)
         if (volp->cbServerpRO) {
             if (!((volp->cbServerpRO->flags & CM_SERVERFLAG_UUID) &&
                 UuidToString((UUID *)&volp->cbServerpRO->uuid, &srvStr) == RPC_S_OK)) {
-                afs_asprintf(&srvStr, "%.0I", volp->cbServerpRO->addr.sin_addr.s_addr);
+                srvStr = malloc(16);
+                if (srvStr != NULL)
+                    afs_inet_ntoa_r(volp->cbServerpRO->addr.sin_addr.s_addr, srvStr);
                 srvStrRpc = FALSE;
             }
         }
@@ -1854,7 +1858,7 @@ cm_VolumeRenewROCallbacks(void)
             cm_InitReq(&req);
 
             lock_ReleaseRead(&cm_volumeLock);
-            if (cm_GetSCache(&fid, &scp, cm_rootUserp, &req) == 0) {
+            if (cm_GetSCache(&fid, NULL, &scp, cm_rootUserp, &req) == 0) {
                 lock_ObtainWrite(&scp->rw);
                 cm_GetCallback(scp, cm_rootUserp, &req, 1);
                 lock_ReleaseWrite(&scp->rw);
