@@ -33,6 +33,8 @@ typedef struct cm_vol_state {
 /* RWVOL, ROVOL, BACKVOL are defined in cm.h */
 #define NUM_VOL_TYPES 3
 
+#define CM_VOL_STATE_FLAG_REPLICATED    1
+
 typedef struct cm_volume {
     osi_queue_t q;                      /* LRU queue; cm_volumeLock */
     afs_uint32 qflags;                  /* by cm_volumeLock */
@@ -48,6 +50,7 @@ typedef struct cm_volume {
     afs_int32 refCount;		        /* by Interlocked operations */
     struct cm_server *cbServerpRO;      /* server granting RO callback; by cm_scacheLock */
     time_t cbExpiresRO;                 /* latest RO expiration time; by cm_scacheLock */
+    time_t cbIssuedRO;                  /* latest RO issue time; by cm_scacheLock */
     time_t creationDateRO;              /* latest volume creation date; 0 if unknown; by cm_scacheLock */
     time_t lastUpdateTime;              /* most recent volume location update cm_volumeLock */
 } cm_volume_t;
