@@ -45,6 +45,8 @@ extern void                   *KeServiceDescriptorTable;
 };
 #endif
 
+typedef NTSTATUS (*PsSetCreateProcessNotifyRoutineEx_t)( PCREATE_PROCESS_NOTIFY_ROUTINE_EX NotifyRoutine, BOOLEAN Remove);
+
 //
 // DriverEntry
 //
@@ -76,6 +78,8 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
     BOOLEAN bExit = FALSE;
     UNICODE_STRING uniRoutine;
     RTL_OSVERSIONINFOW sysVersion;
+    UNICODE_STRING uniPsSetCreateProcessNotifyRoutineEx;
+    PsSetCreateProcessNotifyRoutineEx_t pPsSetCreateProcessNotifyRoutineEx = NULL;
 
     __try
     {
@@ -449,11 +453,36 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
         AFSInitServerStrings();
 
         //
-        // Register the call back for process creation and tear down
+        // Register the call back for process creation and tear down.
+        // On Vista SP1 and above, PsSetCreateProcessNotifyRoutineEx
+        // will be used.  This function returns STATUS_ACCESS_DENIED
+        // if there is a signing error.  In that case, the AFSProcessNotifyEx
+        // routine has not been registered and we can fallback to the
+        // Windows 2000 interface and AFSProcessNotify.
         //
 
-        PsSetCreateProcessNotifyRoutine( AFSProcessNotify,
-                                         FALSE);
+        RtlInitUnicodeString( &uniPsSetCreateProcessNotifyRoutineEx,
+                              L"PsSetCreateProcessNotifyRoutineEx");
+
+        pPsSetCreateProcessNotifyRoutineEx = (PsSetCreateProcessNotifyRoutineEx_t)MmGetSystemRoutineAddress(&uniPsSetCreateProcessNotifyRoutineEx);
+
+        ntStatus = STATUS_ACCESS_DENIED;
+
+        if ( pPsSetCreateProcessNotifyRoutineEx)
+        {
+
+            ntStatus = pPsSetCreateProcessNotifyRoutineEx( AFSProcessNotifyEx,
+                                                           FALSE);
+        }
+
+        if ( ntStatus == STATUS_ACCESS_DENIED)
+        {
+
+            ntStatus = PsSetCreateProcessNotifyRoutine( AFSProcessNotify,
+                                                        FALSE);
+        }
+
+        ntStatus = STATUS_SUCCESS;
 
 try_exit:
 
