@@ -44,57 +44,130 @@ AFSProcessNotify( IN HANDLE  ParentId,
                   IN BOOLEAN  Create)
 {
 
+    //
+    // If this is a create notification then update our tree, otherwise remove the
+    // entry
+    //
+
+    if( Create)
+    {
+
+        AFSProcessCreate( ParentId,
+                          ProcessId,
+                          PsGetCurrentProcessId(),
+                          PsGetCurrentThreadId());
+    }
+    else
+    {
+
+        AFSProcessDestroy( ProcessId);
+    }
+
+    return;
+}
+
+void
+AFSProcessNotifyEx( IN OUT PEPROCESS Process,
+                    IN     HANDLE ProcessId,
+                    IN OUT PPS_CREATE_NOTIFY_INFO CreateInfo)
+{
+
+    if ( CreateInfo)
+    {
+
+        AFSProcessCreate( CreateInfo->ParentProcessId,
+                          ProcessId,
+                          CreateInfo->CreatingThreadId.UniqueProcess,
+                          CreateInfo->CreatingThreadId.UniqueThread);
+    }
+    else
+    {
+
+        AFSProcessDestroy( ProcessId);
+    }
+}
+
+
+void
+AFSProcessCreate( IN HANDLE ParentId,
+                  IN HANDLE ProcessId,
+                  IN HANDLE CreatingProcessId,
+                  IN HANDLE CreatingThreadId)
+{
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    AFSProcessCB *pProcessCB = NULL, *pParentProcessCB = NULL;
     AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
-    AFSProcessAuthGroupCB *pProcessAuthGroup = NULL, *pLastAuthGroup = NULL;
-    AFSThreadCB *pThreadCB = NULL, *pNextThreadCB = NULL;
+    AFSProcessCB *pProcessCB = NULL;
 
     __Enter
     {
 
-        //
-        // If this is a create notification then update our tree, otherwise remove the
-        // entry
-        //
-
         AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSProcessNotify Acquiring Control ProcessTree.TreeLock lock %08lX EXCL %08lX\n",
+                      "AFSProcessCreate Acquiring Control ProcessTree.TreeLock lock %08lX EXCL %08lX\n",
                       pDeviceExt->Specific.Control.ProcessTree.TreeLock,
                       PsGetCurrentThread());
 
         AFSAcquireExcl( pDeviceExt->Specific.Control.ProcessTree.TreeLock,
                         TRUE);
 
-        if( Create)
+        AFSDbgLogMsg( AFS_SUBSYSTEM_PROCESS_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSProcessCreate Parent %08lX Process %08lX %08lX\n",
+                      ParentId,
+                      ProcessId,
+                      PsGetCurrentThread());
+
+        pProcessCB = AFSInitializeProcessCB( (ULONGLONG)ParentId,
+                                             (ULONGLONG)ProcessId);
+
+        if( pProcessCB != NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_PROCESS_PROCESSING,
-                          AFS_TRACE_LEVEL_VERBOSE,
-                          "AFSProcessNotify CREATE Parent %08lX Process %08lX %08lX\n",
-                          ParentId,
-                          ProcessId,
-                          PsGetCurrentThread());
+            pProcessCB->CreatingProcessId = (ULONGLONG)CreatingProcessId;
 
-            pProcessCB = AFSInitializeProcessCB( (ULONGLONG)ParentId,
-                                                 (ULONGLONG)ProcessId);
-
-            if( pProcessCB != NULL)
-            {
-                pProcessCB->CreatingThread = (ULONGLONG)PsGetCurrentThreadId();
-            }
-
-            try_return( ntStatus);
+            pProcessCB->CreatingThreadId = (ULONGLONG)CreatingThreadId;
         }
 
+        //
+        // Now assign the AuthGroup ACE
+        //
+
+        AFSValidateProcessEntry( ProcessId);
+
+        AFSReleaseResource( pDeviceExt->Specific.Control.ProcessTree.TreeLock);
+    }
+
+    return;
+}
+
+void
+AFSProcessDestroy( IN HANDLE ProcessId)
+{
+
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
+    AFSProcessCB *pProcessCB = NULL, *pParentProcessCB = NULL;
+    AFSProcessAuthGroupCB *pProcessAuthGroup = NULL, *pLastAuthGroup = NULL;
+    AFSThreadCB *pThreadCB = NULL, *pNextThreadCB = NULL;
+
+    __Enter
+    {
+
+        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSProcessDestroy Acquiring Control ProcessTree.TreeLock lock %08lX EXCL %08lX\n",
+                      pDeviceExt->Specific.Control.ProcessTree.TreeLock,
+                      PsGetCurrentThreadId());
+
+        AFSAcquireExcl( pDeviceExt->Specific.Control.ProcessTree.TreeLock,
+                        TRUE);
         //
         // It's a remove so pull the entry
         //
 
         AFSDbgLogMsg( AFS_SUBSYSTEM_PROCESS_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSProcessNotify DESTROY Process %08lX %08lX\n",
+                      "AFSProcessDestroy Process %08lX %08lX\n",
                       ProcessId,
                       PsGetCurrentThread());
 
@@ -141,16 +214,13 @@ AFSProcessNotify( IN HANDLE  ParentId,
         {
             AFSDbgLogMsg( AFS_SUBSYSTEM_PROCESS_PROCESSING,
                           AFS_TRACE_LEVEL_WARNING,
-                          "AFSProcessNotify Process %08lX not found in ProcessTree Status %08lX %08lX\n",
+                          "AFSProcessDestroy Process %08lX not found in ProcessTree Status %08lX %08lX\n",
                           ProcessId,
                           ntStatus,
                           PsGetCurrentThread());
         }
 
-try_exit:
-
         AFSReleaseResource( pDeviceExt->Specific.Control.ProcessTree.TreeLock);
-
     }
 
     return;
@@ -163,14 +233,14 @@ try_exit:
 //
 
 GUID *
-AFSValidateProcessEntry( void)
+AFSValidateProcessEntry( IN HANDLE ProcessId)
 {
 
     GUID *pAuthGroup = NULL;
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSProcessCB *pProcessCB = NULL, *pParentProcessCB = NULL;
     AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
-    ULONGLONG ullProcessID = (ULONGLONG)PsGetCurrentProcessId();
+    ULONGLONG ullProcessID = (ULONGLONG)ProcessId;
     UNICODE_STRING uniSIDString;
     ULONG ulSIDHash = 0;
     AFSSIDEntryCB *pSIDEntryCB = NULL;
@@ -203,9 +273,19 @@ AFSValidateProcessEntry( void)
                       ullProcessID);
 
         ntStatus = AFSLocateHashEntry( pDeviceExt->Specific.Control.ProcessTree.TreeHead,
-                                       (ULONGLONG)ullProcessID,
+                                       ullProcessID,
                                        (AFSBTreeEntry **)&pProcessCB);
 
+        if( !NT_SUCCESS( ntStatus) ||
+            pProcessCB == NULL)
+        {
+
+            AFSProcessCreate( 0,
+                              ProcessId,
+                              0,
+                              0);
+        }
+
         if( !NT_SUCCESS( ntStatus) ||
             pProcessCB == NULL)
         {
@@ -216,8 +296,8 @@ AFSValidateProcessEntry( void)
                           __FUNCTION__,
                           ullProcessID);
 
-            ASSERT( FALSE);
             AFSReleaseResource( pDeviceExt->Specific.Control.ProcessTree.TreeLock);
+
             try_return( ntStatus = STATUS_UNSUCCESSFUL);
         }
 
@@ -354,7 +434,7 @@ AFSValidateProcessEntry( void)
                       pParentThreadCB = pParentThreadCB->Next)
                 {
 
-                    if( pParentThreadCB->ThreadId == pProcessCB->CreatingThread)
+                    if( pParentThreadCB->ThreadId == pProcessCB->CreatingThreadId)
                     {
                         break;
                     }
@@ -378,7 +458,7 @@ AFSValidateProcessEntry( void)
 
                     AFSDbgLogMsg( AFS_SUBSYSTEM_AUTHGROUP_PROCESSING,
                                   AFS_TRACE_LEVEL_VERBOSE,
-                                  "%s PID %08lX Session %08lX inherited Active AuthGroup %wZ from thread %I64X\n",
+                                  "%s PID %I64X Session %08lX inherited Active AuthGroup %wZ from thread %I64X\n",
                                   __FUNCTION__,
                                   ullProcessID,
                                   ulSessionId,
@@ -407,7 +487,7 @@ AFSValidateProcessEntry( void)
 
                     AFSDbgLogMsg( AFS_SUBSYSTEM_AUTHGROUP_PROCESSING,
                                   AFS_TRACE_LEVEL_VERBOSE,
-                                  "%s PID %08lX Session %08lX inherited Active AuthGroup %wZ from parent PID %I64X\n",
+                                  "%s PID %I64X Session %08lX inherited Active AuthGroup %wZ from parent PID %I64X\n",
                                   __FUNCTION__,
                                   ullProcessID,
                                   ulSessionId,
