@@ -204,6 +204,12 @@ AFSControlDeviceCreate( IN PIRP Irp);
 NTSTATUS
 AFSOpenRedirector( IN PIRP Irp);
 
+NTSTATUS
+AFSInitRdrFcb( OUT AFSFcb **RdrFcb);
+
+void
+AFSRemoveRdrFcb( IN OUT AFSFcb **RdrFcb);
+
 //
 // AFSClose.cpp Prototypes
 //
@@ -790,8 +796,22 @@ AFSProcessNotify( IN HANDLE  ParentId,
                   IN HANDLE  ProcessId,
                   IN BOOLEAN  Create);
 
+void
+AFSProcessNotifyEx( IN OUT PEPROCESS Process,
+                    IN     HANDLE ProcessId,
+                    IN OUT PPS_CREATE_NOTIFY_INFO CreateInfo);
+
+void
+AFSProcessCreate( IN HANDLE ParentId,
+                  IN HANDLE ProcessId,
+                  IN HANDLE CreatingProcessId,
+                  IN HANDLE CreatingThreadId);
+
+void
+AFSProcessDestroy( IN HANDLE ProcessId);
+
 GUID *
-AFSValidateProcessEntry( void);
+AFSValidateProcessEntry( IN HANDLE ProcessId);
 
 BOOLEAN
 AFSIs64BitProcess( IN ULONGLONG ProcessId);
