@@ -70,7 +70,7 @@ AFSInitializeWorkerPool()
         pDevExt->Specific.Library.WorkerCount = 0;
 
         KeInitializeEvent( &pDevExt->Specific.Library.WorkerQueueHasItems,
-                           NotificationEvent,
+                           SynchronizationEvent,
                            FALSE);
 
         //
@@ -152,7 +152,7 @@ AFSInitializeWorkerPool()
         pDevExt->Specific.Library.IOWorkerCount = 0;
 
         KeInitializeEvent( &pDevExt->Specific.Library.IOWorkerQueueHasItems,
-                           NotificationEvent,
+                           SynchronizationEvent,
                            FALSE);
 
         //
@@ -265,11 +265,34 @@ AFSRemoveWorkerPool()
     pDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
     //
-    // Loop through the workers shutting them down
+    // Loop through the workers shutting them down in two stages.
+    // First, clear AFS_WORKER_PROCESS_REQUESTS so that workers
+    // stop processing requests.  Second, call AFSShutdownWorkerThread()
+    // to wake the workers and wait for them to exit.
     //
 
     pCurrentWorker = pDevExt->Specific.Library.PoolHead;
 
+    while( index < pDevExt->Specific.Library.WorkerCount)
+    {
+
+        ClearFlag( pCurrentWorker->State, AFS_WORKER_PROCESS_REQUESTS);
+
+        pCurrentWorker = pCurrentWorker->fLink;
+
+        if ( pCurrentWorker == NULL)
+        {
+
+            break;
+        }
+
+        index++;
+    }
+
+    pCurrentWorker = pDevExt->Specific.Library.PoolHead;
+
+    index = 0;
+
     while( index < pDevExt->Specific.Library.WorkerCount)
     {
 
@@ -295,13 +318,36 @@ AFSRemoveWorkerPool()
     ExDeleteResourceLite( &pDevExt->Specific.Library.QueueLock);
 
     //
-    // Loop through the IO workers shutting them down
+    // Loop through the IO workers shutting them down in two stages.
+    // First, clear AFS_WORKER_PROCESS_REQUESTS so that workers
+    // stop processing requests.  Second, call AFSShutdownWorkerThread()
+    // to wake the workers and wait for them to exit.
     //
 
     pCurrentWorker = pDevExt->Specific.Library.IOPoolHead;
 
     index = 0;
 
+    while( index < pDevExt->Specific.Library.IOWorkerCount)
+    {
+
+        ClearFlag( pCurrentWorker->State, AFS_WORKER_PROCESS_REQUESTS);
+
+        pCurrentWorker = pCurrentWorker->fLink;
+
+        if ( pCurrentWorker == NULL)
+        {
+
+            break;
+        }
+
+        index++;
+    }
+
+    pCurrentWorker = pDevExt->Specific.Library.IOPoolHead;
+
+    index = 0;
+
     while( index < pDevExt->Specific.Library.IOWorkerCount)
     {
 
@@ -338,6 +384,7 @@ AFSInitVolumeWorker( IN AFSVolumeCB *VolumeCB)
     HANDLE hThread;
     AFSDeviceExt *pControlDeviceExt = (AFSDeviceExt *)AFSControlDeviceObject->DeviceExtension;
     PKSTART_ROUTINE pStartRoutine = NULL;
+    LONG lCount;
 
     __Enter
     {
@@ -395,7 +442,9 @@ AFSInitVolumeWorker( IN AFSVolumeCB *VolumeCB)
                                               FALSE,
                                               NULL);
 
-            if( InterlockedIncrement( &pControlDeviceExt->Specific.Control.VolumeWorkerThreadCount) > 0)
+            lCount = InterlockedIncrement( &pControlDeviceExt->Specific.Control.VolumeWorkerThreadCount);
+
+            if( lCount > 0)
             {
 
                 KeClearEvent( &pControlDeviceExt->Specific.Control.VolumeWorkerCloseEvent);
@@ -512,7 +561,7 @@ AFSShutdownVolumeWorker( IN AFSVolumeCB *VolumeCB)
 //
 // Description:
 //
-//      This function shusdown a worker thread in the pool
+//      This function shutsdown a worker thread in the pool
 //
 // Return:
 //
@@ -530,12 +579,6 @@ AFSShutdownWorkerThread( IN AFSWorkQueueContext *PoolContext)
         BooleanFlagOn( PoolContext->State, AFS_WORKER_INITIALIZED))
     {
 
-        //
-        // Clear the 'keep processing' flag
-        //
-
-        ClearFlag( PoolContext->State, AFS_WORKER_PROCESS_REQUESTS);
-
         //
         // Wake up the thread if it is a sleep
         //
@@ -581,12 +624,6 @@ AFSShutdownIOWorkerThread( IN AFSWorkQueueContext *PoolContext)
         BooleanFlagOn( PoolContext->State, AFS_WORKER_INITIALIZED))
     {
 
-        //
-        // Clear the 'keep processing' flag
-        //
-
-        ClearFlag( PoolContext->State, AFS_WORKER_PROCESS_REQUESTS);
-
         //
         // Wake up the thread if it is a sleep
         //
@@ -629,8 +666,8 @@ AFSWorkerThread( IN PVOID Context)
     AFSWorkQueueContext *pPoolContext = (AFSWorkQueueContext *)Context;
     AFSWorkItem *pWorkItem;
     BOOLEAN freeWorkItem = TRUE;
-    BOOLEAN exitThread = FALSE;
     AFSDeviceExt *pLibraryDevExt = NULL;
+    LONG lCount;
 
     pLibraryDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
@@ -642,35 +679,45 @@ AFSWorkerThread( IN PVOID Context)
                 0,
                 FALSE);
 
-
     //
     // Indicate we are initialized
     //
 
     SetFlag( pPoolContext->State, AFS_WORKER_INITIALIZED);
 
+    ntStatus = KeWaitForSingleObject( &pLibraryDevExt->Specific.Library.WorkerQueueHasItems,
+                                      Executive,
+                                      KernelMode,
+                                      FALSE,
+                                      NULL);
+
     while( BooleanFlagOn( pPoolContext->State, AFS_WORKER_PROCESS_REQUESTS))
     {
 
-        ntStatus = KeWaitForSingleObject( &pLibraryDevExt->Specific.Library.WorkerQueueHasItems,
-                                          Executive,
-                                          KernelMode,
-                                          FALSE,
-                                          NULL);
-
         if( !NT_SUCCESS( ntStatus))
         {
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
                           "AFSWorkerThread Wait for queue items failed Status %08lX\n", ntStatus);
+
+            ntStatus = STATUS_SUCCESS;
         }
         else
         {
 
             pWorkItem = AFSRemoveWorkItem();
 
-            if( pWorkItem != NULL)
+            if( pWorkItem == NULL)
+            {
+
+                ntStatus = KeWaitForSingleObject( &pLibraryDevExt->Specific.Library.WorkerQueueHasItems,
+                                                  Executive,
+                                                  KernelMode,
+                                                  FALSE,
+                                                  NULL);
+            }
+            else
             {
 
                 freeWorkItem = TRUE;
@@ -697,7 +744,7 @@ AFSWorkerThread( IN PVOID Context)
 
                         ASSERT( pWorkItem->Specific.Fcb.Fcb->OpenReferenceCount != 0);
 
-                        InterlockedDecrement( &pWorkItem->Specific.Fcb.Fcb->OpenReferenceCount);
+                        lCount = InterlockedDecrement( &pWorkItem->Specific.Fcb.Fcb->OpenReferenceCount);
 
                         break;
                     }
@@ -733,6 +780,17 @@ AFSWorkerThread( IN PVOID Context)
                         break;
                     }
 
+                    case AFS_WORK_INVALIDATE_OBJECT:
+                    {
+
+                        AFSPerformObjectInvalidate( pWorkItem->Specific.Invalidate.ObjectInfo,
+                                                    pWorkItem->Specific.Invalidate.InvalidateReason);
+
+                        freeWorkItem = TRUE;
+
+                        break;
+                    }
+
                     case AFS_WORK_START_IOS:
                     {
 
@@ -755,10 +813,20 @@ AFSWorkerThread( IN PVOID Context)
 
                     ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
                 }
+
+                ntStatus = STATUS_SUCCESS;
             }
         }
     } // worker thread loop
 
+    ClearFlag( pPoolContext->State, AFS_WORKER_INITIALIZED);
+
+    // Wake up another worker so they too can exit
+
+    KeSetEvent( &pLibraryDevExt->Specific.Library.WorkerQueueHasItems,
+                0,
+                FALSE);
+
     PsTerminateSystemThread( 0);
 
     return;
@@ -772,7 +840,6 @@ AFSIOWorkerThread( IN PVOID Context)
     AFSWorkQueueContext *pPoolContext = (AFSWorkQueueContext *)Context;
     AFSWorkItem *pWorkItem;
     BOOLEAN freeWorkItem = TRUE;
-    BOOLEAN exitThread = FALSE;
     AFSDeviceExt *pLibraryDevExt = NULL, *pRdrDevExt = NULL;
 
     pLibraryDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
@@ -792,28 +859,39 @@ AFSIOWorkerThread( IN PVOID Context)
 
     SetFlag( pPoolContext->State, AFS_WORKER_INITIALIZED);
 
+    ntStatus = KeWaitForSingleObject( &pLibraryDevExt->Specific.Library.IOWorkerQueueHasItems,
+                                      Executive,
+                                      KernelMode,
+                                      FALSE,
+                                      NULL);
+
     while( BooleanFlagOn( pPoolContext->State, AFS_WORKER_PROCESS_REQUESTS))
     {
 
-        ntStatus = KeWaitForSingleObject( &pLibraryDevExt->Specific.Library.IOWorkerQueueHasItems,
-                                          Executive,
-                                          KernelMode,
-                                          FALSE,
-                                          NULL);
-
         if( !NT_SUCCESS( ntStatus))
         {
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
                           "AFSIOWorkerThread Wait for queue items failed Status %08lX\n", ntStatus);
+
+            ntStatus = STATUS_SUCCESS;
         }
         else
         {
 
             pWorkItem = AFSRemoveIOWorkItem();
 
-            if( pWorkItem != NULL)
+            if( pWorkItem == NULL)
+            {
+
+                ntStatus = KeWaitForSingleObject( &pLibraryDevExt->Specific.Library.IOWorkerQueueHasItems,
+                                                  Executive,
+                                                  KernelMode,
+                                                  FALSE,
+                                                  NULL);
+            }
+            else
             {
 
                 freeWorkItem = TRUE;
@@ -869,10 +947,20 @@ AFSIOWorkerThread( IN PVOID Context)
 
                     ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
                 }
+
+                ntStatus = STATUS_SUCCESS;
             }
         }
     } // worker thread loop
 
+    ClearFlag( pPoolContext->State, AFS_WORKER_INITIALIZED);
+
+    // Wake up another IOWorker so they too can exit
+
+    KeSetEvent( &pLibraryDevExt->Specific.Library.IOWorkerQueueHasItems,
+                0,
+                FALSE);
+
     PsTerminateSystemThread( 0);
 
     return;
@@ -886,7 +974,6 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
     AFSWorkQueueContext *pPoolContext = (AFSWorkQueueContext *)&AFSGlobalRoot->VolumeWorkerContext;
     AFSDeviceExt *pControlDeviceExt = NULL;
     AFSDeviceExt *pRDRDeviceExt = NULL;
-    BOOLEAN exitThread = FALSE;
     LARGE_INTEGER DueTime;
     LONG TimeOut;
     KTIMER Timer;
@@ -895,8 +982,11 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
     AFSDirectoryCB *pCurrentDirEntry = NULL, *pNextDirEntry = NULL;
     BOOLEAN bReleaseVolumeLock = FALSE;
     AFSVolumeCB *pVolumeCB = NULL, *pNextVolume = NULL;
+    AFSFcb *pFcb = NULL;
+    LONG lFileType;
     LARGE_INTEGER liCurrentTime;
     BOOLEAN bVolumeObject = FALSE;
+    LONG lCount;
 
     pControlDeviceExt = (AFSDeviceExt *)AFSControlDeviceObject->DeviceExtension;
 
@@ -968,8 +1058,7 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                 continue;
             }
 
-            if( pVolumeCB->ObjectInfoListHead == NULL &&
-                pVolumeCB != AFSGlobalRoot)
+            if( pVolumeCB->ObjectInfoListHead == NULL)
             {
 
                 AFSReleaseResource( pVolumeCB->VolumeLock);
@@ -1004,7 +1093,7 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                     pVolumeCB->VolumeReferenceCount == 1 &&
                     ( pVolumeCB->RootFcb == NULL ||
                       pVolumeCB->RootFcb->OpenReferenceCount == 0) &&
-                    pVolumeCB->ObjectInformation.ObjectReferenceCount == 0)
+                    pVolumeCB->ObjectInformation.ObjectReferenceCount <= 0)
                 {
 
                     if( pVolumeCB->RootFcb != NULL)
@@ -1086,7 +1175,7 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                         //
 
                         if( BooleanFlagOn( pCurrentObject->Flags, AFS_OBJECT_FLAGS_DELETED) &&
-                            pCurrentObject->ObjectReferenceCount == 0 &&
+                            pCurrentObject->ObjectReferenceCount <= 0 &&
                             ( pCurrentObject->Fcb == NULL ||
                               pCurrentObject->Fcb->OpenReferenceCount == 0) &&
                             pCurrentObject->Specific.Directory.DirectoryNodeListHead == NULL &&
@@ -1095,41 +1184,50 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                             AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
 
+                            //
+                            // Dropping the TreeLock permits the
+                            // pCurrentObject->ObjectReferenceCount to change
+                            //
+
                             if( AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
                                                 FALSE))
                             {
 
-                                if( pCurrentObject->Fcb != NULL)
+                                if ( pCurrentObject->ObjectReferenceCount <= 0)
                                 {
 
-                                    AFSRemoveFcb( pCurrentObject->Fcb);
-                                }
-
-                                if( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB != NULL)
-                                {
-
-                                    if( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb != NULL)
+                                    if( pCurrentObject->Fcb != NULL)
                                     {
 
-                                        AFSRemoveFcb( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
+                                        AFSRemoveFcb( &pCurrentObject->Fcb);
                                     }
 
-                                    AFSDeleteObjectInfo( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
+                                    if( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB != NULL)
+                                    {
 
-                                    ExDeleteResourceLite( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
+                                        if( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb != NULL)
+                                        {
 
-                                    AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
+                                            AFSRemoveFcb( &pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
+                                        }
 
-                                    AFSExFreePool( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB);
+                                        AFSDeleteObjectInfo( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
+
+                                        ExDeleteResourceLite( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
+
+                                        AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
+
+                                        AFSExFreePool( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB);
+                                    }
+
+                                    AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
+                                                  AFS_TRACE_LEVEL_VERBOSE,
+                                                  "AFSPrimaryVolumeWorkerThread Deleting deleted object %08lX\n",
+                                                  pCurrentObject);
+
+                                    AFSDeleteObjectInfo( pCurrentObject);
                                 }
 
-                                AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
-                                              AFS_TRACE_LEVEL_VERBOSE,
-                                              "AFSPrimaryWorker Deleting deleted object %08lX\n",
-                                              pCurrentObject);
-
-                                AFSDeleteObjectInfo( pCurrentObject);
-
                                 AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
 
                                 pCurrentObject = pNextObject;
@@ -1278,29 +1376,26 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                     pCurrentChildObject = pCurrentDirEntry->ObjectInformation;
 
+                                    pFcb = NULL;
+
                                     AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                                   AFS_TRACE_LEVEL_VERBOSE,
-                                                  "AFSPrimaryWorker Deleting DE %wZ Object %08lX\n",
+                                                  "AFSPrimaryVolumeWorkerThread Deleting DE %wZ Object %08lX\n",
                                                   &pCurrentDirEntry->NameInformation.FileName,
                                                   pCurrentChildObject);
 
                                     AFSDeleteDirEntry( pCurrentObject,
                                                        pCurrentDirEntry);
 
-                                    if( pCurrentChildObject->ObjectReferenceCount == 0)
+                                    if( pCurrentChildObject->ObjectReferenceCount <= 0)
                                     {
 
                                         if( pCurrentChildObject->Fcb != NULL)
                                         {
 
-                                            if( pCurrentChildObject->FileType == AFS_FILE_TYPE_FILE)
-                                            {
+                                            pFcb = (AFSFcb *) InterlockedCompareExchangePointer( (PVOID *)&pCurrentChildObject->Fcb, NULL, (PVOID)pCurrentChildObject->Fcb);
 
-                                                AFSCleanupFcb( pCurrentChildObject->Fcb,
-                                                               TRUE);
-                                            }
-
-                                            AFSRemoveFcb( pCurrentChildObject->Fcb);
+                                            lFileType = pCurrentChildObject->FileType;
                                         }
 
                                         if( pCurrentChildObject->FileType == AFS_FILE_TYPE_DIRECTORY &&
@@ -1310,7 +1405,7 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                             if( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb != NULL)
                                             {
 
-                                                AFSRemoveFcb( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
+                                                AFSRemoveFcb( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
                                             }
 
                                             AFSDeleteObjectInfo( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
@@ -1324,13 +1419,41 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                         AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                                       AFS_TRACE_LEVEL_VERBOSE,
-                                                      "AFSPrimaryWorker Deleting object %08lX\n",
+                                                      "AFSPrimaryVolumeWorkerThread Deleting object %08lX\n",
                                                       pCurrentChildObject);
 
                                         AFSDeleteObjectInfo( pCurrentChildObject);
                                     }
 
                                     pCurrentDirEntry = pNextDirEntry;
+
+                                    if ( pFcb != NULL)
+                                    {
+
+                                        if( lFileType == AFS_FILE_TYPE_FILE)
+                                        {
+                                            //
+                                            // We must not hold pVolumeCB->ObjectInfoTree.TreeLock exclusive
+                                            // across an AFSCleanupFcb call since it can deadlock with an
+                                            // invalidation call from the service.
+                                            //
+
+                                            AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
+
+                                            //
+                                            // Dropping the TreeLock permits the
+                                            // pCurrentObject->ObjectReferenceCount to change
+                                            //
+
+                                            AFSCleanupFcb( pFcb,
+                                                           TRUE);
+
+                                            AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
+                                                            TRUE);
+                                        }
+
+                                        AFSRemoveFcb( &pFcb);
+                                    }
                                 }
 
                                 pCurrentObject->Specific.Directory.DirectoryNodeListHead = NULL;
@@ -1347,13 +1470,12 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                 AFSDbgLogMsg( AFS_SUBSYSTEM_DIR_NODE_COUNT,
                                               AFS_TRACE_LEVEL_VERBOSE,
-                                              "AFSPrimaryWorker Reset count to 0 on parent FID %08lX-%08lX-%08lX-%08lX\n",
+                                              "AFSPrimaryVolumeWorkerThread Reset count to 0 on parent FID %08lX-%08lX-%08lX-%08lX\n",
                                               pCurrentObject->FileId.Cell,
                                               pCurrentObject->FileId.Volume,
                                               pCurrentObject->FileId.Vnode,
                                               pCurrentObject->FileId.Unique);
 
-
                                 //
                                 // Clear our enumerated flag on this object so we retrieve info again on next access
                                 //
@@ -1361,6 +1483,8 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                 ClearFlag( pCurrentObject->Flags, AFS_OBJECT_FLAGS_DIRECTORY_ENUMERATED);
 
                                 AFSReleaseResource( pCurrentObject->Specific.Directory.DirectoryNodeHdr.TreeLock);
+
+                                AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
                             }
                             else
                             {
@@ -1371,8 +1495,6 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                 break;
                             }
-
-                            AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
                         }
                         else
                         {
@@ -1415,26 +1537,38 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                     {
 
                         if( BooleanFlagOn( pCurrentObject->Flags, AFS_OBJECT_FLAGS_DELETED) &&
-                            pCurrentObject->ObjectReferenceCount == 0 &&
+                            pCurrentObject->ObjectReferenceCount <= 0 &&
                             ( pCurrentObject->Fcb == NULL ||
                               pCurrentObject->Fcb->OpenReferenceCount == 0))
                         {
 
+                            pFcb = (AFSFcb *) InterlockedCompareExchangePointer( (PVOID *)&pCurrentObject->Fcb, NULL, (PVOID)pCurrentObject->Fcb);
+
+                            if( pFcb != NULL)
+                            {
+
+                                AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
+
+                                //
+                                // Dropping the TreeLock permits the
+                                // pCurrentObject->ObjectReferenceCount to change
+                                //
+
+                                AFSCleanupFcb( pFcb,
+                                               TRUE);
+
+                                AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
+                                                TRUE);
+
+                                AFSRemoveFcb( &pFcb);
+                            }
+
                             AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
 
                             if( AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
                                                 FALSE))
                             {
 
-                                if( pCurrentObject->Fcb != NULL)
-                                {
-
-                                    AFSCleanupFcb( pCurrentObject->Fcb,
-                                                   TRUE);
-
-                                    AFSRemoveFcb( pCurrentObject->Fcb);
-                                }
-
                                 AFSDeleteObjectInfo( pCurrentObject);
 
                                 AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
@@ -1456,26 +1590,16 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                             AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
 
-                            if( AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
-                                                FALSE))
-                            {
+                            //
+                            // Dropping the TreeLock permits the
+                            // pCurrentObject->ObjectReferenceCount to change
+                            //
 
-                                AFSCleanupFcb( pCurrentObject->Fcb,
-                                               FALSE);
+                            AFSCleanupFcb( pCurrentObject->Fcb,
+                                           FALSE);
 
-                                AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
-
-                                pCurrentObject = pNextObject;
-
-                                continue;
-                            }
-                            else
-                            {
-
-                                bReleaseVolumeLock = FALSE;
-
-                                break;
-                            }
+                            AFSAcquireShared( pVolumeCB->ObjectInfoTree.TreeLock,
+                                              TRUE);
                         }
                     }
 
@@ -1511,7 +1635,9 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                   AFS_TRACE_LEVEL_VERBOSE,
                   "AFSPrimaryVolumeWorkerThread Exiting\n");
 
-    if( InterlockedDecrement( &pControlDeviceExt->Specific.Control.VolumeWorkerThreadCount) == 0)
+    lCount = InterlockedDecrement( &pControlDeviceExt->Specific.Control.VolumeWorkerThreadCount);
+
+    if( lCount == 0)
     {
 
         KeSetEvent( &pControlDeviceExt->Specific.Control.VolumeWorkerCloseEvent,
@@ -1537,6 +1663,7 @@ AFSVolumeWorkerThread( IN PVOID Context)
     LARGE_INTEGER DueTime;
     LONG TimeOut;
     KTIMER Timer;
+    LONG lCount;
 
     pControlDeviceExt = (AFSDeviceExt *)AFSControlDeviceObject->DeviceExtension;
 
@@ -1605,7 +1732,9 @@ AFSVolumeWorkerThread( IN PVOID Context)
 
     KeCancelTimer( &Timer);
 
-    if( InterlockedDecrement( &pControlDeviceExt->Specific.Control.VolumeWorkerThreadCount) == 0)
+    lCount = InterlockedDecrement( &pControlDeviceExt->Specific.Control.VolumeWorkerThreadCount);
+
+    if( lCount == 0)
     {
 
         KeSetEvent( &pControlDeviceExt->Specific.Control.VolumeWorkerCloseEvent,
@@ -1624,6 +1753,7 @@ AFSInsertWorkitem( IN AFSWorkItem *WorkItem)
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt *pDevExt = NULL;
+    LONG lCount;
 
     pDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
@@ -1636,11 +1766,13 @@ AFSInsertWorkitem( IN AFSWorkItem *WorkItem)
     AFSAcquireExcl( &pDevExt->Specific.Library.QueueLock,
                     TRUE);
 
+    lCount = InterlockedIncrement( &pDevExt->Specific.Library.QueueItemCount);
+
     AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
                   "AFSInsertWorkitem Inserting work item %08lX Count %08lX\n",
                   WorkItem,
-                  InterlockedIncrement( &pDevExt->Specific.Library.QueueItemCount));
+                  lCount);
 
     if( pDevExt->Specific.Library.QueueTail != NULL) // queue already has nodes
     {
@@ -1672,6 +1804,7 @@ AFSInsertIOWorkitem( IN AFSWorkItem *WorkItem)
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt *pDevExt = NULL;
+    LONG lCount;
 
     pDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
@@ -1684,11 +1817,13 @@ AFSInsertIOWorkitem( IN AFSWorkItem *WorkItem)
     AFSAcquireExcl( &pDevExt->Specific.Library.IOQueueLock,
                     TRUE);
 
+    lCount = InterlockedIncrement( &pDevExt->Specific.Library.IOQueueItemCount);
+
     AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
                   "AFSInsertWorkitem Inserting IO work item %08lX Count %08lX\n",
                   WorkItem,
-                  InterlockedIncrement( &pDevExt->Specific.Library.IOQueueItemCount));
+                  lCount);
 
     if( pDevExt->Specific.Library.IOQueueTail != NULL) // queue already has nodes
     {
@@ -1720,6 +1855,7 @@ AFSInsertWorkitemAtHead( IN AFSWorkItem *WorkItem)
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt *pDevExt = NULL;
+    LONG lCount;
 
     pDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
@@ -1736,11 +1872,13 @@ AFSInsertWorkitemAtHead( IN AFSWorkItem *WorkItem)
 
     pDevExt->Specific.Library.QueueHead = WorkItem;
 
+    lCount = InterlockedIncrement( &pDevExt->Specific.Library.QueueItemCount);
+
     AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
                   "AFSInsertWorkitemAtHead Inserting work item %08lX Count %08lX\n",
                   WorkItem,
-                  InterlockedIncrement( &pDevExt->Specific.Library.QueueItemCount));
+                  lCount);
 
     //
     // indicate that the queue has nodes
@@ -1762,6 +1900,7 @@ AFSRemoveWorkItem()
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSWorkItem        *pWorkItem = NULL;
     AFSDeviceExt *pDevExt = NULL;
+    LONG lCount;
 
     pDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
@@ -1779,11 +1918,13 @@ AFSRemoveWorkItem()
 
         pWorkItem = pDevExt->Specific.Library.QueueHead;
 
+        lCount = InterlockedDecrement( &pDevExt->Specific.Library.QueueItemCount);
+
         AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
                       "AFSRemoveWorkItem Removing work item %08lX Count %08lX Thread %08lX\n",
                       pWorkItem,
-                      InterlockedDecrement( &pDevExt->Specific.Library.QueueItemCount),
+                      lCount,
                       PsGetCurrentThreadId());
 
         pDevExt->Specific.Library.QueueHead = pDevExt->Specific.Library.QueueHead->next;
@@ -1792,13 +1933,18 @@ AFSRemoveWorkItem()
         {
 
             pDevExt->Specific.Library.QueueTail = NULL;
-
-            KeResetEvent(&(pDevExt->Specific.Library.WorkerQueueHasItems));
         }
-    }
-    else
-    {
-        KeResetEvent(&(pDevExt->Specific.Library.WorkerQueueHasItems));
+        else
+        {
+
+            //
+            // Wake up another worker
+            //
+
+            KeSetEvent( &(pDevExt->Specific.Library.WorkerQueueHasItems),
+                        0,
+                        FALSE);
+        }
     }
 
     AFSReleaseResource( &pDevExt->Specific.Library.QueueLock);
@@ -1813,6 +1959,7 @@ AFSRemoveIOWorkItem()
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSWorkItem        *pWorkItem = NULL;
     AFSDeviceExt *pDevExt = NULL;
+    LONG lCount;
 
     pDevExt = (AFSDeviceExt *)AFSLibraryDeviceObject->DeviceExtension;
 
@@ -1830,11 +1977,13 @@ AFSRemoveIOWorkItem()
 
         pWorkItem = pDevExt->Specific.Library.IOQueueHead;
 
+        lCount = InterlockedDecrement( &pDevExt->Specific.Library.IOQueueItemCount);
+
         AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
                       "AFSRemoveWorkItem Removing work item %08lX Count %08lX Thread %08lX\n",
                       pWorkItem,
-                      InterlockedDecrement( &pDevExt->Specific.Library.IOQueueItemCount),
+                      lCount,
                       PsGetCurrentThreadId());
 
         pDevExt->Specific.Library.IOQueueHead = pDevExt->Specific.Library.IOQueueHead->next;
@@ -1843,13 +1992,18 @@ AFSRemoveIOWorkItem()
         {
 
             pDevExt->Specific.Library.IOQueueTail = NULL;
-
-            KeResetEvent(&(pDevExt->Specific.Library.IOWorkerQueueHasItems));
         }
-    }
-    else
-    {
-        KeResetEvent(&(pDevExt->Specific.Library.IOWorkerQueueHasItems));
+        else
+        {
+
+            //
+            // Wake up another worker
+            //
+
+            KeSetEvent( &(pDevExt->Specific.Library.IOWorkerQueueHasItems),
+                        0,
+                        FALSE);
+        }
     }
 
     AFSReleaseResource( &pDevExt->Specific.Library.IOQueueLock);
@@ -1958,6 +2112,7 @@ AFSQueueFlushExtents( IN AFSFcb *Fcb,
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt *pRDRDeviceExt = (AFSDeviceExt *)AFSRDRDeviceObject->DeviceExtension;
     AFSWorkItem *pWorkItem = NULL;
+    LONG lCount;
 
     __try
     {
@@ -1975,7 +2130,9 @@ AFSQueueFlushExtents( IN AFSFcb *Fcb,
         // queue down. We'll decrement it just below.
         //
 
-        if( InterlockedIncrement( &Fcb->Specific.File.QueuedFlushCount) > 3)
+        lCount = InterlockedIncrement( &Fcb->Specific.File.QueuedFlushCount);
+
+        if( lCount > 3)
         {
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
@@ -2032,13 +2189,13 @@ AFSQueueFlushExtents( IN AFSFcb *Fcb,
 
         pWorkItem->Specific.Fcb.Fcb = Fcb;
 
-        InterlockedIncrement( &Fcb->OpenReferenceCount);
+        lCount = InterlockedIncrement( &Fcb->OpenReferenceCount);
 
         AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                       AFS_TRACE_LEVEL_VERBOSE,
                       "AFSQueueFlushExtents Increment count on Fcb %08lX Cnt %d\n",
-                                                    Fcb,
-                                                    Fcb->OpenReferenceCount);
+                      Fcb,
+                      lCount);
 
         AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
@@ -2066,7 +2223,9 @@ try_exit:
         // Remove the count we added above
         //
 
-        if( InterlockedDecrement( &Fcb->Specific.File.QueuedFlushCount) == 0)
+        lCount = InterlockedDecrement( &Fcb->Specific.File.QueuedFlushCount);
+
+        if( lCount == 0)
         {
 
             KeSetEvent( &Fcb->NPFcb->Specific.File.QueuedFlushEvent,
@@ -2080,7 +2239,7 @@ try_exit:
             if( pWorkItem != NULL)
             {
 
-                InterlockedDecrement( &Fcb->OpenReferenceCount);
+                lCount = InterlockedDecrement( &Fcb->OpenReferenceCount);
 
                 ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
             }
@@ -2442,3 +2601,77 @@ try_exit:
 
     return ntStatus;
 }
+
+NTSTATUS
+AFSQueueInvalidateObject( IN AFSObjectInfoCB *ObjectInfo,
+                          IN ULONG InvalidateReason)
+{
+
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+    AFSWorkItem *pWorkItem = NULL;
+
+    __try
+    {
+
+        pWorkItem = (AFSWorkItem *) AFSLibExAllocatePoolWithTag( NonPagedPool,
+                                                                 sizeof(AFSWorkItem),
+                                                                 AFS_WORK_ITEM_TAG);
+        if (NULL == pWorkItem)
+        {
+
+            AFSDbgLogMsg( AFS_SUBSYSTEM_IO_PROCESSING,
+                          AFS_TRACE_LEVEL_ERROR,
+                          "AFSQueueInvalidateObject Failed to allocate work item\n");
+
+            try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES );
+        }
+
+        RtlZeroMemory( pWorkItem,
+                       sizeof(AFSWorkItem));
+
+        pWorkItem->Size = sizeof( AFSWorkItem);
+
+        pWorkItem->RequestType = AFS_WORK_INVALIDATE_OBJECT;
+
+        pWorkItem->Specific.Invalidate.ObjectInfo = ObjectInfo;
+
+        pWorkItem->Specific.Invalidate.InvalidateReason = InvalidateReason;
+
+        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSQueueInvalidateObject Workitem %08lX\n",
+                      pWorkItem);
+
+        ntStatus = AFSQueueWorkerRequest( pWorkItem);
+
+try_exit:
+
+        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSQueueInvalidateObject Request complete Status %08lX\n",
+                      ntStatus);
+
+        if( !NT_SUCCESS( ntStatus))
+        {
+
+            if( pWorkItem != NULL)
+            {
+                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
+            }
+
+            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                          AFS_TRACE_LEVEL_ERROR,
+                          "AFSQueueInvalidateObject Failed to queue request Status %08lX\n",
+                          ntStatus);
+        }
+    }
+    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    {
+
+        AFSDbgLogMsg( 0,
+                      0,
+                      "EXCEPTION - AFSQueueInvalidateObject\n");
+    }
+
+    return ntStatus;
+}
