@@ -117,6 +117,7 @@ AFSCommonWrite( IN PDEVICE_OBJECT DeviceObject,
     HANDLE             hCallingUser = OnBehalfOf;
     ULONG              ulExtensionLength = 0;
     BOOLEAN            bRetry = FALSE;
+    ULONGLONG          ullProcessId = (ULONGLONG)PsGetCurrentProcessId();
 
     pIrpSp = IoGetCurrentIrpStackLocation( Irp);
 
@@ -397,13 +398,13 @@ AFSCommonWrite( IN PDEVICE_OBJECT DeviceObject,
 
         if( !bPagingIo &&
             ( pFcb->Specific.File.ExtentRequestProcessId == 0 ||
-              ( PsGetCurrentProcessId() != AFSSysProcess &&
-                pFcb->Specific.File.ExtentRequestProcessId != (ULONGLONG)PsGetCurrentProcessId())))
+              ( ullProcessId != (ULONGLONG)AFSSysProcess &&
+                pFcb->Specific.File.ExtentRequestProcessId != ullProcessId)))
         {
 
-            pFcb->Specific.File.ExtentRequestProcessId = (ULONGLONG)PsGetCurrentProcessId();
+            pFcb->Specific.File.ExtentRequestProcessId = ullProcessId;
 
-            if( pFcb->Specific.File.ExtentRequestProcessId == (ULONGLONG)AFSSysProcess)
+            if( ullProcessId == (ULONGLONG)AFSSysProcess)
             {
                 AFSDbgLogMsg( AFS_SUBSYSTEM_EXTENT_PROCESSING,
                               AFS_TRACE_LEVEL_WARNING,
@@ -411,8 +412,6 @@ AFSCommonWrite( IN PDEVICE_OBJECT DeviceObject,
                               __FUNCTION__,
                               pFcb);
             }
-
-            pFcb->NPFcb->Specific.File.ExtentsRequestStatus = STATUS_SUCCESS;
         }
 
         //
@@ -837,6 +836,7 @@ AFSNonCachedWrite( IN PDEVICE_OBJECT DeviceObject,
     LARGE_INTEGER      liCurrentTime, liLastRequestTime;
     AFSDeviceExt      *pControlDevExt = (AFSDeviceExt *)AFSControlDeviceObject->DeviceExtension;
     PFILE_OBJECT       pCacheFileObject = NULL;
+    BOOLEAN            bDerefExtents = FALSE;
 
     __Enter
     {
@@ -941,7 +941,7 @@ AFSNonCachedWrite( IN PDEVICE_OBJECT DeviceObject,
             bLocked= FALSE;
 
             //
-            // We will re-request the extents after 10 seconds of waiting for them
+            // We will re-request the extents after waiting for them
             //
 
             KeQueryTickCount( &liCurrentTime);
@@ -994,7 +994,7 @@ AFSNonCachedWrite( IN PDEVICE_OBJECT DeviceObject,
             // Wait for it
             //
 
-            ntStatus =  AFSWaitForExtentMapping ( pFcb );
+            ntStatus =  AFSWaitForExtentMapping ( pFcb, pCcb);
 
             if (!NT_SUCCESS(ntStatus))
             {
@@ -1165,6 +1165,8 @@ AFSNonCachedWrite( IN PDEVICE_OBJECT DeviceObject,
         AFSReferenceActiveExtents( pStartExtent,
                                    extentsCount);
 
+        bDerefExtents = TRUE;
+
         AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
                       "AFSNonCachedWrite Releasing(2) Fcb extents lock %08lX SHARED %08lX\n",
@@ -1308,32 +1310,35 @@ AFSNonCachedWrite( IN PDEVICE_OBJECT DeviceObject,
             try_return( ntStatus);
         }
 
-        //
-        // Since this is dirty we can mark the extents dirty now.
-        // AFSMarkDirty will dereference the extents.  Do not call
-        // AFSDereferenceActiveExtents() in this code path.
-        //
-
-        AFSMarkDirty( pFcb,
-                      pStartExtent,
-                      extentsCount,
-                      &StartingByte);
-
-        if (!bPagingIo)
-        {
-            //
-            // This was an uncached user write - tell the server to do
-            // the flush when the worker thread next wakes up
-            //
-            pFcb->Specific.File.LastServerFlush.QuadPart = 0;
-        }
-
-        //
-        // All done
-        //
-
 try_exit:
 
+        if( NT_SUCCESS( ntStatus) &&
+            pStartExtent != NULL &&
+            Irp->IoStatus.Information > 0)
+        {
+
+            //
+            // Since this is dirty we can mark the extents dirty now.
+            // AFSMarkDirty will dereference the extents.  Do not call
+            // AFSDereferenceActiveExtents() in this code path.
+            //
+
+            AFSMarkDirty( pFcb,
+                          pStartExtent,
+                          extentsCount,
+                          &StartingByte,
+                          bDerefExtents);
+
+            if (!bPagingIo)
+            {
+                //
+                // This was an uncached user write - tell the server to do
+                // the flush when the worker thread next wakes up
+                //
+                pFcb->Specific.File.LastServerFlush.QuadPart = 0;
+            }
+        }
+
         if( pCacheFileObject != NULL)
         {
             AFSReleaseCacheFileObject( pCacheFileObject);
@@ -1652,6 +1657,15 @@ AFSExtendingWrite( IN AFSFcb *Fcb,
     NTSTATUS      ntStatus = STATUS_SUCCESS;
     AFSCcb       *pCcb = (AFSCcb *)FileObject->FsContext2;
 
+    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                  AFS_TRACE_LEVEL_VERBOSE,
+                  "AFSExtendingWrite Acquiring Fcb PagingIo lock %08lX EXCL %08lX\n",
+                  &Fcb->NPFcb->PagingResource,
+                  PsGetCurrentThread());
+
+    AFSAcquireExcl( &Fcb->NPFcb->PagingResource,
+                    TRUE);
+
     if( NewLength > Fcb->Header.AllocationSize.QuadPart)
     {
 
@@ -1676,12 +1690,6 @@ AFSExtendingWrite( IN AFSFcb *Fcb,
                                          Fcb->ObjectInformation,
                                          &pCcb->AuthGroup);
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
-                  AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSExtendingWrite Acquiring Fcb lock %08lX EXCL %08lX\n",
-                  &Fcb->NPFcb->Resource,
-                  PsGetCurrentThread());
-
     if (NT_SUCCESS(ntStatus))
     {
 
@@ -1705,6 +1713,8 @@ AFSExtendingWrite( IN AFSFcb *Fcb,
         Fcb->Header.AllocationSize = liSaveAllocation;
     }
 
+    AFSReleaseResource( &Fcb->NPFcb->PagingResource);
+
     //
     // DownConvert file resource to shared
     //
