@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -216,7 +216,7 @@ DWORD NextSearch (int &ii)
 BOOL Help_FindCommand_OnOK (HWND hDlg)
 {
    HWND hCombo = GetDlgItem (hDlg, IDC_FIND_COMMAND);
-   int iiDisplay = -1;
+   size_t iiDisplay = -1;
 
    UNIXUTIL uu = uuUNSPECIFIED;
    TCHAR szText[ cchRESOURCE ];
@@ -363,7 +363,7 @@ void Help_FindError_Shrink (HWND hDlg, BOOL fShrink)
       GetRectInParent (GetDlgItem (hDlg, IDCANCEL), &rClose);
 
       cyShrunk = rClose.top - rAdvanced.top - 9;
-      
+
       ShowWindow (GetDlgItem (hDlg, IDC_ERROR_DESC), SW_HIDE);
 
       SetWindowPos (GetDlgItem (hDlg, IDCANCEL), NULL,
@@ -410,7 +410,7 @@ void Help_FindError_Shrink (HWND hDlg, BOOL fShrink)
 BOOL CALLBACK Help_About_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp);
 void Help_About_OnInitDialog (HWND hDlg);
 void Help_About_OnSysCommand (HWND hDlg, int &cmd);
-LONG procAbout;
+UINT_PTR procAbout;
 
 void Help_About (void)
 {
@@ -465,15 +465,15 @@ BOOL CALLBACK Help_About_Proc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
          SetTimer (GetParent(hDlg), 1000, 1000/8, NULL);
          }
       }
-   return CallWindowProc ((WNDPROC)procAbout, hDlg, msg, wp, lp);
+   return CallWindowProc ((WNDPROC)procAbout, hDlg, msg, wp, lp)?TRUE:FALSE;
 }
 
 
 void Help_About_OnInitDialog (HWND hDlg)
 {
    HWND hAbout = GetDlgItem (hDlg, IDOK);
-   procAbout = (LONG)GetWindowLong (hAbout, GWL_WNDPROC);
-   SetWindowLong (hAbout, GWL_WNDPROC, (LONG)Help_About_Proc);
+   procAbout = (UINT_PTR)GetWindowLongPtr (hAbout, GWLP_WNDPROC);
+   SetWindowLongPtr (hAbout, GWLP_WNDPROC, (LONG_PTR)Help_About_Proc);
 
    LPTSTR pszText = FormatString (IDS_HELPABOUT_DESC1);
    SetDlgItemText (hDlg, IDC_HELPABOUT_DESC, pszText);
