@@ -176,11 +176,14 @@ void lock_ObtainWrite(osi_rwlock_t *lockp)
 
     if (lockp->flags & OSI_LOCKFLAG_EXCL) {
         osi_assertx(lockp->tid[0] != tid, "OSI_RWLOCK_WRITEHELD");
-    } else {
+    }
+#ifdef DEBUG
+    else {
         for ( i=0; i < lockp->readers && i < OSI_RWLOCK_THREADS; i++ ) {
             osi_assertx(lockp->tid[i] != tid, "OSI_RWLOCK_READHELD");
         }
     }
+#endif
 
     /* here we have the fast lock, so see if we can obtain the real lock */
     if (lockp->waiters > 0 || (lockp->flags & OSI_LOCKFLAG_EXCL) ||
@@ -235,11 +238,14 @@ void lock_ObtainRead(osi_rwlock_t *lockp)
 
     if (lockp->flags & OSI_LOCKFLAG_EXCL) {
         osi_assertx(lockp->tid[0] != tid, "OSI_RWLOCK_WRITEHELD");
-    } else {
+    }
+#ifdef DEBUG
+    else {
         for ( i=0; i < lockp->readers && i < OSI_RWLOCK_THREADS; i++ ) {
             osi_assertx(lockp->tid[i] != tid, "OSI_RWLOCK_READHELD");
         }
     }
+#endif
 
     /* here we have the fast lock, so see if we can obtain the real lock */
     if (lockp->waiters > 0 || (lockp->flags & OSI_LOCKFLAG_EXCL)) {
@@ -250,8 +256,11 @@ void lock_ObtainRead(osi_rwlock_t *lockp)
         osi_assert(!(lockp->flags & OSI_LOCKFLAG_EXCL) && lockp->readers > 0);
     } else {
         /* if we're here, all clear to set the lock */
-        if (++lockp->readers <= OSI_RWLOCK_THREADS)
+        lockp->readers++;
+#ifdef DEBUG
+        if (lockp->readers <= OSI_RWLOCK_THREADS)
             lockp->tid[lockp->readers-1] = tid;
+#endif
     }
     LeaveCriticalSection(csp);
 
@@ -302,17 +311,21 @@ void lock_ReleaseRead(osi_rwlock_t *lockp)
 
     osi_assertx(lockp->readers > 0, "read lock not held");
 
-    for ( i=0; i < lockp->readers; i++) {
+#ifdef DEBUG
+    for ( i=0; i < lockp->readers && i < OSI_RWLOCK_THREADS; i++) {
         if ( lockp->tid[i] == tid ) {
-            for ( ; i < lockp->readers - 1; i++)
+            for ( ; i < (lockp->readers - 1) && i < (OSI_RWLOCK_THREADS - 1); i++)
                 lockp->tid[i] = lockp->tid[i+1];
             lockp->tid[i] = 0;
             break;
         }
     }
+#endif
 
-    /* releasing a read lock can allow readers or writers */
-    if (--(lockp->readers) == 0 && !osi_TEmpty(&lockp->d.turn)) {
+	lockp->readers--;
+
+    /* releasing a read lock can allow writers */
+    if (lockp->readers == 0 && lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, 0, csp);
     }
     else {
@@ -361,11 +374,10 @@ void lock_ReleaseWrite(osi_rwlock_t *lockp)
 
     osi_assertx(lockp->flags & OSI_LOCKFLAG_EXCL, "write lock not held");
     osi_assertx(lockp->tid[0] == thrd_Current(), "write lock not held by current thread");
-
     lockp->tid[0] = 0;
 
     lockp->flags &= ~OSI_LOCKFLAG_EXCL;
-    if (!osi_TEmpty(&lockp->d.turn)) {
+    if (lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, 0, csp);
     }
     else {
@@ -394,11 +406,12 @@ void lock_ConvertWToR(osi_rwlock_t *lockp)
 
     /* convert write lock to read lock */
     lockp->flags &= ~OSI_LOCKFLAG_EXCL;
+    lockp->tid[0] = 0;
     lockp->readers++;
 
     osi_assertx(lockp->readers == 1, "read lock not one");
 
-    if (!osi_TEmpty(&lockp->d.turn)) {
+    if (lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, /* still have readers */ 1, csp);
     }
     else {
@@ -426,14 +439,16 @@ void lock_ConvertRToW(osi_rwlock_t *lockp)
     osi_assertx(!(lockp->flags & OSI_LOCKFLAG_EXCL), "write lock held");
     osi_assertx(lockp->readers > 0, "read lock not held");
 
-    for ( i=0; i < lockp->readers; i++) {
+#ifdef DEBUG
+    for ( i=0; i < lockp->readers && i < OSI_RWLOCK_THREADS; i++) {
         if ( lockp->tid[i] == tid ) {
-            for ( ; i < lockp->readers - 1; i++)
+            for ( ; i < (lockp->readers - 1) && i < (OSI_RWLOCK_THREADS - 1); i++)
                 lockp->tid[i] = lockp->tid[i+1];
             lockp->tid[i] = 0;
             break;
         }
     }
+#endif
 
     if (--(lockp->readers) == 0) {
         /* convert read lock to write lock */
@@ -443,7 +458,7 @@ void lock_ConvertRToW(osi_rwlock_t *lockp)
         osi_assertx(lockp->readers > 0, "read lock underflow");
 
         lockp->waiters++;
-        osi_TWait(&lockp->d.turn, OSI_SLEEPINFO_W4WRITE, &lockp->flags, lockp->tid, csp);
+        osi_TWaitExt(&lockp->d.turn, OSI_SLEEPINFO_W4WRITE, &lockp->flags, lockp->tid, csp, FALSE);
         lockp->waiters--;
         osi_assertx(lockp->waiters >= 0, "waiters underflow");
         osi_assert(lockp->readers == 0 && (lockp->flags & OSI_LOCKFLAG_EXCL));
@@ -541,7 +556,7 @@ void lock_ReleaseMutex(struct osi_mutex *lockp)
 
     lockp->flags &= ~OSI_LOCKFLAG_EXCL;
     lockp->tid = 0;
-    if (!osi_TEmpty(&lockp->d.turn)) {
+    if (lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, 0, csp);
     }
     else {
@@ -584,8 +599,11 @@ int lock_TryRead(struct osi_rwlock *lockp)
     }
     else {
         /* if we're here, all clear to set the lock */
-        if (++(lockp->readers) < OSI_RWLOCK_THREADS)
+        lockp->readers++;
+#ifdef DEBUG
+        if (lockp->readers < OSI_RWLOCK_THREADS)
             lockp->tid[lockp->readers-1] = thrd_Current();
+#endif
         i = 1;
     }
 
@@ -741,19 +759,21 @@ void osi_SleepR(LONG_PTR sleepVal, struct osi_rwlock *lockp)
 
     osi_assertx(lockp->readers > 0, "osi_SleepR: not held");
 
-    for ( i=0; i < lockp->readers; i++) {
+#ifdef DEBUG
+    for ( i=0; i < lockp->readers && i < OSI_RWLOCK_THREADS; i++) {
         if ( lockp->tid[i] == tid ) {
-            for ( ; i < lockp->readers - 1; i++)
+            for ( ; i < (lockp->readers - 1) && i < (OSI_RWLOCK_THREADS - 1); i++)
                 lockp->tid[i] = lockp->tid[i+1];
             lockp->tid[i] = 0;
             break;
         }
     }
+#endif
 
     /* XXX better to get the list of things to wakeup from TSignalForMLs, and
      * then do the wakeup after SleepSpin releases the low-level mutex.
      */
-    if (--(lockp->readers) == 0 && !osi_TEmpty(&lockp->d.turn)) {
+    if (--(lockp->readers) == 0 && lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, 0, NULL);
     }
 
@@ -799,7 +819,7 @@ void osi_SleepW(LONG_PTR sleepVal, struct osi_rwlock *lockp)
 
     lockp->flags &= ~OSI_LOCKFLAG_EXCL;
     lockp->tid[0] = 0;
-    if (!osi_TEmpty(&lockp->d.turn)) {
+    if (lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, 0, NULL);
     }
 
@@ -844,7 +864,7 @@ void osi_SleepM(LONG_PTR sleepVal, struct osi_mutex *lockp)
 
     lockp->flags &= ~OSI_LOCKFLAG_EXCL;
     lockp->tid = 0;
-    if (!osi_TEmpty(&lockp->d.turn)) {
+    if (lockp->waiters) {
         osi_TSignalForMLs(&lockp->d.turn, 0, NULL);
     }
 
