@@ -259,6 +259,11 @@ void osi_Init(void)
 }
 
 void osi_TWait(osi_turnstile_t *turnp, int waitFor, void *patchp, DWORD *tidp, CRITICAL_SECTION *releasep)
+{
+    osi_TWaitExt(turnp, waitFor, patchp, tidp, releasep, TRUE);
+}
+
+void osi_TWaitExt(osi_turnstile_t *turnp, int waitFor, void *patchp, DWORD *tidp, CRITICAL_SECTION *releasep, int prepend)
 {
     osi_sleepInfo_t *sp;
     unsigned int code;
@@ -275,9 +280,10 @@ void osi_TWait(osi_turnstile_t *turnp, int waitFor, void *patchp, DWORD *tidp, C
     sp->waitFor = waitFor;
     sp->value = (LONG_PTR) patchp;
     sp->tidp   = tidp;
-    osi_QAddT((osi_queue_t **) &turnp->firstp, (osi_queue_t **) &turnp->lastp, &sp->q);
-    if (!turnp->lastp)
-	turnp->lastp = sp;
+    if (prepend)
+        osi_QAddH((osi_queue_t **) &turnp->firstp, (osi_queue_t **) &turnp->lastp, &sp->q);
+    else
+        osi_QAddT((osi_queue_t **) &turnp->firstp, (osi_queue_t **) &turnp->lastp, &sp->q);
     LeaveCriticalSection(releasep);
 
     /* now wait for the signal */
@@ -319,7 +325,6 @@ void osi_TSignal(osi_turnstile_t *turnp)
         return;
 
     sp = turnp->lastp;
-    turnp->lastp = (osi_sleepInfo_t *) osi_QPrev(&sp->q);
     osi_QRemoveHT((osi_queue_t **) &turnp->firstp, (osi_queue_t **) &turnp->lastp, &sp->q);
     sp->states |= OSI_SLEEPINFO_SIGNALLED;
     ReleaseSemaphore(sp->sema, 1, (long *) 0);
@@ -331,7 +336,6 @@ void osi_TBroadcast(osi_turnstile_t *turnp)
     osi_sleepInfo_t *sp;
 
     while(sp = turnp->lastp) {
-        turnp->lastp = (osi_sleepInfo_t *) osi_QPrev(&sp->q);
         osi_QRemoveHT((osi_queue_t **) &turnp->firstp, (osi_queue_t **) &turnp->lastp, &sp->q);
         sp->states |= OSI_SLEEPINFO_SIGNALLED;
         ReleaseSemaphore(sp->sema, 1, (long *) 0);
@@ -378,7 +382,6 @@ void osi_TSignalForMLs(osi_turnstile_t *turnp, int stillHaveReaders, CRITICAL_SE
          * and move to private one, so we can do the wakeup after releasing
          * the crit sec.
          */
-        turnp->lastp = (osi_sleepInfo_t *) osi_QPrev(&tsp->q);
         osi_QRemoveHT((osi_queue_t **) &turnp->firstp, (osi_queue_t **) &turnp->lastp, &tsp->q);
 
         /* do the patching required for lock obtaining */
@@ -389,9 +392,11 @@ void osi_TSignalForMLs(osi_turnstile_t *turnp, int stillHaveReaders, CRITICAL_SE
         }
         else if (tsp->waitFor & OSI_SLEEPINFO_W4READ) {
             sp = (void *) tsp->value;
+#ifdef DEBUG
+            if ((*sp) < OSI_RWLOCK_THREADS)
+                tsp->tidp[*sp] = tsp->tid;
+#endif
             (*sp)++;
-            if ((*sp) <= OSI_RWLOCK_THREADS)
-                tsp->tidp[(*sp)-1] = tsp->tid;
         }
 
         /* and add to our own list */
