@@ -28,25 +28,18 @@ int afs_vfs_typenum;
 int
 afs_quotactl()
 {
-    return EOPNOTSUPP;
+    return ENOTSUP;
 }
 
 int
-afs_fhtovp(mp, fhp, vpp)
-     struct mount *mp;
-     struct fid *fhp;
-     struct vnode **vpp;
+afs_fhtovp(struct mount *mp, struct fid *fhp, struct vnode **vpp)
 {
-
     return (EINVAL);
 }
 
 int
-afs_vptofh(vp, fhp)
-     struct vnode *vp;
-     struct fid *fhp;
+afs_vptofh(struct vnode *vp, struct fid *fhp)
 {
-
     return (EINVAL);
 }
 
@@ -63,10 +56,7 @@ afs_vptofh(vp, fhp)
 #define PROC_DECL(out,in) struct proc *out = CTX_PROC_CONVERT(in)
 
 int
-afs_start(mp, flags, p)
-     struct mount *mp;
-     int flags;
-     CTX_TYPE p;
+afs_start(struct mount *mp, int flags, CTX_TYPE p)
 {
     return (0);			/* nothing to do. ? */
 }
@@ -75,19 +65,10 @@ int
 afs_statfs(struct mount *mp, STATFS_TYPE *abp, CTX_TYPE ctx);
 #ifdef AFS_DARWIN80_ENV
 int
-afs_mount(mp, devvp, data, ctx)
-     register struct mount *mp;
-     vnode_t *devvp;
-     user_addr_t data;
-     vfs_context_t ctx;
+afs_mount(struct mount *mp, vnode_t *devvp, user_addr_t data, vfs_context_t ctx)
 #else
 int
-afs_mount(mp, path, data, ndp, ctx)
-     register struct mount *mp;
-     char *path;
-     caddr_t data;
-     struct nameidata *ndp;
-     CTX_TYPE ctx;
+afs_mount(struct mount *mp, char *path, caddr_t data, struct nameidata *ndp, CTX_TYPE ctx)
 #endif
 {
     /* ndp contains the mounted-from device.  Just ignore it.
@@ -189,10 +170,7 @@ afs_mount(mp, path, data, ndp, ctx)
 }
 
 int
-afs_unmount(mp, flags, ctx)
-     struct mount *mp;
-     int flags;
-     CTX_TYPE ctx;
+afs_unmount(struct mount *mp, int flags, CTX_TYPE ctx)
 {
     void *mdata = vfs_fsprivate(mp);
     AFS_GLOCK();
@@ -243,10 +221,11 @@ afs_root(struct mount *mp, struct vnode **vpp)
     void *mdata = vfs_fsprivate(mp);
     int error;
     struct vrequest treq;
-    register struct vcache *tvp = 0;
+    struct vcache *tvp = 0;
+    struct vcache *gvp;
+    int needref=0;
 #ifdef AFS_DARWIN80_ENV
     struct ucred *cr = vfs_context_ucred(ctx);
-    int needref=0;
 #else
     struct proc *p = current_proc();
     struct ucred _cr;
@@ -258,13 +237,13 @@ afs_root(struct mount *mp, struct vnode **vpp)
 #endif
     AFS_GLOCK();
     AFS_STATCNT(afs_root);
+
+again:
     if (mdata == NULL && afs_globalVp
 	&& (afs_globalVp->f.states & CStatd)) {
 	tvp = afs_globalVp;
 	error = 0;
-#ifdef AFS_DARWIN80_ENV
         needref=1;
-#endif
     } else if (mdata == (qaddr_t) - 1) {
 	error = ENOENT;
     } else {
@@ -276,11 +255,11 @@ afs_root(struct mount *mp, struct vnode **vpp)
 #ifdef AFS_DARWIN80_ENV
             if (tvp) {
 	        AFS_GUNLOCK();
-                error = afs_darwin_finalizevnode(tvp, NULL, NULL, 1);
+                error = afs_darwin_finalizevnode(tvp, NULL, NULL, 1, 0);
 	        AFS_GLOCK();
                 if (error)
                    tvp = NULL;
-                else 
+                else
                    /* re-acquire the usecount that finalizevnode disposed of */
                    vnode_ref(AFSTOV(tvp));
             }
@@ -288,36 +267,46 @@ afs_root(struct mount *mp, struct vnode **vpp)
 	    /* we really want this to stay around */
 	    if (tvp) {
 		if (mdata == NULL) {
-		    if (afs_globalVp) {
-			afs_PutVCache(afs_globalVp);
-			afs_globalVp = NULL;
-		    }
+		    gvp = afs_globalVp;
 		    afs_globalVp = tvp;
-#ifdef AFS_DARWIN80_ENV
+		    if (gvp) {
+			afs_PutVCache(gvp);
+			if (tvp != afs_globalVp) {
+			    /* someone else got there before us! */
+			    afs_PutVCache(tvp);
+			    tvp = 0;
+			    goto again;
+			}
+		    }
                     needref=1;
-#endif
                 }
 	    } else
 		error = ENOENT;
 	}
     }
     if (tvp) {
-#ifndef AFS_DARWIN80_ENV /* DARWIN80 caller does not need a usecount reference */
+#ifndef AFS_DARWIN80_ENV /* KPI callers don't need a usecount reference */
 	osi_vnhold(tvp, 0);
 	AFS_GUNLOCK();
 	vn_lock(AFSTOV(tvp), LK_EXCLUSIVE | LK_RETRY, p);
 	AFS_GLOCK();
 #endif
+
+        if (needref)
 #ifdef AFS_DARWIN80_ENV
-        if (needref) /* this iocount is for the caller. the initial iocount
-                        is for the eventual afs_PutVCache. for mdata != null,
-                        there will not be a PutVCache, so the caller gets the
-                        initial (from GetVCache or finalizevnode) iocount*/
-           vnode_get(AFSTOV(tvp));
+	    /* This iocount is for the caller. the initial iocount
+	     * is for the eventual afs_PutVCache. for mdata != null,
+	     * there will not be a PutVCache, so the caller gets the
+	     * initial (from GetVCache or finalizevnode) iocount
+	     */
+	    vnode_get(AFSTOV(tvp));
+#else
+	    ;
 #endif
-	if (mdata == NULL) {
+
+	if (mdata == NULL)
 	    afs_globalVFS = mp;
-	}
+
 	*vpp = AFSTOV(tvp);
 #ifndef AFS_DARWIN80_ENV 
 	AFSTOV(tvp)->v_flag |= VROOT;
@@ -333,10 +322,7 @@ afs_root(struct mount *mp, struct vnode **vpp)
 
 #ifndef AFS_DARWIN80_ENV /* vget vfsop never had this prototype AFAIK */
 int
-afs_vget(mp, lfl, vp)
-     struct mount *mp;
-     struct vnode *vp;
-     int lfl;
+afs_vget(struct mount *mp, int lfl, struct vnode *vp)
 {
     int error;
     //printf("vget called. help!\n");
@@ -420,6 +406,8 @@ afs_vfs_getattr(struct mount *mp, struct vfs_attr *outattrs,
                    VOL_CAP_FMT_ZERO_RUNS |
                    VOL_CAP_FMT_CASE_SENSITIVE |
                    VOL_CAP_FMT_CASE_PRESERVING |
+		   VOL_CAP_FMT_PERSISTENTOBJECTIDS |
+		   VOL_CAP_FMT_2TB_FILESIZE |
                    VOL_CAP_FMT_FAST_STATFS;
          vcapattrptr->capabilities[VOL_CAPABILITIES_INTERFACES] = 
                    VOL_CAP_INT_ADVLOCK | 
@@ -439,6 +427,8 @@ afs_vfs_getattr(struct mount *mp, struct vfs_attr *outattrs,
                  VOL_CAP_FMT_ZERO_RUNS |
                  VOL_CAP_FMT_CASE_SENSITIVE |
                  VOL_CAP_FMT_CASE_PRESERVING |
+		 VOL_CAP_FMT_PERSISTENTOBJECTIDS |
+		 VOL_CAP_FMT_2TB_FILESIZE |
                  VOL_CAP_FMT_FAST_STATFS;
          vcapattrptr->valid[VOL_CAPABILITIES_INTERFACES] =
                  VOL_CAP_INT_SEARCHFS |
@@ -462,34 +452,58 @@ afs_vfs_getattr(struct mount *mp, struct vfs_attr *outattrs,
 
 #ifdef AFS_DARWIN80_ENV
 int
-afs_sync(mp, waitfor, ctx)
-     struct mount *mp;
-     int waitfor;
-     CTX_TYPE ctx;
+afs_sync(struct mount *mp, int waitfor, CTX_TYPE ctx)
 #else
 int
-afs_sync(mp, waitfor, cred, p)
-     struct mount *mp;
-     int waitfor;
-     struct ucred *cred;
-     struct proc *p;
+afs_sync(struct mount *mp, int waitfor, struct ucred *cred, struct proc *p)
 #endif
 {
     return 0;
 }
 
 u_int32_t afs_darwin_realmodes = 0;
+u_int32_t afs_darwin_fsevents = 0;
+extern int AFSDOBULK;
+
+int
+afs_sysctl_int(int *name, u_int namelen, user_addr_t oldp, size_t *oldlenp,
+	       user_addr_t newp, size_t newlen, u_int32_t *object)
+{
+#ifdef AFS_DARWIN80_ENV
+    int error;
+
+    if (oldp != USER_ADDR_NULL && oldlenp == NULL)
+	return (EFAULT);
+    if (oldp && *oldlenp < sizeof(u_int32_t))
+	return (ENOMEM);
+    if (newp && newlen != sizeof(u_int32_t))
+	return (EINVAL);
+    *oldlenp = sizeof(u_int32_t);
+    if (oldp) {
+	if ((error = copyout(object,
+			     oldp, sizeof(u_int32_t)))) {
+	    return error;
+	}
+    }
+    if (newp)
+	return copyin(newp, object, sizeof(u_int32_t));
+    return 0;
+#else
+    return sysctl_int(oldp, oldlenp, newp, newlen,
+		      object);
+#endif
+}
 
 #ifdef AFS_DARWIN80_ENV
-int afs_sysctl(int *name, u_int namelen, user_addr_t oldp, size_t *oldlenp, 
+int
+afs_sysctl(int *name, u_int namelen, user_addr_t oldp, size_t *oldlenp,
 	       user_addr_t newp, size_t newlen, vfs_context_t context)
 #else
-int afs_sysctl(int *name, u_int namelen, void *oldp, size_t *oldlenp, 
+int
+afs_sysctl(int *name, u_int namelen, void *oldp, size_t *oldlenp,
 	       void *newp, size_t newlen, struct proc *p)
 #endif
 {
-    int error;
-
     switch (name[0]) {
     case AFS_SC_ALL:
         /* nothing defined */
@@ -501,35 +515,21 @@ int afs_sysctl(int *name, u_int namelen, void *oldp, size_t *oldlenp,
 	case AFS_SC_DARWIN_ALL:
 	    switch (name[2]) {
 	    case AFS_SC_DARWIN_ALL_REALMODES:
-#ifdef AFS_DARWIN80_ENV
-		if (oldp != USER_ADDR_NULL && oldlenp == NULL)
-		    return (EFAULT);
-		if (oldp && *oldlenp < sizeof(u_int32_t))
-		    return (ENOMEM);
-		if (newp && newlen != sizeof(u_int32_t))
-		    return (EINVAL);
-		*oldlenp = sizeof(u_int32_t);
-		if (oldp) {
-		    if ((error = copyout(&afs_darwin_realmodes,
-					 oldp, sizeof(u_int32_t)))) {
-			return error;
-		    }
-		}
-		if (newp)
-		    return copyin(newp, &afs_darwin_realmodes,
-				  sizeof(u_int32_t));
-		return 0;
-#else
-	        return sysctl_int(oldp, oldlenp, newp, newlen,
-				  &afs_darwin_realmodes);
-#endif
+		return afs_sysctl_int(name, namelen, oldp, oldlenp,
+				      newp, newlen, &afs_darwin_realmodes);
+	    case AFS_SC_DARWIN_ALL_FSEVENTS:
+		return afs_sysctl_int(name, namelen, oldp, oldlenp,
+				      newp, newlen, &afs_darwin_fsevents);
+	    case AFS_SC_DARWIN_ALL_BULKSTAT:
+		return afs_sysctl_int(name, namelen, oldp, oldlenp,
+				      newp, newlen, &AFSDOBULK);
 	    }
 	    break;
 	    /* darwin version specific sysctl's goes here */
 	}
 	break;
     }
-    return EOPNOTSUPP;
+    return ENOTSUP;
 }
 
 typedef (*PFI) ();
