@@ -24,7 +24,7 @@ afs_vrdwr(struct usr_vnode *avc, struct usr_uio *uio, int rw, int io,
     if (rw == UIO_WRITE) {
 	rc = afs_write(VTOAFS(avc), uio, io, cred, 0);
     } else {
-	rc = afs_read(VTOAFS(avc), uio, cred, 0, 0, 0);
+	rc = afs_read(VTOAFS(avc), uio, cred, 0);
     }
 
     return rc;
@@ -33,7 +33,7 @@ afs_vrdwr(struct usr_vnode *avc, struct usr_uio *uio, int rw, int io,
 int
 afs_inactive(struct vcache *avc, afs_ucred_t *acred)
 {
-    if (afs_shuttingdown != AFS_RUNNING)
+    if (afs_shuttingdown)
 	return 0;
 
     usr_assert(avc->vrefCount == 0);
