@@ -40,12 +40,10 @@
 #include "afs/afs_stats.h"	/* afs statistics */
 #include "afs/afs_dynroot.h"
 
-#if	defined(AFS_SUN56_ENV)
+#if	defined(AFS_SUN5_ENV)
 #include <inet/led.h>
 #include <inet/common.h>
-#if     defined(AFS_SUN58_ENV)
 #include <netinet/ip6.h>
-#endif
 #include <inet/ip.h>
 #endif
 
@@ -586,39 +584,27 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 	    return NULL;
 	}
 	memset(tv, 0, sizeof(struct volume));
-
-	for (j = fvTable[FVHash(tcell->cellNum, volid)]; j != 0; j = tf->next) {
+	tv->cell = tcell->cellNum;
+	AFS_RWLOCK_INIT(&tv->lock, "volume lock");
+	tv->next = afs_volumes[i];	/* thread into list */
+	afs_volumes[i] = tv;
+	tv->volume = volid;
+	for (j = fvTable[FVHash(tv->cell, volid)]; j != 0; j = tf->next) {
 	    if (afs_FVIndex != j) {
 		struct osi_file *tfile;
 	        tfile = osi_UFSOpen(&volumeInode);
 		err =
 		    afs_osi_Read(tfile, sizeof(struct fvolume) * j,
 				 &staticFVolume, sizeof(struct fvolume));
+		if (err != sizeof(struct fvolume))
+		    osi_Panic("read volumeinfo2");
 		osi_UFSClose(tfile);
-		if (err != sizeof(struct fvolume)) {
-		    afs_warn("afs_SetupVolume: error %d reading volumeinfo\n",
-		             (int)err);
-		    /* put tv back on the free list; the data in it is not valid */
-		    tv->next = afs_freeVolList;
-		    afs_freeVolList = tv;
-		    /* staticFVolume contents are not valid */
-		    afs_FVIndex = -1;
-		    ReleaseWriteLock(&afs_xvolume);
-		    return NULL;
-		}
 		afs_FVIndex = j;
 	    }
 	    tf = &staticFVolume;
-	    if (tf->cell == tcell->cellNum && tf->volume == volid)
+	    if (tf->cell == tv->cell && tf->volume == volid)
 		break;
 	}
-
-	tv->cell = tcell->cellNum;
-	AFS_RWLOCK_INIT(&tv->lock, "volume lock");
-	tv->next = afs_volumes[i];	/* thread into list */
-	afs_volumes[i] = tv;
-	tv->volume = volid;
-
 	if (tf && (j != 0)) {
 	    tv->vtix = afs_FVIndex;
 	    tv->mtpoint = tf->mtpoint;
@@ -754,7 +740,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     struct cell *tcell;
     char *tbuffer, *ve;
     struct afs_conn *tconn;
-    struct vrequest *treq = NULL;
+    struct vrequest treq;
     struct rx_connection *rxconn;
 
     if (strlen(aname) > VL_MAXNAMELEN)	/* Invalid volume name */
@@ -765,14 +751,9 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	return NULL;
     }
 
-    code = afs_CreateReq(&treq, afs_osi_credp);	/* *must* be unauth for vldb */
-    if (code) {
-	return NULL;
-    }
-
     /* allow null request if we don't care about ENODEV/ETIMEDOUT distinction */
     if (!areq)
-	areq = treq;
+	areq = &treq;
 
 
     afs_Trace2(afs_iclSetp, CM_TRACE_GETVOL, ICL_TYPE_STRING, aname,
@@ -781,18 +762,18 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     tve = (struct vldbentry *)(tbuffer + 1024);
     ntve = (struct nvldbentry *)tve;
     utve = (struct uvldbentry *)tve;
-
+    afs_InitReq(&treq, afs_osi_credp);	/* *must* be unauth for vldb */
     do {
 	tconn =
 	    afs_ConnByMHosts(tcell->cellHosts, tcell->vlport, tcell->cellNum,
-			     treq, SHARED_LOCK, 0, &rxconn);
+			     &treq, SHARED_LOCK, &rxconn);
 	if (tconn) {
-	    if (tconn->srvr->server->flags & SNO_LHOSTS) {
+	    if (tconn->parent->srvr->server->flags & SNO_LHOSTS) {
 		type = 0;
 		RX_AFS_GUNLOCK();
 		code = VL_GetEntryByNameO(rxconn, aname, tve);
 		RX_AFS_GLOCK();
-	    } else if (tconn->srvr->server->flags & SYES_LHOSTS) {
+	    } else if (tconn->parent->srvr->server->flags & SYES_LHOSTS) {
 		type = 1;
 		RX_AFS_GUNLOCK();
 		code = VL_GetEntryByNameN(rxconn, aname, ntve);
@@ -802,7 +783,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 		RX_AFS_GUNLOCK();
 		code = VL_GetEntryByNameU(rxconn, aname, utve);
 		RX_AFS_GLOCK();
-		if (!(tconn->srvr->server->flags & SVLSRV_UUID)) {
+		if (!(tconn->parent->srvr->server->flags & SVLSRV_UUID)) {
 		    if (code == RXGEN_OPCODE) {
 			type = 1;
 			RX_AFS_GUNLOCK();
@@ -810,20 +791,20 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 			RX_AFS_GLOCK();
 			if (code == RXGEN_OPCODE) {
 			    type = 0;
-			    tconn->srvr->server->flags |= SNO_LHOSTS;
+			    tconn->parent->srvr->server->flags |= SNO_LHOSTS;
 			    RX_AFS_GUNLOCK();
 			    code = VL_GetEntryByNameO(rxconn, aname, tve);
 			    RX_AFS_GLOCK();
 			} else if (!code)
-			    tconn->srvr->server->flags |= SYES_LHOSTS;
+			    tconn->parent->srvr->server->flags |= SYES_LHOSTS;
 		    } else if (!code)
-			tconn->srvr->server->flags |= SVLSRV_UUID;
+			tconn->parent->srvr->server->flags |= SVLSRV_UUID;
 		}
 		lastnvcode = code;
 	    }
 	} else
 	    code = -1;
-    } while (afs_Analyze(tconn, rxconn, code, NULL, treq, -1,	/* no op code for this */
+    } while (afs_Analyze(tconn, rxconn, code, NULL, &treq, -1,	/* no op code for this */
 			 SHARED_LOCK, tcell));
 
     if (code) {
@@ -846,10 +827,9 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	    }
 	}
 #endif
-	afs_CopyError(treq, areq);
+	afs_CopyError(&treq, areq);
 	osi_FreeLargeSpace(tbuffer);
 	afs_PutCell(tcell, READ_LOCK);
-	afs_DestroyReq(treq);
 	return NULL;
     }
     /*
@@ -869,14 +849,14 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	ve = (char *)ntve;
     else
 	ve = (char *)tve;
-    tv = afs_SetupVolume(0, aname, ve, tcell, agood, type, treq);
+    tv = afs_SetupVolume(0, aname, ve, tcell, agood, type, &treq);
     if ((agood == 3) && tv && tv->backVol) {
 	/*
 	 * This means that very soon we'll ask for the BK volume so
 	 * we'll prefetch it (well we did already.)
 	 */
 	tv1 =
-	    afs_SetupVolume(tv->backVol, (char *)0, ve, tcell, 0, type, treq);
+	    afs_SetupVolume(tv->backVol, (char *)0, ve, tcell, 0, type, &treq);
 	if (tv1) {
 	    tv1->refCount--;
 	}
@@ -886,14 +866,13 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	 * This means that very soon we'll ask for the RO volume so
 	 * we'll prefetch it (well we did already.)
 	 */
-	tv1 = afs_SetupVolume(tv->roVol, NULL, ve, tcell, 0, type, treq);
+	tv1 = afs_SetupVolume(tv->roVol, NULL, ve, tcell, 0, type, &treq);
 	if (tv1) {
 	    tv1->refCount--;
 	}
     }
     osi_FreeLargeSpace(tbuffer);
     afs_PutCell(tcell, READ_LOCK);
-    afs_DestroyReq(treq);
     return tv;
 
 }				/*afs_NewVolumeByName */
@@ -1135,10 +1114,9 @@ LockAndInstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		    tconn =
 			afs_ConnByMHosts(tcell->cellHosts, tcell->vlport,
 					 tcell->cellNum, areq, SHARED_LOCK,
-					 0, &rxconn);
+					 &rxconn);
 		    if (tconn) {
 			RX_AFS_GUNLOCK();
-			xdr_free((xdrproc_t) xdr_bulkaddrs, &addrs);
 			code =
 			    VL_GetAddrsU(rxconn, &attrs, &uuid, &unique,
 					 &nentries, &addrs);
@@ -1159,10 +1137,6 @@ LockAndInstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		    return;
 		}
 
-		if (addrs.bulkaddrs_len < nentries) {
-		    nentries = addrs.bulkaddrs_len;
-		}
-
 		addrp = addrs.bulkaddrs_val;
 		for (k = 0; k < nentries; k++) {
 		    addrp[k] = htonl(addrp[k]);
@@ -1221,11 +1195,6 @@ afs_ResetVolumeInfo(struct volume *tv)
     AFS_STATCNT(afs_ResetVolumeInfo);
     ObtainWriteLock(&tv->lock, 117);
     tv->states |= VRecheck;
-
-    /* the hard-mount code in afs_Analyze may not be able to reset this flag
-     * when VRecheck is set, so clear it here to ensure it gets cleared. */
-    tv->states &= ~VHardMount;
-
     for (i = 0; i < AFS_MAXHOSTS; i++)
 	tv->status[i] = not_busy;
     if (tv->name) {
