@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,30 +10,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+#include <afs/opr.h>
 
 #include <afs/pthread_glock.h>
 #include <afs/afsutil.h>
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#include <fcntl.h>
-#include <io.h>
-#else
-#include <sys/file.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <rx/rxkad.h>
+
 #include "cellconfig.h"
 #include "keys.h"
 
@@ -42,10 +25,10 @@
 */
 
 static int
-VerifyEntries(register struct afsconf_cell *aci)
+VerifyEntries(struct afsconf_cell *aci)
 {
-    register int i;
-    register struct hostent *th;
+    int i;
+    struct hostent *th;
 
     for (i = 0; i < aci->numServers; i++) {
 	if (aci->hostAddr[i].sin_addr.s_addr == 0) {
@@ -83,7 +66,7 @@ VerifyEntries(register struct afsconf_cell *aci)
    */
 
 int
-afsconf_SetCellInfo(struct afsconf_dir *adir, const char *apath, 
+afsconf_SetCellInfo(struct afsconf_dir *adir, const char *apath,
 		    struct afsconf_cell *acellInfo)
 {
     afs_int32 code;
@@ -93,15 +76,15 @@ afsconf_SetCellInfo(struct afsconf_dir *adir, const char *apath,
 }
 
 int
-afsconf_SetExtendedCellInfo(struct afsconf_dir *adir, 
-			    const char *apath, 
+afsconf_SetExtendedCellInfo(struct afsconf_dir *adir,
+			    const char *apath,
 			    struct afsconf_cell *acellInfo, char clones[])
 {
-    register afs_int32 code;
-    register int fd;
+    afs_int32 code;
+    int fd;
     char tbuffer[1024];
-    register FILE *tf;
-    register afs_int32 i;
+    FILE *tf;
+    afs_int32 i;
 
     LOCK_GLOBAL_MUTEX;
     /* write ThisCell file */
