@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,34 +10,22 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/bucoord/config.c,v 1.8.14.1 2007/01/05 03:32:24 shadow Exp $");
-
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-#include <errno.h>
+#include <roken.h>
 
 #include "bc.h"
 
 struct bc_config *bc_globalConfig;
 
-static
-TrimLine(abuffer, aport)
-     afs_int32 *aport;
-     register char *abuffer;
+#if 0
+static int
+TrimLine(char *abuffer, afs_int32 *aport)
 {
-    register int tc;
+    int tc;
     char garb[100];
 
     *aport = 0;
     sscanf(abuffer, "%s %u", garb, aport);
-    while (tc = *abuffer) {
+    while ((tc = *abuffer)) {
 	if (tc == ' ') {
 	    *abuffer = 0;
 	    return 0;
@@ -46,15 +34,12 @@ TrimLine(abuffer, aport)
     }
     return 0;
 }
+#endif
 
 FILE *
-bc_open(aconfig, aname, aext, amode)
-     register struct bc_config *aconfig;
-     char *amode;
-     char *aext;
-     char *aname;
+bc_open(struct bc_config *aconfig, char *aname, char *aext, char *amode)
 {
-    register FILE *tf;
+    FILE *tf;
     char tpath[256];
 
     strcpy(tpath, aconfig->path);
@@ -66,10 +51,10 @@ bc_open(aconfig, aname, aext, amode)
     return tf;
 }
 
-bc_InitConfig(apath)
-     char *apath;
+int
+bc_InitConfig(char *apath)
 {
-    register struct bc_config *tb;
+    struct bc_config *tb;
 
     /* initialize global config structure */
     tb = (struct bc_config *)malloc(sizeof(struct bc_config));
@@ -91,13 +76,10 @@ bc_InitConfig(apath)
     return 0;
 }
 
-static
-HostAdd(alist, aname, aport)
-     struct bc_hostEntry **alist;
-     afs_int32 aport;
-     char *aname;
+static int
+HostAdd(struct bc_hostEntry **alist, char *aname, afs_int32 aport)
 {
-    register struct bc_hostEntry **tlast, *tentry;
+    struct bc_hostEntry **tlast, *tentry;
     struct hostent *th;
 
     /* check that the host address is real */
@@ -131,13 +113,10 @@ HostAdd(alist, aname, aport)
     return 0;
 }
 
-static
-HostDelete(alist, aname, aport)
-     struct bc_hostEntry **alist;
-     afs_int32 aport;
-     char *aname;
+static int
+HostDelete(struct bc_hostEntry **alist, char *aname, afs_int32 aport)
 {
-    register struct bc_hostEntry **tlast, *tentry;
+    struct bc_hostEntry **tlast, *tentry;
 
     /* find guy to zap */
     tlast = alist;
@@ -154,24 +133,20 @@ HostDelete(alist, aname, aport)
     return 0;
 }
 
-bc_AddTapeHost(aconfig, aname, aport)
-     struct bc_config *aconfig;
-     afs_int32 aport;
-     char *aname;
+int
+bc_AddTapeHost(struct bc_config *aconfig, char *aname, afs_int32 aport)
 {
-    register afs_int32 code;
+    afs_int32 code;
 
     code = HostAdd(&aconfig->tapeHosts, aname, aport);
 
     return code;
 }
 
-bc_DeleteTapeHost(aconfig, aname, aport)
-     struct bc_config *aconfig;
-     afs_int32 aport;
-     char *aname;
+int
+bc_DeleteTapeHost(struct bc_config *aconfig, char *aname, afs_int32 aport)
 {
-    register afs_int32 code;
+    afs_int32 code;
 
     code = HostDelete(&aconfig->tapeHosts, aname, aport);
 
