@@ -12,19 +12,12 @@
 #include <afsconfig.h>
 #include <afs/stds.h>
 
+#include <roken.h>
+
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wstrict-prototypes"
 #endif
 
-#include <sys/types.h>
-#include <fcntl.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#elif defined(AFS_SUN5_ENV)
-#include <netdb.h>
-#else
-#include <sys/param.h>		/* for hostnames etc */
-#endif
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <ubik.h>
@@ -35,7 +28,6 @@
 #include <afs/budb_client.h>
 #include <afs/budb.h>
 #include <afs/com_err.h>
-#include <errno.h>
 
 #include "bc.h"
 #include "error_macros.h"
@@ -126,12 +118,6 @@ afs_int32 bcdb_listDumps (afs_int32 sflags, afs_int32 groupId,
 	free(dumpsList.budb_dumpsList_val);
     if (flagsList.budb_dumpsList_val)
 	free(flagsList.budb_dumpsList_val);
-
-    if (code == 0 &&
-	dumpsPtr->budb_dumpsList_len != flagsPtr->budb_dumpsList_len) {
-	code = BUDB_INTERNALERROR;
-    }
-
     return (code);
 }
 
@@ -913,18 +899,14 @@ udbClientInit(int noAuthFlag, int localauth, char *cellName)
 	afs_com_err(whoami, code, "(configuring connection security)");
 	ERROR(BC_NOCELLCONFIG);
     }
-    if (udbHandle.uh_scIndex == RX_SECIDX_NULL && !noAuthFlag)
+    if (&udbHandle.uh_scIndex == RX_SECIDX_NULL && !noAuthFlag)
 	afs_com_err(whoami, 0, "Can't get tokens - running unauthenticated");
 
-    /* We have to have space for the trailing NULL that terminates the server
-     * conneciton array - so we can only store MAXSERVERS-1 real elements in
-     * that array.
-     */
-    if (info.numServers >= MAXSERVERS) {
+    if (info.numServers > MAXSERVERS) {
 	afs_com_err(whoami, 0,
 		"Warning: %d BDB servers exist for cell '%s', can only remember the first %d",
-		info.numServers, cellName, MAXSERVERS-1);
-	info.numServers = MAXSERVERS - 1;
+		info.numServers, cellName, MAXSERVERS);
+	info.numServers = MAXSERVERS;
     }
 
     /* establish connections to the servers. Check for failed connections? */
@@ -1227,9 +1209,7 @@ int
 bc_openTextFile(udbClientTextP ctPtr, char *tmpFileName)
 {
     int code = 0;
-#ifdef HAVE_MKSTEMP
     int fd;
-#endif
 
     if (ctPtr->textStream != NULL) {
 	fclose(ctPtr->textStream);
@@ -1237,15 +1217,10 @@ bc_openTextFile(udbClientTextP ctPtr, char *tmpFileName)
     }
 
     sprintf(tmpFileName, "%s/bu_XXXXXX", gettmpdir());
-#ifdef HAVE_MKSTEMP
     fd = mkstemp(tmpFileName);
     if (fd == -1)
 	ERROR(BUDB_INTERNALERROR);
     ctPtr->textStream = fdopen(fd, "w+");
-#else
-    mktemp(tmpFileName);
-    ctPtr->textStream = fopen(tmpFileName, "w+");
-#endif
     if (ctPtr->textStream == NULL)
 	ERROR(BUDB_INTERNALERROR);
 
