@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,19 +14,13 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#include <sys/param.h>
-#endif
 #include <afs/stds.h>
-#include <sys/types.h>
+
+#include <roken.h>
+
 #include <ubik.h>
 #include <lock.h>
-#include <string.h>
+#include <afs/audit.h>
 
 #include "database.h"
 #include "budb.h"
@@ -34,7 +28,6 @@
 #include "error_macros.h"
 #include "budb_errs.h"
 #include "budb_internal.h"
-#include "afs/audit.h"
 
 
 /* dump ubik database - routines to scan the database and dump all
@@ -62,7 +55,9 @@ dumpSyncP dumpSyncPtr = &dumpSync;
 afs_int32
 canWrite(int fid)
 {
+#ifndef AFS_PTHREAD_ENV
     afs_int32 code = 0;
+#endif
     extern dumpSyncP dumpSyncPtr;
 
     ObtainWriteLock(&dumpSyncPtr->ds_lock);
@@ -72,7 +67,7 @@ canWrite(int fid)
 	if (dumpSyncPtr->ds_readerStatus == DS_WAITING) {
 	    dumpSyncPtr->ds_readerStatus = 0;
 #ifdef AFS_PTHREAD_ENV
-	    assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
+	    CV_BROADCAST(&dumpSyncPtr->ds_readerStatus_cond);
 #else
 	    code = LWP_SignalProcess(&dumpSyncPtr->ds_readerStatus);
 	    if (code)
@@ -82,9 +77,9 @@ canWrite(int fid)
 	dumpSyncPtr->ds_writerStatus = DS_WAITING;
 	ReleaseWriteLock(&dumpSyncPtr->ds_lock);
 #ifdef AFS_PTHREAD_ENV
-	assert(pthread_mutex_lock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
-	assert(pthread_cond_wait(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex) == 0);
-	assert(pthread_mutex_unlock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
+	MUTEX_ENTER(&dumpSyncPtr->ds_writerStatus_mutex);
+	CV_WAIT(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex);
+	MUTEX_EXIT(&dumpSyncPtr->ds_writerStatus_mutex);
 #else
 	LWP_WaitProcess(&dumpSyncPtr->ds_writerStatus);
 #endif
@@ -104,14 +99,16 @@ canWrite(int fid)
 void
 haveWritten(afs_int32 nbytes)
 {
+#ifndef AFS_PTHREAD_ENV
     afs_int32 code = 0;
+#endif
     extern dumpSyncP dumpSyncPtr;
 
     dumpSyncPtr->ds_bytes += nbytes;
     if (dumpSyncPtr->ds_readerStatus == DS_WAITING) {
 	dumpSyncPtr->ds_readerStatus = 0;
 #ifdef AFS_PTHREAD_ENV
-	assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
+	CV_BROADCAST(&dumpSyncPtr->ds_readerStatus_cond);
 #else
 	code = LWP_SignalProcess(&dumpSyncPtr->ds_readerStatus);
 	if (code)
@@ -129,7 +126,9 @@ haveWritten(afs_int32 nbytes)
 void
 doneWriting(afs_int32 error)
 {
+#ifndef AFS_PTHREAD_ENV
     afs_int32 code = 0;
+#endif
 
     /* wait for the reader */
     ObtainWriteLock(&dumpSyncPtr->ds_lock);
@@ -138,9 +137,9 @@ doneWriting(afs_int32 error)
 	dumpSyncPtr->ds_writerStatus = DS_WAITING;
 	ReleaseWriteLock(&dumpSyncPtr->ds_lock);
 #ifdef AFS_PTHREAD_ENV
-	assert(pthread_mutex_lock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
-	assert(pthread_cond_wait(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex) == 0);
-	assert(pthread_mutex_unlock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
+	MUTEX_ENTER(&dumpSyncPtr->ds_writerStatus_mutex);
+	CV_WAIT(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex);
+	MUTEX_EXIT(&dumpSyncPtr->ds_writerStatus_mutex);
 #else
 	LWP_WaitProcess(&dumpSyncPtr->ds_writerStatus);
 #endif
@@ -156,7 +155,7 @@ doneWriting(afs_int32 error)
 	dumpSyncPtr->ds_writerStatus = DS_DONE;
     dumpSyncPtr->ds_readerStatus = 0;
 #ifdef AFS_PTHREAD_ENV
-    assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
+    CV_BROADCAST(&dumpSyncPtr->ds_readerStatus_cond);
 #else
     code = LWP_NoYieldSignal(&dumpSyncPtr->ds_readerStatus);
     if (code)
