@@ -10,56 +10,52 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <afs/procmgmt.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wimplicit-function-declaration"
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
-#else
-#include <netinet/in.h>
-#include <sys/time.h>
 #endif
-#include <afs/procmgmt.h>
+
+#include <ctype.h>
+
+#include <rx/rx.h>
+#include <rx/rx_globals.h>
+#include <rx/rxkad.h>
 #include <rx/xdr.h>
+
 #include <afs/afsint.h>
-#include <stdio.h>
-#include <afs/afs_assert.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
-#include <string.h>
 #include <afs/vlserver.h>
 #include <lwp.h>
 #include <lock.h>
 #include <afs/afsutil.h>
-#include <rx/rx.h>
-#include <rx/rx_globals.h>
-#include <rx/rxkad.h>
 #include <afs/cellconfig.h>
 #include <afs/keys.h>
 #include <afs/volser.h>
 #include <ubik.h>
-#include <afs/audit.h>
 #include <afs/com_err.h>
-#include <errno.h>
 #include <afs/cmd.h>
 #include <afs/tcdata.h>
 #include <afs/bubasics.h>
-#include <ctype.h>
-#include "error_macros.h"
 #include <afs/budb_errs.h>
 #include <afs/budb_client.h>
 #include <afs/bucoord_prototypes.h>
-#include "afs/butx.h"
+#include <afs/butx.h>
+#include <afs/kautils.h>
+#include <afs/bc.h>
+
+#include "error_macros.h"
 #define XBSA_TCMAIN
 #include "butc_xbsa.h"
 #include "butc_prototypes.h"
-#include "butc_internal.h"
-#include <afs/kautils.h>
-#include <afs/bc.h>
 
 #define N_SECURITY_OBJECTS 3
 #define ERRCODE_RANGE 8		/* from error_table.h */
@@ -108,12 +104,18 @@ afs_int32 BufferSize;		/* Size in B stored for data */
 char *centralLogFile;
 afs_int32 lastLog;		/* Log last pass info */
 int rxBind = 0;
-struct afsconf_dir *butc_confdir;
-int allow_unauth = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
 
+/* dummy routine for the audit work.  It should do nothing since audits */
+/* occur at the server level and bos is not a server. */
+int
+osi_audit(void)
+{
+    return 0;
+}
+
 static afs_int32
 SafeATOL(char *anum)
 {
@@ -827,20 +829,11 @@ GetConfigParams(char *filename, afs_int32 port)
     return (code);
 }
 
-#ifdef xbsa
-static void
-xbsa_shutdown(int x)
-{
-    xbsa_Finalize(&butxInfo);
-    exit(0);
-}
-#endif
-
 static int
 WorkerBee(struct cmd_syndesc *as, void *arock)
 {
-    afs_int32 code, numClasses;
-    struct rx_securityClass *(nullObjects[1]), **secObjs, **allObjs;
+    afs_int32 code;
+    struct rx_securityClass *(securityObjects[3]);
     struct rx_service *service;
     time_t tokenExpires;
     char cellName[64];
@@ -855,8 +848,6 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
     PROCESS dbWatcherPid;
 #endif
     afs_uint32 host = htonl(INADDR_ANY);
-    char *auditFileName = NULL;
-    char *auditInterface = NULL;
 
     debugLevel = 0;
 
@@ -1002,29 +993,6 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	}
     }
 
-    /* Open the configuration directory */
-    butc_confdir = afsconf_Open(AFSDIR_SERVER_ETC_DIRPATH);
-    if (butc_confdir == NULL) {
-	TLog(0, "Failed to open server configuration directory");
-	exit(1);
-    }
-
-    /* Start auditing */
-    osi_audit_init();
-    if (as->parms[9].items) {
-	auditFileName = as->parms[9].items->data;
-    }
-    if (auditFileName != NULL)
-	osi_audit_file(auditFileName);
-    if (as->parms[10].items) {
-	auditInterface = as->parms[10].items->data;
-	if (osi_audit_interface(auditInterface)) {
-	    TLog(0, "Invalid audit interface '%s'\n", auditInterface);
-	    exit(1);
-	}
-    }
-    osi_audit(TC_StartEvent, 0, AUD_END);
-
     if (as->parms[1].items) {
 	debugLevel = SafeATOL(as->parms[1].items->data);
 	if (debugLevel == -1) {
@@ -1049,8 +1017,6 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	rc = InitToServer(0 /*taskid */ , &butxInfo, adsmServerName);
 	if (rc != XBSA_SUCCESS)
 	    return (1);
-	(void)signal(SIGINT, xbsa_shutdown);
-	(void)signal(SIGHUP, xbsa_shutdown);
     }
 #endif /*xbsa */
 
@@ -1065,13 +1031,6 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     localauth = (as->parms[5].items ? 1 : 0);
     rxBind = (as->parms[8].items ? 1 : 0);
-    allow_unauth = (as->parms[11].items ? 1 : 0);
-
-    if (!allow_unauth && !localauth) {
-	const char *errstr = "Neither -localauth nor -allow_unauthenticated was provided; refusing to start in unintended insecure configuration\n";
-	TLog(0, "%s", (char *)errstr);
-	exit(1);
-    }
 
     if (rxBind) {
         afs_int32 ccode;
@@ -1117,48 +1076,20 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     /* initialize database support, volume support, and logs */
 
-    /*
-     * Create security objects for the Rx server functionality.  Historically
-     * this was a single rxnull security object, since the tape controller was
-     * run by an operator that had local access to the tape device and some
-     * administrative privilege in the cell (to be able to perform volume-level
-     * accesses), but on a machine that was not necessarily trusted to hold the
-     * cell-wide key.
-     *
-     * Such a configuration is, of course, insecure because anyone can make
-     * inbound RPCs and manipulate the database, including creating bogus
-     * dumps and restoring them!  Additionally, in modern usage, butc is
-     * frequently run with -localauth to authenticate its outbound connections
-     * to the volservers and budb with the cell-wide key, in which case the
-     * cell-wide key is present and could be used to authenticate incoming
-     * connections as well.
-     *
-     * If -localauth is in use, create the full barrage of server security
-     * objects, including rxkad, so that inbound connections can be verified
-     * to only be made by authenticated clients.  Otherwise, only the rxnull
-     * class is in use with a single server security object.  Note that butc
-     * will refuse to start in this configuration unless the
-     * "-allow_unauthenticated" flag is provided, indicating that the operator
-     * has ensured that incoming connections are appropriately restricted by
-     * firewall configuration or network topology.
+    /* Create a single security object, in this case the null security
+     * object, for unauthenticated connections, which will be used to control
+     * security on connections made to this server
      */
 
-    if (allow_unauth) {
-	nullObjects[RX_SECIDX_NULL] = rxnull_NewServerSecurityObject();
-	if (!nullObjects[RX_SECIDX_NULL]) {
-	    TLog(0, "rxnull_NewServerSecurityObject");
-	    exit(1);
-	}
-	numClasses = 1;
-	secObjs = nullObjects;
-    } else {
-	/* Must be -localauth, so the cell keys are available. */
-	afsconf_BuildServerSecurityObjects(butc_confdir, 0, &allObjs, &numClasses);
-	secObjs = allObjs;
+    securityObjects[0] = rxnull_NewServerSecurityObject();
+    securityObjects[1] = (struct rx_securityClass *)0;	/* don't bother with rxvab */
+    if (!securityObjects[0]) {
+	TLog(0, "rxnull_NewServerSecurityObject");
+	exit(1);
     }
 
     service =
-	rx_NewServiceHost(host, 0, 1, "BUTC", secObjs, numClasses, TC_ExecuteRequest);
+	rx_NewServiceHost(host, 0, 1, "BUTC", securityObjects, 3, TC_ExecuteRequest);
     if (!service) {
 	TLog(0, "rx_NewService");
 	exit(1);
@@ -1261,11 +1192,6 @@ main(int argc, char **argv)
 		"Force multiple XBSA server support");
     cmd_AddParm(ts, "-rxbind", CMD_FLAG, CMD_OPTIONAL,
 		"bind Rx socket");
-    cmd_AddParm(ts, "-auditlog", CMD_SINGLE, CMD_OPTIONAL, "location of audit log");
-    cmd_AddParm(ts, "-audit-interface", CMD_SINGLE, CMD_OPTIONAL,
-		"interface to use for audit logging");
-    cmd_AddParm(ts, "-allow_unauthenticated", CMD_FLAG, CMD_OPTIONAL,
-		"allow unauthenticated inbound RPCs (requires firewalling)");
 
     /* Initialize dirpaths */
     if (!(initAFSDirPath() & AFSDIR_SERVER_PATHS_OK)) {
