@@ -17,10 +17,9 @@
 # pragma GCC diagnostic warning "-Wimplicit-function-declaration"
 #endif
 
-#include <rx/xdr.h>
+#include <afs/opr.h>
 #include <rx/rx.h>
 #include <afs/afsint.h>
-#include <afs/afs_assert.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
 #include <lwp.h>
