@@ -3,17 +3,20 @@
 #ifndef	AFS_PARAM_COMMON_H
 #define	AFS_PARAM_COMMON_H 1
 
-#define AFS_64BIT_ENV  1
 #define AFS_NAMEI_ENV  1	/* User space interface to file system */
 #define AFS_64BIT_IOPS_ENV 1	/* Needed for NAMEI */
 #define AFS_64BIT_CLIENT 1
 
 #define AFS_MOUNT_AFS "afs"	/* The name of the filesystem type. */
-#define AFS_SYSCALL 210
+#define AFS_SYSCALL 318 /* 210 */
 
+#if 0
+/* including this file before sysincludes.h is canonical, but
+ * NBSD40's mount.h defines MOUNT_AFS */
 #ifndef	MOUNT_AFS
 #define	MOUNT_AFS AFS_MOUNT_AFS
 #endif
+#endif
 
 #define AFS_XBSD_ENV 1		/* {Free,Open,Net}BSD */
 
@@ -47,10 +50,8 @@
 #define ICHG 0x0040
 #define IMOD 0x0080
 
-#define IN_LOCK(ip)     lockmgr(&ip->i_lock, LK_EXCLUSIVE, \
-                                NULL, curproc)
-#define IN_UNLOCK(ip)   lockmgr(&ip->i_lock, LK_RELEASE, \
-                                NULL, curproc)
+#define RXK_LISTENER_ENV      1
+
 
 #include <afs/afs_sysnames.h>
 
@@ -64,6 +65,14 @@
 /* Extra kernel definitions (from kdefs file) */
 #ifdef _KERNEL
 #define AFS_GLOBAL_SUNLOCK        1
+#define	AFS_SHORTGID	1	/* are group id's short? */
+#endif
+
+/* apparently not in 40 */
+#define inline
+
+#ifdef _KERNEL_DEPRECATED
+#define AFS_GLOBAL_SUNLOCK        1
 #define	AFS_VFS34	1	/* What is VFS34??? */
 #define	AFS_SHORTGID	1	/* are group id's short? */
 #define	afsio_iov	uio_iov
@@ -111,8 +120,8 @@ enum vcexcl { NONEXCL, EXCL };
 #endif
 #endif /* KERNEL */
 
-#endif /* ! ASSEMBLER & ! __LANGUAGE_ASSEMBLY__ && !defined(IGNORE_STDS_H) */ 
-#endif /* _KERNEL */
+#endif /* ! ASSEMBLER & ! __LANGUAGE_ASSEMBLY__ && !defined(IGNORE_STDS_H) */
+#endif /* _KERNEL_DEPRECATED */
 
 #else /* !defined(UKERNEL) */
 
