@@ -4,7 +4,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -21,7 +21,6 @@
 #define AFS_AIX52_ENV		1
 #define AFS_AIX53_ENV		1
 
-#define AFS_64BIT_ENV		1
 #define AFS_64BIT_CLIENT	1
 #define AFS_NAMEI_ENV		1
 #ifdef AFS_NAMEI_ENV
@@ -90,7 +89,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -99,14 +98,11 @@
 #ifndef	AFS_PARAM_H
 #define	AFS_PARAM_H
 
-#define AFS_64BIT_ENV           1
 #define AFS_64BIT_CLIENT        1
 #define AFS_NAMEI_ENV           1
 #ifdef AFS_NAMEI_ENV
 #define AFS_64BIT_IOPS_ENV	1
 #endif
-#define BITMAP_LATER            1
-#define FAST_RESTART            1
 
 #define AFS_VFS_ENV	1
 /* Used only in vfsck code; is it needed any more???? */
@@ -151,7 +147,7 @@
 #define SYS_NAME_ID	SYS_NAME_ID_rs_aix51
 #define AFSBIG_ENDIAN	1
 #define AFS_HAVE_FFS            1	/* Use system's ffs. */
-#define AFS_HAVE_STATVFS	0	/* System doesn't support statvfs */
+#define AFS_HAVE_STATVFS	1	/* System supports statvfs */
 
 /* Extra kernel definitions (from kdefs file) */
 #ifdef KERNEL
