@@ -14,26 +14,18 @@
 /* These two needed for rxgen output to work */
 #include <afsconfig.h>
 #include <afs/param.h>
+#include <afs/stds.h>
 
+#include <roken.h>
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <errno.h>
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <afs/fs_utils.h>
 #else
-#include <unistd.h>
-#include <netinet/in.h>
 #include <afs/venus.h>
 #endif
-#include <signal.h>
-#include <afs/stds.h>
+
 #include <rx/xdr.h>
 #include <afs/prs_fs.h>
-#include <stdlib.h>
-#include <string.h>
 #include <afs/sys_prototypes.h>
 #include <afs/afs_consts.h>
 
@@ -53,6 +45,7 @@ simplify_name(char *orig_name, char *true_name)
 {
     struct stat statbuff;
 
+
 #ifdef AFS_NT40_ENV
     if (stat(orig_name, &statbuff) < 0) {
 	*true_name = '\0';
@@ -198,7 +191,7 @@ find_me(char *arg, char *parent_dir)
     return 1;			/* found it */
 }
 
-#define SkipLine(str) do { while (*str != '\0' && *str !='\n') str++; if (*str == '\n') str++; } while(0)
+#define SkipLine(str) { while (*str !='\n') str++; str++; }
 
 /* this function returns TRUE (1) if the file is in AFS, otherwise false (0) */
 static int
@@ -236,8 +229,8 @@ struct AclEntry {
 static struct Acl *
 ParseAcl(char *astr)
 {
-    int nplus = 0, nminus = 0, i, trights = 0;
-    char tname[MAXNAME + 1] = "";
+    int nplus, nminus, i, trights;
+    char tname[MAXNAME];
     struct AclEntry *first, *last, *tl;
     struct Acl *ta;
     sscanf(astr, "%d", &nplus);
@@ -245,7 +238,7 @@ ParseAcl(char *astr)
     sscanf(astr, "%d", &nminus);
     SkipLine(astr);
 
-    ta = calloc(sizeof(*ta), 1);
+    ta = (struct Acl *)malloc(sizeof(struct Acl));
     ta->nplus = nplus;
 
     last = 0;
@@ -396,10 +389,8 @@ init_child(char *myname)
      * reads from the latter, the child reads from the former, and
      * writes to the latter.
      */
-    if (pipe(pipe1) || pipe(pipe2)) {
-	using_child = 0;
-	return 0;
-    }
+    pipe(pipe1);
+    pipe(pipe2);
 
     /* fork a child */
     pid = fork();
@@ -440,8 +431,7 @@ password_bad(char *pw)
     if (using_child) {
 	fprintf(childin, "%s\n", pw);
 	fflush(childin);
-	if (fscanf(childout, "%d", &rc) < 1)
-	    rc = -1;
+	fscanf(childout, "%d", &rc);
     }
 
     return (rc);
