@@ -9,14 +9,14 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <rx/rx.h>
+#include <rx/rxstat.h>
+
 #include <afs/afs_Admin.h>
-#include <stdio.h>
-#include <string.h>
-#include <afs/afs_Admin.h>
-#include "afs_AdminInternal.h"
 #include <afs/pthread_glock.h>
 #include <afs/cellconfig.h>
 #include <afs/dirpath.h>
@@ -28,72 +28,14 @@
 #include <afs/bnode.h>
 #include <afs/volser.h>
 #include <afs/afscbint.h>
-#include <rx/rx.h>
-#include <rx/rxstat.h>
+
 #ifdef AFS_NT40_ENV
-# include <winsock2.h>
 # include <afs/krb5_nt.h>
-#else
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
 #endif
+
+#include "afs_AdminInternal.h"
 #include "afs_utilAdmin.h"
 
-/*
- * The 'admin_strlcpy' function is copied from heimdal, under the following
- * license:
- *
- * Copyright (c) 1995-2002 Kungliga Tekniska Högskolan
- * (Royal Institute of Technology, Stockholm, Sweden).
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * 3. Neither the name of the Institute nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-static size_t
-admin_strlcpy(char *dst, const char *src, size_t dst_sz)
-{
-    size_t n;
-
-    for (n = 0; n < dst_sz; n++) {
-	if ((*dst++ = *src++) == '\0')
-	    break;
-    }
-
-    if (n < dst_sz)
-	return n;
-    if (n > 0)
-	*(dst - 1) = '\0';
-    return n + strlen (src);
-}
-
 /*
  * AIX 4.2 has PTHREAD_CREATE_UNDETACHED and not PTHREAD_CREATE_JOINABLE
  *
@@ -2059,14 +2001,14 @@ ListCellsRPC(void *rpc_specific, int slot, int *last_item,
     int rc = 0;
     afs_status_t tst = 0;
     cm_list_cell_get_p t = (cm_list_cell_get_p) rpc_specific;
-    char *name = NULL;
-    size_t len;
+    char *name;
     serverList sl;
     unsigned int n;
 
     /*
      * Get the next entry in the CellServDB.
      */
+    name = t->cell[slot].cellname;
     sl.serverList_len = 0;
     sl.serverList_val = NULL;
     memset(t->cell[slot].serverAddr, 0, sizeof(afs_int32)*UTIL_MAX_CELL_HOSTS);
@@ -2075,11 +2017,7 @@ ListCellsRPC(void *rpc_specific, int slot, int *last_item,
     if (tst) {
 	goto fail_ListCellsRPC;
     }
-    len = admin_strlcpy(t->cell[slot].cellname, name, sizeof(t->cell[slot].cellname));
-    if (len >= sizeof(t->cell[slot].cellname)) {
-	tst = ADMRPCTOOBIG;
-	goto fail_ListCellsRPC;
-    }
+    strcpy(t->cell[slot].cellname, name);
     if (sl.serverList_val) {
         for (n=0; n<sl.serverList_len && n<UTIL_MAX_CELL_HOSTS; n++) {
             t->cell[slot].serverAddr[n] = sl.serverList_val[n];
@@ -2099,7 +2037,6 @@ ListCellsRPC(void *rpc_specific, int slot, int *last_item,
     rc = 1;
 
   fail_ListCellsRPC:
-    xdr_free((xdrproc_t) xdr_string, &name);
 
     if (st != NULL) {
 	*st = tst;
@@ -2297,8 +2234,7 @@ util_CMLocalCell(struct rx_connection *conn, afs_CMCellName_p cellName,
 {
     int rc = 0;
     afs_status_t tst = 0;
-    char *name = NULL;
-    size_t len;
+    afs_CMCellName_p name;
 
     if (conn == NULL) {
 	tst = ADMRXCONNNULL;
@@ -2310,21 +2246,15 @@ util_CMLocalCell(struct rx_connection *conn, afs_CMCellName_p cellName,
 	goto fail_util_CMLocalCell;
     }
 
+    name = cellName;
     tst = RXAFSCB_GetLocalCell(conn, &name);
-    if (tst != 0) {
-	goto fail_util_CMLocalCell;
-    }
 
-    len = admin_strlcpy(cellName, name, sizeof(cellName));
-    if (len >= sizeof(cellName)) {
-	tst = ADMRPCTOOBIG;
-	goto fail_util_CMLocalCell;
+    if (!tst) {
+	rc = 1;
     }
-    rc = 1;
 
   fail_util_CMLocalCell:
 
-    xdr_free((xdrproc_t)xdr_string, &name);
     if (st != NULL) {
 	*st = tst;
     }
@@ -2377,9 +2307,6 @@ util_CMClientConfig(struct rx_connection *conn, afs_ClientConfig_p config,
     afs_uint32 allocbytes;
     struct cacheConfig tconfig;
 
-    tconfig.cacheConfig_val = NULL;
-    tconfig.cacheConfig_len = 0;
-
     if (conn == NULL) {
 	tst = ADMRXCONNNULL;
 	goto fail_util_CMClientConfig;
@@ -2391,6 +2318,8 @@ util_CMClientConfig(struct rx_connection *conn, afs_ClientConfig_p config,
     }
 
     config->clientVersion = AFS_CLIENT_RETRIEVAL_VERSION;
+    tconfig.cacheConfig_val = NULL;
+    tconfig.cacheConfig_len = 0;
     tst =
 	RXAFSCB_GetCacheConfig(conn, config->clientVersion,
 			       &config->serverVersion, &allocbytes, &tconfig);
@@ -2399,17 +2328,12 @@ util_CMClientConfig(struct rx_connection *conn, afs_ClientConfig_p config,
 	goto fail_util_CMClientConfig;
     }
 
-    if (tconfig.cacheConfig_len != sizeof(cm_initparams_v1)/sizeof(afs_uint32)) {
-	tst = RXGEN_CC_UNMARSHAL;
-	goto fail_util_CMClientConfig;
-    }
-
     UnmarshallCMClientConfig(config->serverVersion, tconfig.cacheConfig_val,
 			     &config->c);
     rc = 1;
+    free(tconfig.cacheConfig_val);
 
   fail_util_CMClientConfig:
-    free(tconfig.cacheConfig_val);
 
     if (st != NULL) {
 	*st = tst;
