@@ -13,28 +13,16 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
 
-#include <stddef.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
+#include <roken.h>
 
 #include <pthread.h>
 
 #ifdef AFS_NT40_ENV
 #include <WINNT/vptab.h>
 #include <WINNT/afsreg.h>
-#include <io.h>
 #include <direct.h>
-#else
-#include <unistd.h>
 #endif /* AFS_NT40_ENV */
 
 #include <rx/rx.h>
@@ -48,7 +36,6 @@
 #include <afs/afs_kasAdmin.h>
 #include <afs/afs_ptsAdmin.h>
 
-
 #include <afs/kautils.h>
 #include <afs/bnode.h>
 #include <afs/prerror.h>
@@ -97,6 +84,7 @@ cfg_HostQueryStatus(const char *hostName,	/* name of host */
 		    afs_status_p st)
 {				/* completion status */
     int rc = 1;
+    struct afsconf_keys keys;
     afs_status_t tst2, tst = 0;
     afs_status_t serverSt = 0;
     char *serverCellName = NULL;
@@ -162,12 +150,13 @@ cfg_HostQueryStatus(const char *hostName,	/* name of host */
 	    /* one or more config files appears to be invalid */
 	    serverSt = ADMCFGSERVERBASICINFOINVALID;
 	} else {
-	    struct afsconf_entry *cellentry;
+	    struct afsconf_entry *cellentry = NULL;
 
 	    if (confdir->cellName == NULL || *confdir->cellName == '\0') {
 		/* no cell set for server */
 		serverSt = ADMCFGSERVERNOTINCELL;
-	    } else if (confdir->keystr == NULL || confdir->keystr->nkeys == 0) {
+	    } else if ((afsconf_GetKeys(confdir, &keys) != 0)
+		       || (keys.nkeys == 0)) {
 		/* no server keys */
 		serverSt = ADMCFGSERVERNOKEYS;
 	    } else {
