@@ -9,20 +9,23 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <stdio.h>
-#include <ctype.h>
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <ctype.h>
+
 #include <rx/rx.h>
 #include <rx/rxstat.h>
-#include "afs_ptsAdmin.h"
-#include "../adminutil/afs_AdminInternal.h"
+
 #include <afs/afs_AdminErrors.h>
 #include <afs/afs_utilAdmin.h>
 #include <afs/ptint.h>
 #include <afs/ptserver.h>
 
+#include "afs_ptsAdmin.h"
+#include "../adminutil/afs_AdminInternal.h"
+
 /*
  * IsValidCellHandle - validate the cell handle for making pts
  * requests.
@@ -122,10 +125,6 @@ TranslatePTSNames(const afs_cell_handle_p cellHandle, namelist * names,
 	goto fail_TranslatePTSNames;
     }
 
-    if (ids->idlist_len != names->namelist_len) {
-	tst = ADMPTSFAILEDNAMETRANSLATE;
-	goto fail_TranslatePTSNames;
-    }
 
     /*
      * Check to see if the lookup failed
@@ -469,8 +468,6 @@ pts_GroupOwnerChange(const void *cellHandle, const char *targetGroup,
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
     idlist ids;
 
-    memset(&ids, 0, sizeof(ids));
-
     /*
      * Validate arguments
      */
