@@ -16,19 +16,14 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/libadmin/samples/rxstat_get_peer.c,v 1.7.14.2 2009/03/16 13:26:52 shadow Exp $");
+#include <roken.h>
 
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <pthread.h>
 #endif
 
-#include <string.h>
-
 #include <rx/rx.h>
 #include <rx/rxstat.h>
-
 #include <afs/afs_Admin.h>
 #include <afs/afs_AdminErrors.h>
 #include <afs/afs_clientAdmin.h>
