@@ -1,24 +1,24 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#include <stdio.h>
-#include <assert.h>
-#include <unistd.h>
-#include <errno.h>
-#include <syslog.h>
-#include <security/pam_appl.h>
 #include <afsconfig.h>
 #include <afs/param.h>
+
+#include <roken.h>
+
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
+
+#include <assert.h>
 #include <limits.h>
-#include <string.h>
-#include <stdlib.h>
+
 #ifdef AFS_AIX51_ENV
 #include <sys/cred.h>
 #ifdef HAVE_SYS_PAG_H
@@ -26,8 +26,9 @@
 #endif
 #endif
 
-RCSID
-    ("$Header: /cvs/openafs/src/pam/afs_util.c,v 1.12.14.5 2009/03/20 22:45:22 shadow Exp $");
+#include <security/pam_appl.h>
+
+#include <afs/auth.h>
 
 #include "afs_util.h"
 
@@ -52,36 +53,13 @@ nil_cleanup(pam_handle_t * pamh, void *data, int pam_end_status)
     return;
 }
 
-/* The PAM module needs to be free from libucb dependency. Otherwise, 
-dynamic linking is a problem, the AFS PAM library refuses to coexist
-with the DCE library. The sigvec() and sigsetmask() are the only two
-calls that neccesiate the inclusion of libucb.a.  There are used by
-the lwp library to support premeptive threads and signalling between 
-threads. Since the lwp support used by the PAM module uses none of 
-these facilities, we can safely define these to be null functions */
-
-#if !defined(AFS_HPUX110_ENV)
-/* For HP 11.0, this function is in util/hputil.c */
-int
-sigvec(int sig, const struct sigvec *vec, struct sigvec *ovec)
-{
-    assert(0);
-}
-
-int
-sigsetmask(int mask)
-{
-    assert(0);
-}
-#endif /* AFS_HPUX110_ENV */
-
 /* converts string to integer */
 
 char *
-cv2string(register char *ttp, register unsigned long aval)
+cv2string(char *ttp, unsigned long aval)
 {
-    register char *tp = ttp;
-    register int i;
+    char *tp = ttp;
+    int i;
     int any = 0;
 
     *(--tp) = 0;
@@ -176,49 +154,13 @@ do_klog(const char *user, const char *password, const char *lifetime,
     return (ret);
 }
 
-/* get the current AFS pag for the calling process */
-static afs_int32
-curpag(void)
-{
-#if defined(AFS_AIX51_ENV)
-    int code = getpagvalue("afs");
-    if (code < 0 && errno == EINVAL)
-        code = 0;
-    return code;
-#else
-    gid_t groups[NGROUPS_MAX];
-    afs_uint32 g0, g1;
-    afs_uint32 h, l, ret;
-
-    if (getgroups(sizeof groups / sizeof groups[0], groups) < 2)
-	return 0;
-
-    g0 = groups[0] & 0xffff;
-    g1 = groups[1] & 0xffff;
-    g0 -= 0x3f00;
-    g1 -= 0x3f00;
-    if (g0 < 0xc000 && g1 < 0xc000) {
-	l = ((g0 & 0x3fff) << 14) | (g1 & 0x3fff);
-	h = (g0 >> 14);
-	h = (g1 >> 14) + h + h + h;
-	ret = ((h << 28) | l);
-	/* Additional testing */
-	if (((ret >> 24) & 0xff) == 'A')
-	    return ret;
-	else
-	    return -1;
-    }
-    return -1;
-#endif
-}
-
 /* Returns the AFS pag number, if any, otherwise return -1 */
 afs_int32
 getPAG(void)
 {
     afs_int32 pag;
 
-    pag = curpag();
+    pag = ktc_curpag();
     if (pag == 0 || pag == -1)
 	return -1;
 
