@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,13 +10,9 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <stdio.h>
 #include <security/pam_appl.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-
 
 static int my_conv(int num_msg, PAM_CONST struct pam_message **msg,
 		   struct pam_response **response, void *appdata_ptr);
@@ -32,12 +28,12 @@ static const char *service = "afstest";
 static const char *new_envstring = "GOTHEREVIATESTPAM=1";
 static const char *new_homestring = "HOME=/tmp";
 
-#if defined(AFS_LINUX20_ENV) || defined(AFS_FBSD_ENV) || defined(AFS_DFBSD_ENV) || defined(AFS_NBSD_ENV)
+#if defined(AFS_LINUX20_ENV) || defined(AFS_FBSD_ENV) || defined(AFS_DFBSD_ENV) || defined(AFS_NBSD_ENV) || defined(AFS_DARWIN_ENV)
 #define getpassphrase getpass
 #endif
 
 
-void
+int
 main(int argc, char *argv[])
 {
     int authenticated = 0;
@@ -101,7 +97,7 @@ main(int argc, char *argv[])
     putenv((char *)new_homestring);
     chdir("/tmp");
     printf("Type exit to back out.\n");
-    execl("/bin/csh", "/bin/csh", NULL);
+    return execl("/bin/csh", "/bin/csh", NULL);
 }
 
 
