@@ -12,6 +12,10 @@
 
 
 #include "rx/rx_kcommon.h"
+#include "rx/rx_atomic.h"
+#include "rx/rx_internal.h"
+#include "rx/rx_packet.h"
+#include "rx/rx_stats.h"
 
 #ifdef AFS_DARWIN80_ENV
 #define soclose sock_close
@@ -34,9 +38,6 @@ rx_upcall(socket_t so, void *arg, __unused int waitflag)
     afs_int32 savelen;          /* was using rlen but had aliasing problems */
     size_t nbytes, resid, noffset;
 
-    /* See if a check for additional packets was issued */
-    rx_CheckPackets();
-
     p = rxi_AllocPacket(RX_PACKET_CLASS_RECEIVE);
     rx_computelen(p, tlen);
     rx_SetDataSize(p, tlen);    /* this is the size of the user data area */
@@ -105,7 +106,7 @@ rx_upcall(socket_t so, void *arg, __unused int waitflag)
 	    if (nbytes <= 0) {
 		if (rx_stats_active) {
 		    MUTEX_ENTER(&rx_stats_mutex);
-		    rx_stats.bogusPacketOnRead++;
+		    rx_atomic_inc(&rx_stats.bogusPacketOnRead);
 		    rx_stats.bogusHost = from.sin_addr.s_addr;
 		    MUTEX_EXIT(&rx_stats_mutex);
 		}
@@ -121,9 +122,7 @@ rx_upcall(socket_t so, void *arg, __unused int waitflag)
 	    port = from.sin_port;
 	    if (p->header.type > 0 && p->header.type < RX_N_PACKET_TYPES) {
 		if (rx_stats_active) {
-		    MUTEX_ENTER(&rx_stats_mutex);
-		    rx_stats.packetsRead[p->header.type - 1]++;
-		    MUTEX_EXIT(&rx_stats_mutex);
+		    rx_atomic_inc(&rx_stats.packetsRead[p->header.type - 1]);
 		}
 	    }
 
