@@ -59,7 +59,8 @@
 #include "rx_clock.h"
 #include "rx_queue.h"
 #include "rx_globals.h"
-
+#include "rx_atomic.h"
+#include "rx_internal.h"
 #include "rx_conn.h"
 #include "rx_call.h"
 #include "rx_packet.h"
@@ -68,6 +69,91 @@
 /* rxdb_fileID is used to identify the lock location, along with line#. */
 static int rxdb_fileID = RXDB_FILE_RX_RDWR;
 #endif /* RX_LOCKS_DB */
+
+/* Get the next packet in the receive queue
+ *
+ * Dispose of the call's currentPacket, and move the next packet in the
+ * receive queue into the currentPacket field. If the next packet isn't
+ * available, then currentPacket is left NULL.
+ *
+ * @param call
+ * 	The RX call to manipulate
+ * @returns
+ * 	0 on success, an error code on failure
+ *
+ * @notes
+ * 	Must be called with the call locked. Unlocks the call if returning
+ * 	with an error.
+ */
+
+static int
+rxi_GetNextPacket(struct rx_call *call) {
+    struct rx_packet *rp;
+    int error;
+
+    if (call->currentPacket != NULL) {
+#ifdef RX_TRACK_PACKETS
+	call->currentPacket->flags |= RX_PKTFLAG_CP;
+#endif
+	rxi_FreePacket(call->currentPacket);
+	call->currentPacket = NULL;
+    }
+
+    if (queue_IsEmpty(&call->rq))
+	return 0;
+
+    /* Check that next packet available is next in sequence */
+    rp = queue_First(&call->rq, rx_packet);
+    if (rp->header.seq != call->rnext)
+	return 0;
+
+    queue_Remove(rp);
+#ifdef RX_TRACK_PACKETS
+    rp->flags &= ~RX_PKTFLAG_RQ;
+#endif
+#ifdef RXDEBUG_PACKET
+    call->rqc--;
+#endif /* RXDEBUG_PACKET */
+
+    /* RXS_CheckPacket called to undo RXS_PreparePacket's work.  It may
+     * reduce the length of the packet by up to conn->maxTrailerSize,
+     * to reflect the length of the data + the header. */
+    if ((error = RXS_CheckPacket(call->conn->securityObject, call, rp))) {
+	/* Used to merely shut down the call, but now we shut down the whole
+	 * connection since this may indicate an attempt to hijack it */
+
+	MUTEX_EXIT(&call->lock);
+	rxi_ConnectionError(call->conn, error);
+	MUTEX_ENTER(&call->conn->conn_data_lock);
+	rp = rxi_SendConnectionAbort(call->conn, rp, 0, 0);
+	MUTEX_EXIT(&call->conn->conn_data_lock);
+	rxi_FreePacket(rp);
+
+	return error;
+     }
+
+    call->rnext++;
+    call->currentPacket = rp;
+#ifdef RX_TRACK_PACKETS
+    call->currentPacket->flags |= RX_PKTFLAG_CP;
+#endif
+    call->curvec = 1;	/* 0th vec is always header */
+
+    /* begin at the beginning [ more or less ], continue on until the end,
+     * then stop. */
+    call->curpos = (char *)call->currentPacket->wirevec[1].iov_base +
+		   call->conn->securityHeaderSize;
+    call->curlen = call->currentPacket->wirevec[1].iov_len -
+		   call->conn->securityHeaderSize;
+
+    call->nLeft = call->currentPacket->length;
+    hadd32(call->bytesRcvd, call->currentPacket->length);
+
+    call->nHardAcks++;
+
+    return 0;
+}
+
 /* rxi_ReadProc -- internal version.
  *
  * LOCKS USED -- called at netpri
@@ -76,9 +162,8 @@ int
 rxi_ReadProc(struct rx_call *call, char *buf,
 	     int nbytes)
 {
-    struct rx_packet *cp = call->currentPacket;
-    struct rx_packet *rp;
     int requestCount;
+    int code;
     unsigned int t;
 
 /* XXXX took out clock_NewTime from here.  Was it needed? */
@@ -110,83 +195,23 @@ rxi_ReadProc(struct rx_call *call, char *buf,
 			continue;
 		    }
 		}
-		if (queue_IsNotEmpty(&call->rq)) {
-		    /* Check that next packet available is next in sequence */
-		    rp = queue_First(&call->rq, rx_packet);
-		    if (rp->header.seq == call->rnext) {
-			afs_int32 error;
-			struct rx_connection *conn = call->conn;
-			queue_Remove(rp);
-#ifdef RX_TRACK_PACKETS
-			rp->flags &= ~RX_PKTFLAG_RQ;
-#endif
-#ifdef RXDEBUG_PACKET
-                        call->rqc--;
-#endif /* RXDEBUG_PACKET */
 
-			/* RXS_CheckPacket called to undo RXS_PreparePacket's
-			 * work.  It may reduce the length of the packet by up
-			 * to conn->maxTrailerSize, to reflect the length of the
-			 * data + the header. */
-			if ((error =
-			     RXS_CheckPacket(conn->securityObject, call,
-					     rp))) {
-			    /* Used to merely shut down the call, but now we
-			     * shut down the whole connection since this may
-			     * indicate an attempt to hijack it */
+		code = rxi_GetNextPacket(call);
+		if (code)
+		     return 0;
 
-			    MUTEX_EXIT(&call->lock);
-			    rxi_ConnectionError(conn, error);
-			    MUTEX_ENTER(&conn->conn_data_lock);
-			    rp = rxi_SendConnectionAbort(conn, rp, 0, 0);
-			    MUTEX_EXIT(&conn->conn_data_lock);
-			    rxi_FreePacket(rp);
-
-			    return 0;
+		if (call->currentPacket) {
+		    if (!(call->flags & RX_CALL_RECEIVE_DONE)) {
+			if (call->nHardAcks > (u_short) rxi_HardAckRate) {
+			    rxevent_Cancel(&call->delayedAckEvent, call,
+					   RX_CALL_REFCOUNT_DELAY);
+			    rxi_SendAck(call, 0, 0, RX_ACK_DELAY, 0);
+			} else {
+			    /* Delay to consolidate ack packets */
+			    rxi_PostDelayedAckEvent(call, &rx_hardAckDelay);
 			}
-			call->rnext++;
-			cp = call->currentPacket = rp;
-#ifdef RX_TRACK_PACKETS
-			call->currentPacket->flags |= RX_PKTFLAG_CP;
-#endif
-			call->curvec = 1;	/* 0th vec is always header */
-			/* begin at the beginning [ more or less ], continue
-			 * on until the end, then stop. */
-			call->curpos =
-			    (char *)cp->wirevec[1].iov_base +
-			    call->conn->securityHeaderSize;
-			call->curlen =
-			    cp->wirevec[1].iov_len -
-			    call->conn->securityHeaderSize;
-
-			/* Notice that this code works correctly if the data
-			 * size is 0 (which it may be--no reply arguments from
-			 * server, for example).  This relies heavily on the
-			 * fact that the code below immediately frees the packet
-			 * (no yields, etc.).  If it didn't, this would be a
-			 * problem because a value of zero for call->nLeft
-			 * normally means that there is no read packet */
-			call->nLeft = cp->length;
-			hadd32(call->bytesRcvd, cp->length);
-
-			/* Send a hard ack for every rxi_HardAckRate+1 packets
-			 * consumed. Otherwise schedule an event to send
-			 * the hard ack later on.
-			 */
-			call->nHardAcks++;
-			if (!(call->flags & RX_CALL_RECEIVE_DONE)) {
-			    if (call->nHardAcks > (u_short) rxi_HardAckRate) {
-				rxevent_Cancel(&call->delayedAckEvent, call,
-					       RX_CALL_REFCOUNT_DELAY);
-				rxi_SendAck(call, 0, 0, RX_ACK_DELAY, 0);
-			    } else {
-				/* Delay to consolidate ack packets */
-				rxi_PostDelayedAckEvent(call,
-							&rx_hardAckDelay);
-			    }
-			}
-			break;
 		    }
+		    break;
 		}
 
                 /*
@@ -212,7 +237,6 @@ rxi_ReadProc(struct rx_call *call, char *buf,
 		    osi_rxSleep(&call->rq);
 #endif
 		}
-                cp = call->currentPacket;
 
 		call->startWait = 0;
 #ifdef RX_ENABLE_LOCKS
@@ -233,7 +257,7 @@ rxi_ReadProc(struct rx_call *call, char *buf,
 	     * number of bytes read in the length field of the packet struct.  On
 	     * the final portion of a received packet, it's almost certain that
 	     * call->nLeft will be smaller than the final buffer. */
-	    while (nbytes && cp) {
+	    while (nbytes && call->currentPacket) {
 		t = MIN((int)call->curlen, nbytes);
 		t = MIN(t, (int)call->nLeft);
 		memcpy(buf, call->curpos, t);
@@ -248,23 +272,24 @@ rxi_ReadProc(struct rx_call *call, char *buf,
 #ifdef RX_TRACK_PACKETS
 		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		    rxi_FreePacket(cp);
-		    cp = call->currentPacket = (struct rx_packet *)0;
+		    rxi_FreePacket(call->currentPacket);
+		    call->currentPacket = NULL;
 		} else if (!call->curlen) {
 		    /* need to get another struct iov */
-		    if (++call->curvec >= cp->niovecs) {
+		    if (++call->curvec >= call->currentPacket->niovecs) {
 			/* current packet is exhausted, get ready for another */
 			/* don't worry about curvec and stuff, they get set somewhere else */
 #ifdef RX_TRACK_PACKETS
 			call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-			rxi_FreePacket(cp);
-			cp = call->currentPacket = (struct rx_packet *)0;
+			rxi_FreePacket(call->currentPacket);
+			call->currentPacket = NULL;
 			call->nLeft = 0;
 		    } else {
 			call->curpos =
-			    (char *)cp->wirevec[call->curvec].iov_base;
-			call->curlen = cp->wirevec[call->curvec].iov_len;
+			    call->currentPacket->wirevec[call->curvec].iov_base;
+			call->curlen =
+			    call->currentPacket->wirevec[call->curvec].iov_len;
 		    }
 		}
 	    }
@@ -282,9 +307,6 @@ int
 rx_ReadProc(struct rx_call *call, char *buf, int nbytes)
 {
     int bytes;
-    int tcurlen;
-    int tnLeft;
-    char *tcurpos;
     SPLVAR;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
@@ -299,15 +321,12 @@ rx_ReadProc(struct rx_call *call, char *buf, int nbytes)
      * Most common case, all of the data is in the current iovec.
      * We are relying on nLeft being zero unless the call is in receive mode.
      */
-    tcurlen = call->curlen;
-    tnLeft = call->nLeft;
-    if (!call->error && tcurlen > nbytes && tnLeft > nbytes) {
-	tcurpos = call->curpos;
-        memcpy(buf, tcurpos, nbytes);
+    if (!call->error && call->curlen > nbytes && call->nLeft > nbytes) {
+        memcpy(buf, call->curpos, nbytes);
 
-	call->curpos = tcurpos + nbytes;
-	call->curlen = tcurlen - nbytes;
-	call->nLeft = tnLeft - nbytes;
+	call->curpos += nbytes;
+	call->curlen -= nbytes;
+	call->nLeft  -= nbytes;
 
         if (!call->nLeft && call->currentPacket != NULL) {
             /* out of packet.  Get another one. */
@@ -328,9 +347,6 @@ int
 rx_ReadProc32(struct rx_call *call, afs_int32 * value)
 {
     int bytes;
-    int tcurlen;
-    int tnLeft;
-    char *tcurpos;
     SPLVAR;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
@@ -345,17 +361,15 @@ rx_ReadProc32(struct rx_call *call, afs_int32 * value)
      * Most common case, all of the data is in the current iovec.
      * We are relying on nLeft being zero unless the call is in receive mode.
      */
-    tcurlen = call->curlen;
-    tnLeft = call->nLeft;
-    if (!call->error && tcurlen >= sizeof(afs_int32)
-	&& tnLeft >= sizeof(afs_int32)) {
-	tcurpos = call->curpos;
+    if (!call->error && call->curlen >= sizeof(afs_int32)
+	&& call->nLeft >= sizeof(afs_int32)) {
 
-        memcpy((char *)value, tcurpos, sizeof(afs_int32));
+        memcpy((char *)value, call->curpos, sizeof(afs_int32));
+
+        call->curpos += sizeof(afs_int32);
+	call->curlen -= sizeof(afs_int32);
+	call->nLeft  -= sizeof(afs_int32);
 
-        call->curpos = tcurpos + sizeof(afs_int32);
-	call->curlen = (u_short)(tcurlen - sizeof(afs_int32));
-	call->nLeft = (u_short)(tnLeft - sizeof(afs_int32));
         if (!call->nLeft && call->currentPacket != NULL) {
             /* out of packet.  Get another one. */
             rxi_FreePacket(call->currentPacket);
@@ -382,91 +396,32 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 {
     int didConsume = 0;
     int didHardAck = 0;
+    int code;
     unsigned int t;
-    struct rx_packet *rp;
-    struct rx_packet *curp;
     struct iovec *call_iov;
     struct iovec *cur_iov = NULL;
 
-    curp = call->currentPacket;
-    if (curp) {
-	cur_iov = &curp->wirevec[call->curvec];
+    if (call->currentPacket) {
+	cur_iov = &call->currentPacket->wirevec[call->curvec];
     }
     call_iov = &call->iov[call->iovNext];
 
     while (!call->error && call->iovNBytes && call->iovNext < call->iovMax) {
 	if (call->nLeft == 0) {
 	    /* Get next packet */
-	    if (queue_IsNotEmpty(&call->rq)) {
-		/* Check that next packet available is next in sequence */
-		rp = queue_First(&call->rq, rx_packet);
-		if (rp->header.seq == call->rnext) {
-		    afs_int32 error;
-		    struct rx_connection *conn = call->conn;
-		    queue_Remove(rp);
-#ifdef RX_TRACK_PACKETS
-		    rp->flags &= ~RX_PKTFLAG_RQ;
-#endif
-#ifdef RXDEBUG_PACKET
-                    call->rqc--;
-#endif /* RXDEBUG_PACKET */
-
-		    /* RXS_CheckPacket called to undo RXS_PreparePacket's
-		     * work.  It may reduce the length of the packet by up
-		     * to conn->maxTrailerSize, to reflect the length of the
-		     * data + the header. */
-		    if ((error =
-			 RXS_CheckPacket(conn->securityObject, call, rp))) {
-			/* Used to merely shut down the call, but now we
-			 * shut down the whole connection since this may
-			 * indicate an attempt to hijack it */
-
-			MUTEX_EXIT(&call->lock);
-			rxi_ConnectionError(conn, error);
-			MUTEX_ENTER(&conn->conn_data_lock);
-			rp = rxi_SendConnectionAbort(conn, rp, 0, 0);
-			MUTEX_EXIT(&conn->conn_data_lock);
-			rxi_FreePacket(rp);
-			MUTEX_ENTER(&call->lock);
-
-			return 1;
-		    }
-		    call->rnext++;
-		    curp = call->currentPacket = rp;
-#ifdef RX_TRACK_PACKETS
-		    call->currentPacket->flags |= RX_PKTFLAG_CP;
-#endif
-		    call->curvec = 1;	/* 0th vec is always header */
-		    cur_iov = &curp->wirevec[1];
-		    /* begin at the beginning [ more or less ], continue
-		     * on until the end, then stop. */
-		    call->curpos =
-			(char *)curp->wirevec[1].iov_base +
-			call->conn->securityHeaderSize;
-		    call->curlen =
-			curp->wirevec[1].iov_len -
-			call->conn->securityHeaderSize;
-
-		    /* Notice that this code works correctly if the data
-		     * size is 0 (which it may be--no reply arguments from
-		     * server, for example).  This relies heavily on the
-		     * fact that the code below immediately frees the packet
-		     * (no yields, etc.).  If it didn't, this would be a
-		     * problem because a value of zero for call->nLeft
-		     * normally means that there is no read packet */
-		    call->nLeft = curp->length;
-		    hadd32(call->bytesRcvd, curp->length);
-
-		    /* Send a hard ack for every rxi_HardAckRate+1 packets
-		     * consumed. Otherwise schedule an event to send
-		     * the hard ack later on.
-		     */
-		    call->nHardAcks++;
-		    didConsume = 1;
-		    continue;
-		}
+	    code = rxi_GetNextPacket(call);
+	    if (code) {
+		MUTEX_ENTER(&call->lock);
+	        return 1;
+	    }
+
+	    if (call->currentPacket) {
+		cur_iov = &call->currentPacket->wirevec[1];
+		didConsume = 1;
+		continue;
+	    } else {
+		break;
 	    }
-	    break;
 	}
 
 	/* It's possible for call->nLeft to be smaller than any particular
@@ -475,7 +430,9 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 	 * number of bytes read in the length field of the packet struct.  On
 	 * the final portion of a received packet, it's almost certain that
 	 * call->nLeft will be smaller than the final buffer. */
-	while (call->iovNBytes && call->iovNext < call->iovMax && curp) {
+	while (call->iovNBytes
+	       && call->iovNext < call->iovMax
+	       && call->currentPacket) {
 
 	    t = MIN((int)call->curlen, call->iovNBytes);
 	    t = MIN(t, (int)call->nLeft);
@@ -491,28 +448,28 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 	    if (!call->nLeft) {
 		/* out of packet.  Get another one. */
 #ifdef RX_TRACK_PACKETS
-                curp->flags &= ~RX_PKTFLAG_CP;
-                curp->flags |= RX_PKTFLAG_IOVQ;
+                call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+                call->currentPacket->flags |= RX_PKTFLAG_IOVQ;
 #endif
-		queue_Append(&call->iovq, curp);
+		queue_Append(&call->iovq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                 call->iovqc++;
 #endif /* RXDEBUG_PACKET */
-		curp = call->currentPacket = (struct rx_packet *)0;
+		call->currentPacket = NULL;
 	    } else if (!call->curlen) {
 		/* need to get another struct iov */
-		if (++call->curvec >= curp->niovecs) {
+		if (++call->curvec >= call->currentPacket->niovecs) {
 		    /* current packet is exhausted, get ready for another */
 		    /* don't worry about curvec and stuff, they get set somewhere else */
 #ifdef RX_TRACK_PACKETS
-		    curp->flags &= ~RX_PKTFLAG_CP;
-		    curp->flags |= RX_PKTFLAG_IOVQ;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags |= RX_PKTFLAG_IOVQ;
 #endif
-		    queue_Append(&call->iovq, curp);
+		    queue_Append(&call->iovq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                     call->iovqc++;
 #endif /* RXDEBUG_PACKET */
-		    curp = call->currentPacket = (struct rx_packet *)0;
+		    call->currentPacket = NULL;
 		    call->nLeft = 0;
 		} else {
 		    cur_iov++;
@@ -639,7 +596,6 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 	      int nbytes)
 {
     struct rx_connection *conn = call->conn;
-    struct rx_packet *cp = call->currentPacket;
     unsigned int t;
     int requestCount = nbytes;
 
@@ -655,12 +611,12 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 	if ((conn->type == RX_SERVER_CONNECTION)
 	    && (call->mode == RX_MODE_RECEIVING)) {
 	    call->mode = RX_MODE_SENDING;
-	    if (cp) {
+	    if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		cp->flags &= ~RX_PKTFLAG_CP;
+		call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		rxi_FreePacket(cp);
-		cp = call->currentPacket = (struct rx_packet *)0;
+		rxi_FreePacket(call->currentPacket);
+		call->currentPacket = NULL;
 		call->nLeft = 0;
 		call->nFree = 0;
 	    }
@@ -676,50 +632,44 @@ rxi_WriteProc(struct rx_call *call, char *buf,
     do {
 	if (call->nFree == 0) {
 	    MUTEX_ENTER(&call->lock);
-            cp = call->currentPacket;
             if (call->error)
                 call->mode = RX_MODE_ERROR;
-	    if (!call->error && cp) {
-                /* Clear the current packet now so that if
-                 * we are forced to wait and drop the lock
-                 * the packet we are planning on using
-                 * cannot be freed.
-                 */
-#ifdef RX_TRACK_PACKETS
-                cp->flags &= ~RX_PKTFLAG_CP;
-#endif
-		call->currentPacket = (struct rx_packet *)0;
+	    if (!call->error && call->currentPacket) {
 		clock_NewTime();	/* Bogus:  need new time package */
 		/* The 0, below, specifies that it is not the last packet:
 		 * there will be others. PrepareSendPacket may
 		 * alter the packet length by up to
 		 * conn->securityMaxTrailerSize */
-		hadd32(call->bytesSent, cp->length);
-		rxi_PrepareSendPacket(call, cp, 0);
+		hadd32(call->bytesSent, call->currentPacket->length);
+		rxi_PrepareSendPacket(call, call->currentPacket, 0);
 #ifdef	AFS_GLOBAL_RXLOCK_KERNEL
                 /* PrepareSendPacket drops the call lock */
                 rxi_WaitforTQBusy(call);
 #endif /* AFS_GLOBAL_RXLOCK_KERNEL */
 #ifdef RX_TRACK_PACKETS
-		cp->flags |= RX_PKTFLAG_TQ;
+		call->currentPacket->flags |= RX_PKTFLAG_TQ;
 #endif
-		queue_Append(&call->tq, cp);
+		queue_Append(&call->tq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                 call->tqc++;
 #endif /* RXDEBUG_PACKET */
-                cp = (struct rx_packet *)0;
+#ifdef RX_TRACK_PACKETS
+                call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+#endif
+                call->currentPacket = NULL;
+
 		/* If the call is in recovery, let it exhaust its current
 		 * retransmit queue before forcing it to send new packets
 		 */
 		if (!(call->flags & (RX_CALL_FAST_RECOVER))) {
 		    rxi_Start(call, 0);
 		}
-	    } else if (cp) {
+	    } else if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		cp->flags &= ~RX_PKTFLAG_CP;
+		call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		rxi_FreePacket(cp);
-		cp = call->currentPacket = (struct rx_packet *)0;
+		rxi_FreePacket(call->currentPacket);
+		call->currentPacket = NULL;
 	    }
 	    /* Wait for transmit window to open up */
 	    while (!call->error
@@ -743,28 +693,28 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 		}
 #endif /* RX_ENABLE_LOCKS */
 	    }
-	    if ((cp = rxi_AllocSendPacket(call, nbytes))) {
+	    if ((call->currentPacket = rxi_AllocSendPacket(call, nbytes))) {
 #ifdef RX_TRACK_PACKETS
-		cp->flags |= RX_PKTFLAG_CP;
+		call->currentPacket->flags |= RX_PKTFLAG_CP;
 #endif
-		call->currentPacket = cp;
-		call->nFree = cp->length;
+		call->nFree = call->currentPacket->length;
 		call->curvec = 1;	/* 0th vec is always header */
 		/* begin at the beginning [ more or less ], continue
 		 * on until the end, then stop. */
 		call->curpos =
-		    (char *)cp->wirevec[1].iov_base +
+		    (char *) call->currentPacket->wirevec[1].iov_base +
 		    call->conn->securityHeaderSize;
 		call->curlen =
-		    cp->wirevec[1].iov_len - call->conn->securityHeaderSize;
+		    call->currentPacket->wirevec[1].iov_len -
+		    call->conn->securityHeaderSize;
 	    }
 	    if (call->error) {
                 call->mode = RX_MODE_ERROR;
-		if (cp) {
+		if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		    cp->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		    rxi_FreePacket(cp);
+		    rxi_FreePacket(call->currentPacket);
 		    call->currentPacket = NULL;
 		}
 		MUTEX_EXIT(&call->lock);
@@ -773,18 +723,19 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 	    MUTEX_EXIT(&call->lock);
 	}
 
-	if (cp && (int)call->nFree < nbytes) {
+	if (call->currentPacket && (int)call->nFree < nbytes) {
 	    /* Try to extend the current buffer */
 	    int len, mud;
-	    len = cp->length;
+	    len = call->currentPacket->length;
 	    mud = rx_MaxUserDataSize(call);
 	    if (mud > len) {
 		int want;
 		want = MIN(nbytes - (int)call->nFree, mud - len);
-		rxi_AllocDataBuf(cp, want, RX_PACKET_CLASS_SEND_CBUF);
-		if (cp->length > (unsigned)mud)
-		    cp->length = mud;
-		call->nFree += (cp->length - len);
+		rxi_AllocDataBuf(call->currentPacket, want,
+				 RX_PACKET_CLASS_SEND_CBUF);
+		if (call->currentPacket->length > (unsigned)mud)
+		    call->currentPacket->length = mud;
+		call->nFree += (call->currentPacket->length - len);
 	    }
 	}
 
@@ -792,7 +743,7 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 	 * and return.  Don't ship a buffer that's full immediately to
 	 * the peer--we don't know if it's the last buffer yet */
 
-	if (!cp) {
+	if (!call->currentPacket) {
 	    call->nFree = 0;
 	}
 
@@ -809,12 +760,14 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 
 	    if (!call->curlen) {
 		/* need to get another struct iov */
-		if (++call->curvec >= cp->niovecs) {
+		if (++call->curvec >= call->currentPacket->niovecs) {
 		    /* current packet is full, extend or send it */
 		    call->nFree = 0;
 		} else {
-		    call->curpos = (char *)cp->wirevec[call->curvec].iov_base;
-		    call->curlen = cp->wirevec[call->curvec].iov_len;
+		    call->curpos =
+			call->currentPacket->wirevec[call->curvec].iov_base;
+		    call->curlen =
+			call->currentPacket->wirevec[call->curvec].iov_len;
 		}
 	    }
 	}			/* while bytes to send and room to send them */
@@ -924,7 +877,7 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 		int nbytes)
 {
     struct rx_connection *conn = call->conn;
-    struct rx_packet *cp = call->currentPacket;
+    struct rx_packet *cp;
     int requestCount;
     int nextio;
     /* Temporary values, real work is done in rxi_WritevProc */
@@ -948,12 +901,12 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 	if ((conn->type == RX_SERVER_CONNECTION)
 	    && (call->mode == RX_MODE_RECEIVING)) {
 	    call->mode = RX_MODE_SENDING;
-	    if (cp) {
+	    if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		cp->flags &= ~RX_PKTFLAG_CP;
+		call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		rxi_FreePacket(cp);
-		cp = call->currentPacket = (struct rx_packet *)0;
+		rxi_FreePacket(call->currentPacket);
+		call->currentPacket = NULL;
 		call->nLeft = 0;
 		call->nFree = 0;
 	    }
@@ -967,6 +920,7 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
     tcurvec = call->curvec;
     tcurpos = call->curpos;
     tcurlen = call->curlen;
+    cp = call->currentPacket;
     do {
 	int t;
 
@@ -1061,7 +1015,6 @@ rx_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 int
 rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 {
-    struct rx_packet *cp = NULL;
 #ifdef RX_TRACK_PACKETS
     struct rx_packet *p, *np;
 #endif
@@ -1084,21 +1037,20 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 #ifdef AFS_GLOBAL_RXLOCK_KERNEL
     rxi_WaitforTQBusy(call);
 #endif /* AFS_GLOBAL_RXLOCK_KERNEL */
-    cp = call->currentPacket;
 
     if (call->error) {
         call->mode = RX_MODE_ERROR;
 	MUTEX_EXIT(&call->lock);
-	if (cp) {
+	if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-            cp->flags &= ~RX_PKTFLAG_CP;
-            cp->flags |= RX_PKTFLAG_IOVQ;
+            call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+            call->currentPacket->flags |= RX_PKTFLAG_IOVQ;
 #endif
-	    queue_Prepend(&call->iovq, cp);
+	    queue_Prepend(&call->iovq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
             call->iovqc++;
 #endif /* RXDEBUG_PACKET */
-	    call->currentPacket = (struct rx_packet *)0;
+	    call->currentPacket = NULL;
 	}
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
@@ -1118,23 +1070,23 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
     tmpqc = 0;
 #endif /* RXDEBUG_PACKET */
     do {
-	if (call->nFree == 0 && cp) {
+	if (call->nFree == 0 && call->currentPacket) {
 	    clock_NewTime();	/* Bogus:  need new time package */
 	    /* The 0, below, specifies that it is not the last packet:
 	     * there will be others. PrepareSendPacket may
 	     * alter the packet length by up to
 	     * conn->securityMaxTrailerSize */
-	    hadd32(call->bytesSent, cp->length);
-	    rxi_PrepareSendPacket(call, cp, 0);
+	    hadd32(call->bytesSent, call->currentPacket->length);
+	    rxi_PrepareSendPacket(call, call->currentPacket, 0);
 #ifdef	AFS_GLOBAL_RXLOCK_KERNEL
             /* PrepareSendPacket drops the call lock */
             rxi_WaitforTQBusy(call);
 #endif /* AFS_GLOBAL_RXLOCK_KERNEL */
-	    queue_Append(&tmpq, cp);
+	    queue_Append(&tmpq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
             tmpqc++;
 #endif /* RXDEBUG_PACKET */
-            cp = call->currentPacket = (struct rx_packet *)0;
+            call->currentPacket = NULL;
 
 	    /* The head of the iovq is now the current packet */
 	    if (nbytes) {
@@ -1147,25 +1099,23 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
                         rxi_FreePackets(0, &tmpq);
 		    return 0;
 		}
-		cp = queue_First(&call->iovq, rx_packet);
-		queue_Remove(cp);
+		call->currentPacket = queue_First(&call->iovq, rx_packet);
+		queue_Remove(call->currentPacket);
 #ifdef RX_TRACK_PACKETS
-                cp->flags &= ~RX_PKTFLAG_IOVQ;
+                call->currentPacket->flags &= ~RX_PKTFLAG_IOVQ;
+		call->currentPacket->flags |= RX_PKTFLAG_CP;
 #endif
 #ifdef RXDEBUG_PACKET
                 call->iovqc--;
 #endif /* RXDEBUG_PACKET */
-#ifdef RX_TRACK_PACKETS
-                cp->flags |= RX_PKTFLAG_CP;
-#endif
-		call->currentPacket = cp;
-		call->nFree = cp->length;
+		call->nFree = call->currentPacket->length;
 		call->curvec = 1;
 		call->curpos =
-		    (char *)cp->wirevec[1].iov_base +
+		    (char *) call->currentPacket->wirevec[1].iov_base +
 		    call->conn->securityHeaderSize;
 		call->curlen =
-		    cp->wirevec[1].iov_len - call->conn->securityHeaderSize;
+		    call->currentPacket->wirevec[1].iov_len -
+		    call->conn->securityHeaderSize;
 	    }
 	}
 
@@ -1175,15 +1125,15 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 		|| iov[nextio].iov_len > (int)call->curlen) {
 		call->error = RX_PROTOCOL_ERROR;
                 MUTEX_EXIT(&call->lock);
-		if (cp) {
+		if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		    cp->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-                    queue_Prepend(&tmpq, cp);
+                    queue_Prepend(&tmpq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                     tmpqc++;
 #endif /* RXDEBUG_PACKET */
-                    cp = call->currentPacket = (struct rx_packet *)0;
+                    call->currentPacket = NULL;
 		}
 #ifdef RXDEBUG_PACKET
                 tmpqc -=
@@ -1197,11 +1147,13 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 	    call->nFree -= iov[nextio].iov_len;
 	    nextio++;
 	    if (call->curlen == 0) {
-		if (++call->curvec > cp->niovecs) {
+		if (++call->curvec > call->currentPacket->niovecs) {
 		    call->nFree = 0;
 		} else {
-		    call->curpos = (char *)cp->wirevec[call->curvec].iov_base;
-		    call->curlen = cp->wirevec[call->curvec].iov_len;
+		    call->curpos =
+			call->currentPacket->wirevec[call->curvec].iov_base;
+		    call->curlen =
+			call->currentPacket->wirevec[call->curvec].iov_len;
 		}
 	    }
 	}
@@ -1242,18 +1194,15 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 	call->startWait = 0;
     }
 
-    /* cp is no longer valid since we may have given up the lock */
-    cp = call->currentPacket;
-
     if (call->error) {
         call->mode = RX_MODE_ERROR;
         call->currentPacket = NULL;
         MUTEX_EXIT(&call->lock);
-	if (cp) {
+	if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-	    cp->flags &= ~RX_PKTFLAG_CP;
+	    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-	    rxi_FreePacket(cp);
+	    rxi_FreePacket(call->currentPacket);
 	}
 	return 0;
     }
