@@ -28,11 +28,10 @@
  */
 
 #include <afsconfig.h>
-#ifdef KERNEL
-#include "afs/param.h"
-#else
 #include <afs/param.h>
-#include <string.h>
+
+#ifndef KERNEL
+# include <roken.h>
 #endif
 
 
@@ -49,14 +48,10 @@
 #ifndef	NeXT
 
 #if defined(KERNEL) && !defined(UKERNEL)
-#if !defined(AFS_LINUX26_ENV)
 #include <sys/param.h>
-#endif
 #ifndef AFS_LINUX20_ENV
 #include <sys/systm.h>
 #endif
-#else
-#include <stdio.h>
 #endif
 #include "xdr.h"
 #include "rx.h"
@@ -410,28 +405,10 @@ xdr_bytes(XDR * xdrs, char **cpp, u_int * sizep,
     /*
      * first deal with the length since xdr bytes are counted
      */
-
-    if (xdrs->x_op == XDR_DECODE && sp != NULL) {
-	/*
-	 * We've been given a preallocated array to decode into. Before we
-	 * modify *sizep, check that we have enough space to fit the elements
-	 * that follow.
-	 */
-	if (!xdr_u_int(xdrs, &nodesize)) {
-	   return FALSE;
-	}
-	if (nodesize > *sizep) {
-	   return FALSE;
-	}
-	*sizep = nodesize;
-
-    } else {
-	if (!xdr_u_int(xdrs, sizep)) {
-	    return (FALSE);
-	}
-	nodesize = *sizep;
+    if (!xdr_u_int(xdrs, sizep)) {
+	return (FALSE);
     }
-
+    nodesize = *sizep;
     if ((nodesize > maxsize) && (xdrs->x_op != XDR_FREE)) {
 	return (FALSE);
     }
@@ -555,7 +532,7 @@ xdr_string(XDR * xdrs, char **cpp, u_int maxsize)
     if (!xdr_u_int(xdrs, &size)) {
 	return (FALSE);
     }
-    if (xdrs->x_op != XDR_FREE && size > maxsize) {
+    if (size > maxsize) {
 	return (FALSE);
     }
     nodesize = size + 1;
@@ -566,40 +543,13 @@ xdr_string(XDR * xdrs, char **cpp, u_int maxsize)
     switch (xdrs->x_op) {
 
     case XDR_DECODE:
-	if (sp != NULL) {
-	    /*
-	     * Refuse to use preallocated strings, since we cannot verify
-	     * whether enough memory has been allocated to handle the decoded
-	     * string.
-	     */
-	    return FALSE;
-	}
-
-	*cpp = sp = (char *)osi_alloc(nodesize);
+	if (sp == NULL)
+	    *cpp = sp = (char *)osi_alloc(nodesize);
 	if (sp == NULL) {
 	    return (FALSE);
 	}
 	sp[size] = 0;
-
-	/* Get the actual string. */
-	if (!xdr_opaque(xdrs, sp, size)) {
-	    /* Make sure strlen(sp) == size, so we can calculate the correct
-	     * size for osi_free when freeing the string. */
-	    memset(sp, 'z', size);
-	    return FALSE;
-	}
-
-	/*
-	 * If the string contains a '\0' character, the string is invalid.
-	 * Don't allow this, because this makes it impossible for us to pass
-	 * the correct size to osi_free later on, when freeing the string.
-	 */
-	if (strlen(sp) != size) {
-	    /* Make sure strlen(sp) == size. */
-	    memset(sp, 'z', size);
-	    return FALSE;
-	}
-	return TRUE;
+	/* fall into ... */
 
     case XDR_ENCODE:
 	return (xdr_opaque(xdrs, sp, size));
