@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,34 +9,12 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <sys/types.h>
-#include <errno.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <sys/file.h>
-#include <netdb.h>
-#include <arpa/inet.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <string.h>
-#include <sys/stat.h>
-#include <afs/afsutil.h>
 #include <afs/stds.h>
-#include <afs/cmd.h>
 
-#include <stdio.h>
+#include <roken.h>
+
+#include <afs/afsutil.h>
+#include <afs/cmd.h>
 
 #include <rx/rx_user.h>
 #include <rx/rx_clock.h>
@@ -50,8 +28,8 @@
 static short
 PortNumber(char *aport)
 {
-    register int tc;
-    register short total;
+    int tc;
+    short total;
 
     total = 0;
     while ((tc = *aport++)) {
@@ -66,7 +44,7 @@ PortNumber(char *aport)
 static short
 PortName(char *aname)
 {
-    register struct servent *ts;
+    struct servent *ts;
     ts = getservbyname(aname, NULL);
     if (!ts)
 	return -1;
@@ -76,7 +54,7 @@ PortName(char *aname)
 int
 MainCommand(struct cmd_syndesc *as, void *arock)
 {
-    register int i;
+    int i;
     osi_socket s;
     int j;
     struct sockaddr_in taddr;
@@ -84,7 +62,7 @@ MainCommand(struct cmd_syndesc *as, void *arock)
     struct in_addr hostAddr;
     short port;
     struct hostent *th;
-    register afs_int32 code;
+    afs_int32 code;
     int nodally;
     int allconns;
     int rxstats;
@@ -209,7 +187,15 @@ MainCommand(struct cmd_syndesc *as, void *arock)
     hostAddr.s_addr = host;
     afs_inet_ntoa_r(hostAddr.s_addr, hoststr);
     printf("Trying %s (port %d):\n", hoststr, ntohs(port));
-    s = socket(AF_INET, SOCK_DGRAM, 0);
+    s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    if (s == OSI_NULLSOCKET) {
+#ifdef AFS_NT40_ENV
+        fprintf(stderr, "socket() failed with error %u\n", WSAGetLastError());
+#else
+	perror("socket");
+#endif
+	exit(1);
+    }
     taddr.sin_family = AF_INET;
     taddr.sin_port = 0;
     taddr.sin_addr.s_addr = 0;
@@ -218,7 +204,11 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 #endif
     code = bind(s, (struct sockaddr *)&taddr, sizeof(struct sockaddr_in));
     if (code) {
+#ifdef AFS_NT40_ENV
+        fprintf(stderr, "bind() failed with error %u\n", WSAGetLastError());
+#else
 	perror("bind");
+#endif
 	exit(1);
     }
 
@@ -255,7 +245,7 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 
     if (withPackets)
         printf("Free packets: %d/%d, packet reclaims: %d, calls: %d, used FDs: %d\n",
-               tstats.nFreePackets, tstats.nPackets, tstats.packetReclaims, 
+               tstats.nFreePackets, tstats.nPackets, tstats.packetReclaims,
                tstats.callsExecuted, tstats.usedFDs);
     else
         printf("Free packets: %d, packet reclaims: %d, calls: %d, used FDs: %d\n",
@@ -278,21 +268,20 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 	    fprintf(stderr,
 		    "WARNING: Server doesn't support retrieval of Rx statistics\n");
 	} else {
-	    union {
-		struct rx_statistics rxstats;
-		struct rx_debugIn debug;
-	    } packet;
+	    struct rx_statistics rxstats;
 
 	    /* should gracefully handle the case where rx_stats grows */
 	    code =
-		rx_GetServerStats(s, host, port, &packet.rxstats,
+		rx_GetServerStats(s, host, port, &rxstats,
 				  &supportedStatValues);
 	    if (code < 0) {
 		printf("rxstats call failed with code %d\n", code);
 		exit(1);
 	    }
-	    if (code != sizeof(packet.rxstats)) {
-		if (packet.debug.type == RX_DEBUGI_BADTYPE)
+	    if (code != sizeof(rxstats)) {
+		struct rx_debugIn debug;
+		memcpy(&debug, &rxstats, sizeof(debug));
+		if (debug.type == RX_DEBUGI_BADTYPE)
 		    goto noRxStats;
 		printf
 		    ("WARNING: returned Rx statistics of unexpected size (got %d)\n",
@@ -300,7 +289,7 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 		/* handle other versions?... */
 	    }
 
-	    rx_PrintTheseStats(stdout, &packet.rxstats, sizeof(packet.rxstats),
+	    rx_PrintTheseStats(stdout, &rxstats, sizeof(rxstats),
 			       tstats.nFreePackets, tstats.version);
 	}
     }
@@ -497,6 +486,8 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 		    printf("dally, ");
 		else if (tconn.callState[j] == RX_STATE_HOLD)
 		    printf("hold, ");
+		else if (tconn.callState[j] == RX_STATE_RESET)
+		    printf("reset, ");
 		printf("mode: ");
 		if (tconn.callMode[j] == RX_MODE_SENDING)
 		    printf("sending");
@@ -558,7 +549,7 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 	    /* now display the peer */
 	    hostAddr.s_addr = tpeer.host;
 	    afs_inet_ntoa_r(hostAddr.s_addr, hoststr);
-	    printf("Peer at host %s, port %hu\n", hoststr, 
+	    printf("Peer at host %s, port %hu\n", hoststr,
 		   ntohs(tpeer.port));
 	    printf("\tifMTU %hu\tnatMTU %hu\tmaxMTU %hu\n", tpeer.ifMTU,
 		   tpeer.natMTU, tpeer.maxMTU);
