@@ -15,30 +15,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <errno.h>
 #include <limits.h>
-#include <sys/types.h>
+
 #include <afs/vice.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netdb.h>
-#include <netinet/in.h>
-#include <sys/file.h>
-#endif
-#include <sys/stat.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_GRP_H
-#include <grp.h>
-#endif
 #include <rx/xdr.h>
-#include <afs/afsutil.h>
+
 #include "rmtsys.h"
 #include "sys_prototypes.h"
 
@@ -74,7 +57,7 @@ GetAfsServerAddr(char *syscall)
     if (!(afs_server = getenv("AFSSERVER"))) {
 	char *home_dir;
 	FILE *fp;
-	int len = 0;
+	int len;
 
 	if (!(home_dir = getenv("HOME"))) {
 	    /* Our last chance is the "/.AFSSERVER" file */
@@ -82,15 +65,13 @@ GetAfsServerAddr(char *syscall)
 	    if (fp == 0) {
 		return 0;
 	    }
+	    fgets(server_name, 128, fp);
+	    fclose(fp);
 	} else {
-	    char *pathname;
+	    char pathname[256];
 
-	    afs_asprintf(&pathname, "%s/%s", home_dir, ".AFSSERVER");
-	    if (pathname == NULL)
-		return 0;
+	    sprintf(pathname, "%s/%s", home_dir, ".AFSSERVER");
 	    fp = fopen(pathname, "r");
-	    free(pathname);
-
 	    if (fp == 0) {
 		/* Our last chance is the "/.AFSSERVER" file */
 		fp = fopen("/.AFSSERVER", "r");
@@ -98,10 +79,10 @@ GetAfsServerAddr(char *syscall)
 		    return 0;
 		}
 	    }
+	    fgets(server_name, 128, fp);
+	    fclose(fp);
 	}
-	if (fgets(server_name, 128, fp) != NULL)
-	    len = strlen(server_name);
-	fclose(fp);
+	len = strlen(server_name);
 	if (len == 0) {
 	    return 0;
 	}
@@ -245,7 +226,7 @@ pioctl(char *path, afs_int32 cmd, struct ViceIoctl *data, afs_int32 follow)
     inparam_conversion(cmd, InData.rmtbulk_val, 0);
 
     OutData.rmtbulk_len = MAXBUFFERLEN * sizeof(*OutData.rmtbulk_val);
-    OutData.rmtbulk_val = calloc(1, OutData.rmtbulk_len);
+    OutData.rmtbulk_val = malloc(OutData.rmtbulk_len);
     if (!OutData.rmtbulk_val) {
 	free(inbuffer);
 	return -1;
@@ -291,7 +272,7 @@ pioctl(char *path, afs_int32 cmd, struct ViceIoctl *data, afs_int32 follow)
 	    errno = EINVAL;
 	    errorcode = -1;
 	} else {
-	    memcpy(data->out, OutData.rmtbulk_val, OutData.rmtbulk_len);
+	    memcpy(data->out, OutData.rmtbulk_val, data->out_size);
 	    outparam_conversion(cmd, data->out, 1);
 	}
     }
