@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,18 +14,17 @@
 /*
  * --------------------- Required definitions ---------------------
  */
-#include <stdlib.h>
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/uss/uss.c,v 1.8.14.5 2009/03/16 13:43:43 shadow Exp $");
+#include <roken.h>
 
-#ifdef	AFS_AIX32_ENV
-#include <signal.h>
-#endif
-
-#include <string.h>
+#include <afs/cmd.h>		/*Command line parsing */
+#include <afs/cellconfig.h>	/*Cell config defs */
+#include <afs/kautils.h>	/*MAXKTCREALMLEN & MAXKTCNAMELEN */
+#include <afs/pterror.h>
+#include <afs/vlserver.h>
+#include <ubik.h>
 
 #include "uss_common.h"		/*Common uss definitions, globals */
 #include "uss_procs.h"		/*Main uss operations */
@@ -34,12 +33,6 @@ RCSID
 #include "uss_ptserver.h"
 #include "uss_vol.h"
 #include "uss_acl.h"
-#include <afs/cmd.h>		/*Command line parsing */
-#include <afs/cellconfig.h>	/*Cell config defs */
-#include <afs/kautils.h>	/*MAXKTCREALMLEN & MAXKTCNAMELEN */
-#include <afs/pterror.h>
-#include <afs/vlserver.h>
-#include <ubik.h>
 
 extern int yylex(void);
 extern int yyparse (void);
@@ -131,7 +124,7 @@ static int DoAdd(void);
  *------------------------------------------------------------------------*/
 
 static int
-GetCommon(register struct cmd_syndesc *a_as, void *arock)
+GetCommon(struct cmd_syndesc *a_as, void *arock)
 {				/*GetCommon */
 
     int code;			/*Result of ka_LocalCell */
@@ -220,7 +213,7 @@ SaveRestoreInfo(void)
 #ifdef USS_DB
     static char rn[] = "uss:SaveRestoreInfo";	/*Routine name */
 #endif
-    register afs_int32 code;	/*Return code */
+    afs_int32 code;	/*Return code */
     afs_int32 deletedUid;	/*Uid to be nuked */
 
     /*
@@ -541,7 +534,7 @@ static int
 DoBulkAddLine(char *a_buf,  char *a_tp)
 {				/*DoBulkAddLine */
 
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
 #ifdef USS_DB
     static char rn[] = "DoBulkAddLine";	/*Routine name */
 #endif
@@ -918,7 +911,7 @@ static int
 DoBulkPurgeVolumeLine(char *a_buf, char *a_tp)
 {				/*DoBulkPurgeVolumeLine */
 
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
     int overflow;		/*Did a field copy overflow happen? */
 
     /*
@@ -1096,7 +1089,7 @@ static int
 DoBulkRestoreLine(char *a_buf, char *a_tp)
 {				/*DoBulkRestoreLine */
 
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
     int overflow;		/*Overflow occur on field copy? */
 
     /*
@@ -1270,7 +1263,7 @@ static int
 DoBulkExecLine(char *a_buf, char *a_tp)
 {				/*DoBulkExecLine */
 
-    register afs_int32 code;	/*Return code */
+    afs_int32 code;	/*Return code */
 
     /*
      * Really, uss_procs_Exec does all the work for us!
@@ -1304,7 +1297,7 @@ DoBulkExecLine(char *a_buf, char *a_tp)
  *------------------------------------------------------------------------*/
 extern int Pipe;
 static int
-HandleBulk(register struct cmd_syndesc *a_as, void *a_rock)
+HandleBulk(struct cmd_syndesc *a_as, void *a_rock)
 {				/*HandleBulk */
 
 #define USS_BULK_CMD_CHARS	 128
@@ -1513,11 +1506,11 @@ HandleBulk(register struct cmd_syndesc *a_as, void *a_rock)
  *------------------------------------------------------------------------*/
 
 static int
-AddUser(register struct cmd_syndesc *a_as, void *a_rock)
+AddUser(struct cmd_syndesc *a_as, void *a_rock)
 {				/*AddUser */
 
     int i;
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
     int code;
 
     /*
@@ -1792,13 +1785,12 @@ int
 main(int argc, char *argv[])
 {				/*Main routine */
 
-    register struct cmd_syndesc *cs;	/*Command line syntax descriptor */
-    register afs_int32 code;	/*Return code */
+    struct cmd_syndesc *cs;	/*Command line syntax descriptor */
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -1848,7 +1840,7 @@ main(int argc, char *argv[])
     cmd_AddParm(cs, "-admin", CMD_SINGLE, CMD_OPTIONAL,
 		"administrator to authenticate");
     cmd_AddParm(cs, "-dryrun", CMD_FLAG, CMD_OPTIONAL,
-		"only list what would be done, don't do it");
+		"list what would be done, don't do it");
     cmd_AddParm(cs, "-skipauth", CMD_FLAG, CMD_OPTIONAL,
 		"ignore all contact with the authentication server (kaserver)");
     cmd_AddParm(cs, "-overwrite", CMD_FLAG, CMD_OPTIONAL,
@@ -1868,7 +1860,7 @@ main(int argc, char *argv[])
     cmd_AddParm(cs, "-admin", CMD_SINGLE, CMD_OPTIONAL,
 		"administrator to authenticate");
     cmd_AddParm(cs, "-dryrun", CMD_FLAG, CMD_OPTIONAL,
-		"only list what would be done, don't do it");
+		"list what would be done, don't do it");
     cmd_AddParm(cs, "-skipauth", CMD_FLAG, CMD_OPTIONAL,
 		"ignore all contact with the authentication server (kaserver)");
     cmd_AddParm(cs, "-overwrite", CMD_FLAG, CMD_OPTIONAL,
@@ -1909,7 +1901,7 @@ main(int argc, char *argv[])
     cmd_AddParm(cs, "-admin", CMD_SINGLE, CMD_OPTIONAL,
 		"administrator to authenticate");
     cmd_AddParm(cs, "-dryrun", CMD_FLAG, CMD_OPTIONAL,
-		"only list what would be done, don't do it");
+		"list what would be done, don't do it");
     cmd_AddParm(cs, "-skipauth", CMD_FLAG, CMD_OPTIONAL,
 		"ignore all contact with the authentication server (kaserver)");
 #if USS_FUTURE_FEATURES
@@ -1929,7 +1921,7 @@ main(int argc, char *argv[])
     cmd_AddParm(cs, "-admin", CMD_SINGLE, CMD_OPTIONAL,
 		"administrator to authenticate");
     cmd_AddParm(cs, "-dryrun", CMD_FLAG, CMD_OPTIONAL,
-		"only list what would be done, don't do it");
+		"list what would be done, don't do it");
     cmd_AddParm(cs, "-skipauth", CMD_FLAG, CMD_OPTIONAL,
 		"ignore all contact with the authentication server (kaserver)");
 #endif /* USS_DONT_HIDE_SOME_FEATURES */
@@ -1962,7 +1954,7 @@ main(int argc, char *argv[])
     cmd_AddParm(cs, "-admin", CMD_SINGLE, CMD_OPTIONAL,
 		"administrator to authenticate");
     cmd_AddParm(cs, "-dryrun", CMD_FLAG, CMD_OPTIONAL,
-		"only list what would be done, don't do it");
+		"list what would be done, don't do it");
     cmd_AddParm(cs, "-skipauth", CMD_FLAG, CMD_OPTIONAL,
 		"ignore all contact with the authentication server (kaserver)");
 #endif /* USS_DONT_HIDE_SOME_FEATURES */
@@ -1980,16 +1972,9 @@ main(int argc, char *argv[])
     /*
      * Execute the parsed command.
      */
-    code = cmd_Dispatch(argc, argv);
-#if 0
-    if (code) {
-	fprintf(stderr, "%s: Call to cmd_Dispatch() failed; code is %d\n",
-		uss_whoami, code);
-	exit(-1);
-    }
-#endif /* 0 */
+    cmd_Dispatch(argc, argv);
     if (doUnlog) {
-	code = uss_fs_UnlogToken(uss_Cell);
+	uss_fs_UnlogToken(uss_Cell);
     }
     return 0;
 }				/*Main routine */
