@@ -18,12 +18,27 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#ifdef AFS_HPUX_ENV
+/* We need the old directory type headers (included below), so don't include
+ * the normal dirent.h, or it will conflict. */
+# undef HAVE_DIRENT_H
+# include <sys/inode.h>
+# define	LONGFILENAMES	1
+# include <sys/sysmacros.h>
+# include <sys/ino.h>
+# define	DIRSIZ_MACRO
+# ifdef HAVE_USR_OLD_USR_INCLUDE_NDIR_H
+#  include </usr/old/usr/include/ndir.h>
+# else
+#  include <ndir.h>
+# endif
+#endif
+
+#include <roken.h>
+
+#include <ctype.h>
 
 #define VICE			/* control whether AFS changes are present */
-#include <stdio.h>
-
-#include <sys/time.h>
-#include <sys/param.h>
 
 #ifdef	AFS_OSF_ENV
 #include <sys/mount.h>
@@ -54,18 +69,7 @@
 #endif
 #else /* AFS_VFSINCL_ENV */
 #include <sys/inode.h>
-#ifdef	AFS_HPUX_ENV
-#include <ctype.h>
-#define	LONGFILENAMES	1
-#include <sys/sysmacros.h>
-#include <sys/ino.h>
-#define	DIRSIZ_MACRO
-#ifdef HAVE_USR_OLD_USR_INCLUDE_NDIR_H
-#include </usr/old/usr/include/ndir.h>
-#else
-#include <ndir.h>
-#endif
-#else
+#ifndef	AFS_HPUX_ENV
 #include <sys/dir.h>
 #endif
 #include <sys/fs.h>
@@ -89,9 +93,9 @@ struct bufarea *pbp = 0;
 
 ckinode(dp, idesc)
      struct dinode *dp;
-     register struct inodesc *idesc;
+     struct inodesc *idesc;
 {
-    register daddr_t *ap;
+    daddr_t *ap;
     long ret, n, ndb, offset;
     struct dinode dino;
     UOFF_T indir_data_blks;
@@ -141,7 +145,7 @@ ckinode(dp, idesc)
 	    return (ret);
     }
     idesc->id_numfrags = sblock.fs_frag;
-#if	defined(AFS_SUN56_ENV)
+#if	defined(AFS_SUN5_ENV)
     /*
      * indir_data_blks determine the no. of data blocks
      * in the previous levels. ie., at level 3 it
@@ -186,15 +190,15 @@ ckinode(dp, idesc)
 
 iblock(idesc, ilevel, isize)
      struct inodesc *idesc;
-     register long ilevel;
+     long ilevel;
      UOFF_T isize;
 {
-    register daddr_t *ap;
-    register daddr_t *aplim;
+    daddr_t *ap;
+    daddr_t *aplim;
     int i, n, (*func) ();
     UOFF_T sizepb;
     OFF_T nif;
-    register struct bufarea *bp;
+    struct bufarea *bp;
     char buf[BUFSIZ];
     extern int dirscan(), pass1check();
 
@@ -208,7 +212,7 @@ iblock(idesc, ilevel, isize)
 	return (SKIP);
     bp = getdatablk(idesc->id_blkno, sblock.fs_bsize);
     ilevel--;
-#if	defined(AFS_SUN56_ENV)
+#if	defined(AFS_SUN5_ENV)
     for (sizepb = 1, i = 0; i < ilevel; i++) {
 	sizepb *= (u_offset_t) NINDIR(&sblock);
     }
@@ -266,7 +270,7 @@ iblock(idesc, ilevel, isize)
 	if (*ap) {
 	    idesc->id_blkno = *ap;
 	    if (ilevel > 0) {
-#if	defined(AFS_SUN56_ENV)
+#if	defined(AFS_SUN5_ENV)
 		n = iblock(idesc, ilevel, isize);
 		/*
 		 * each iteration decrease "remaining block
@@ -297,7 +301,7 @@ chkrange(blk, cnt)
      daddr_t blk;
      int cnt;
 {
-    register int c;
+    int c;
 
     if ((unsigned)(blk + cnt) > maxfsblock)
 	return (1);
@@ -346,7 +350,7 @@ ginode(inumber)
 #ifdef	AFS_SUN5_ENVX
 inocleanup()
 {
-    register struct inoinfo **inpp;
+    struct inoinfo **inpp;
 
     if (inphead == NULL)
 	return;
@@ -365,11 +369,11 @@ inodirty()
 }
 
 clri(idesc, type, flag)
-     register struct inodesc *idesc;
+     struct inodesc *idesc;
      char *type;
      int flag;
 {
-    register struct dinode *dp;
+    struct dinode *dp;
 #if defined(ACLS) && defined(AFS_HPUX_ENV)
     struct inodesc cidesc;
 #endif /* ACLS */
@@ -394,8 +398,8 @@ clri(idesc, type, flag)
 	    n_files--;
 
 	/*
-	 * If there is a CI associated with this inode, we must 
-	 * clear it as well.  
+	 * If there is a CI associated with this inode, we must
+	 * clear it as well.
 	 */
 	if (statemap[idesc->id_number] & HASCINODE) {
 	    if (!(dp->di_contin < ROOTINO || dp->di_contin > maxino))
@@ -419,7 +423,7 @@ clri(idesc, type, flag)
 findname(idesc)
      struct inodesc *idesc;
 {
-    register struct direct *dirp = idesc->id_dirp;
+    struct direct *dirp = idesc->id_dirp;
 
     if (dirp->d_ino != idesc->id_parent)
 	return (KEEPON);
@@ -430,7 +434,7 @@ findname(idesc)
 findino(idesc)
      struct inodesc *idesc;
 {
-    register struct direct *dirp = idesc->id_dirp;
+    struct direct *dirp = idesc->id_dirp;
 
     if (dirp->d_ino == 0)
 	return (KEEPON);
@@ -445,8 +449,8 @@ findino(idesc)
 pinode(ino)
      ino_t ino;
 {
-    register struct dinode *dp;
-    register char *p;
+    struct dinode *dp;
+    char *p;
     struct passwd *pw;
     char *ctime();
     time_t t;
@@ -490,7 +494,7 @@ pinode(ino)
     printf("MODE=%o\n", dp->di_mode);
     if (preen)
 	printf("%s: ", devname);
-#if	defined(AFS_SUN56_ENV)
+#if	defined(AFS_SUN5_ENV)
     printf("SIZE=%" AFS_INT64_FMT " ", dp->di_size);
 #else
     printf("SIZE=%ld ", dp->di_size);
@@ -515,7 +519,7 @@ blkerror(ino, type, blk)
 #endif /* VICE */
     case FSTATE:
 #if defined(ACLS) && defined(AFS_HPUX_ENV)
-	/* 
+	/*
 	 *  Keep the continuation inode info
 	 */
 	if (statemap[ino] & HASCINODE)
@@ -529,7 +533,7 @@ blkerror(ino, type, blk)
 
     case DSTATE:
 #if defined(ACLS) && defined(AFS_HPUX_ENV)
-	/* 
+	/*
 	 *  Keep the continuation inode info
 	 */
 	if (statemap[ino] & HASCINODE)
@@ -559,8 +563,8 @@ allocino(request, type)
      ino_t request;
      int type;
 {
-    register ino_t ino;
-    register struct dinode *dp;
+    ino_t ino;
+    struct dinode *dp;
 
     if (request == 0)
 	request = ROOTINO;
@@ -609,7 +613,7 @@ freeino(ino)
     extern int pass4check();
     struct dinode *dp;
 
-    memset((char *)&idesc, 0, sizeof(struct inodesc));
+    memset(&idesc, 0, sizeof(struct inodesc));
     idesc.id_type = ADDR;
     idesc.id_func = pass4check;
     idesc.id_number = ino;
