@@ -9,34 +9,16 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <sys/types.h>
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
+
+#include <roken.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#include <stdio.h>
-#include <string.h>
 #include <ctype.h>
 
 #include <afs/afsutil.h>
@@ -49,6 +31,7 @@
 #include <afs/cmd.h>
 #include <lock.h>
 #include <ubik.h>
+
 #include "vlserver.h"
 
 void fill_listattributes_entry(struct VldbListByAttributes *, char **, int);
@@ -77,7 +60,7 @@ static char *opcode_names[VL_NUMBER_OPCODESX] = {
     "ListAttributes",
     "LinkedList",
     "GetStats",
-    "Probe",	/* RPC is ProbeServer */
+    "Probe",
     "GetAddrs",
     "ChangeAddr",
     "CreateEntryN",
@@ -220,42 +203,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 	exit(1);
     }
 
-    if (as->parms[6].items) {	/* -probe */
-	int nconns = 0;
-	int status = 0;
-	int i;
-	char hoststr[16];
-	afs_uint32 addr;
-
-	for (i = 0; cstruct->conns[i]; i++, nconns++) {
-	    rx_SetConnDeadTime(cstruct->conns[i], 6);
-	    addr = rx_HostOf(rx_PeerOf(cstruct->conns[i]));
-	    if (!nconns) {
-		printf("probing");
-	    }
-	    printf(" %s", afs_inet_ntoa_r(addr, hoststr));
-	}
-	if (nconns == 0) {
-	    printf("no connections\n");
-	    return 255;
-	}
-	printf("\n");
-	multi_Rx(cstruct->conns, nconns) {
-	    multi_VL_ProbeServer();
-	    addr = rx_HostOf(rx_PeerOf(cstruct->conns[multi_i]));
-	    if (!multi_error) {
-		printf(" ok: %s\n", afs_inet_ntoa_r(addr, hoststr));
-	    } else {
-		status = 255;
-		printf(" not ok (%d): %s\n", multi_error,
-		       afs_inet_ntoa_r(addr, hoststr));
-	    }
-	}
-	multi_End;
-	return status;
-    }
-
-    if (as->parms[5].items) {	/* -getstats */
+    if (as->parms[5].items) {	/* -gstats */
 	vldstats stats;
 	vital_vlheader vital_header;
 	code = ubik_VL_GetStats(cstruct, 0, &stats, &vital_header);
@@ -584,9 +532,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    printf("VL_ListAttributes returned code = %d\n", code);
 		    continue;
 		}
-		if (entries.bulkentries_len < nentries) {
-		    nentries = entries.bulkentries_len;
-		}
 		entry = (struct vldbentry *)entries.bulkentries_val;
 		for (i = 0; i < nentries; i++, entry++)
 		    display_entry(entry, 0);
@@ -622,9 +567,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 			       code);
 			break;
 		    }
-		    if (entries.nbulkentries_len < nentries) {
-			nentries = entries.nbulkentries_len;
-		    }
 
 		    t += nentries;
 		    entry = (struct nvldbentry *)entries.nbulkentries_val;
@@ -793,9 +735,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    printf("VL_GetAddrs returned code = %d\n", code);
 		    continue;
 		}
-		if (addrs.bulkaddrs_len < nentries) {
-		    nentries = addrs.bulkaddrs_len;
-		}
 		addrp = addrs.bulkaddrs_val;
 		for (i = 0; i < nentries; i++, addrp++) {
 		    if ((*addrp & 0xff000000) == 0xff000000)
@@ -821,9 +760,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    printf("VL_GetAddrs returned code = %d\n", code);
 		    continue;
 		}
-		if (addrs.bulkaddrs_len < nentries) {
-		    nentries = addrs.bulkaddrs_len;
-		}
 		addrp = addrs.bulkaddrs_val;
 		for (i = 0; i < nentries; i++, addrp++) {
 		    if ((*addrp & 0xff000000) == 0xff000000) {
@@ -836,7 +772,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 			printf("[0x%x %u] (special multi-homed entry)\n",
 			       *addrp, *addrp);
-			memset(&attrs, 0, sizeof(attrs));
 			attrs.Mask = VLADDR_INDEX;
 			mhaddrs.bulkaddrs_val = 0;
 			mhaddrs.bulkaddrs_len = 0;
@@ -849,9 +784,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 			    printf("VL_GetAddrsU returned code = %d\n", code);
 			    continue;
 			}
-			if (mhaddrs.bulkaddrs_len < mhnentries) {
-			    mhnentries = mhaddrs.bulkaddrs_len;
-			}
 			printf
 			    ("   [%d]: uuid[%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x]\n   addrunique=%d, ip address(es):\n",
 			     attrs.index, uuid.time_low, uuid.time_mid,
@@ -902,9 +834,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 		    printf("VL_GetAddrs returned code = %d\n", code);
 		    continue;
 		}
-		if (addrs1.bulkaddrs_len < nentries1) {
-		    nentries1 = addrs1.bulkaddrs_len;
-		}
 		addrp1 = addrs1.bulkaddrs_val;
 		for (i = 0; i < nentries1; i++, addrp1++) {
 		    if ((*addrp1 & 0xff000000) != 0xff000000) {
@@ -918,7 +847,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 			addrs2.bulkaddrs_val = 0;
 			addrs2.bulkaddrs_len = 0;
-			memset(&attrs, 0, sizeof(attrs));
 			attrs.Mask = VLADDR_INDEX;
 			attrs.index = (base * VL_MHSRV_PERBLK) + index;
 			code =
@@ -928,9 +856,6 @@ handleit(struct cmd_syndesc *as, void *arock)
 			    printf("VL_GetAddrsU returned code = %d\n", code);
 			    break;
 			}
-			if (addrs2.bulkaddrs_len < nentries2) {
-			    nentries2 = addrs2.bulkaddrs_len;
-			}
 
 			addrp2 = addrs2.bulkaddrs_val;
 			for (j = 0; j < nentries2; j++) {
@@ -1094,8 +1019,6 @@ main(int argc, char **argv)
 		"cellname '-host' belongs to (required for auth conns)");
     cmd_AddParm(ts, "-getstats", CMD_FLAG, CMD_OPTIONAL,
 		"print vldb statistics (non interactive)");
-    cmd_AddParm(ts, "-probe", CMD_FLAG, CMD_OPTIONAL,
-		"probe vldb servers, use with -host to probe only one. (non interactive)");
     code = cmd_Dispatch(argc, argv);
     exit(code);
 }
@@ -1351,9 +1274,10 @@ dump_stats(vldstats *stats, vital_vlheader *vital_header)
     int i;
     char strg[30];
     time_t start_time = stats->start_time;
+    struct tm tm;
 
-    afs_ctime(&start_time, strg, sizeof(strg));
-    strg[strlen(strg) - 1] = 0;
+    strftime(strg, sizeof(strg), "%a %b %d %H:%M:%S %Y",
+	     localtime_r(&start_time, &tm));
     printf("Dynamic statistics stats (starting time: %s):\n", strg);
     printf("OpcodeName\t# Requests\t# Aborts\n");
     for (i = 0; i < VL_NUMBER_OPCODESX; i++)
