@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -17,22 +17,14 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vol/physio.c,v 1.12.14.2 2009/03/16 13:11:55 shadow Exp $");
+#include <roken.h>
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <errno.h>
-#ifdef AFS_NT40_ENV
-#include <fcntl.h>
-#else
+#include <assert.h>
+
+#ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
-#include <unistd.h>
-#include <string.h>
-#ifdef	AFS_SUN5_ENV
-#include <sys/fcntl.h>
-#endif
 #endif
+
 #include <rx/xdr.h>
 #include <afs/afsint.h>
 #include <errno.h>
@@ -40,8 +32,8 @@ RCSID
 #include "nfs.h"
 #include "ihandle.h"
 #include "salvage.h"
-#include "afs/assert.h"
-#include "afs/dir.h"
+#include <afs/dir.h>
+#include "vol_internal.h"
 
 /* returns 0 on success, errno on failure */
 int
@@ -49,20 +41,17 @@ ReallyRead(DirHandle * file, int block, char *data)
 {
     FdHandle_t *fdP;
     int code;
+    ssize_t nBytes;
     errno = 0;
     fdP = IH_OPEN(file->dirh_handle);
     if (fdP == NULL) {
 	code = errno;
 	return code;
     }
-    if (FDH_SEEK(fdP, block * AFS_PAGESIZE, SEEK_SET) < 0) {
-	code = errno;
-	FDH_REALLYCLOSE(fdP);
-	return code;
-    }
-    code = FDH_READ(fdP, data, (afs_fsize_t) AFS_PAGESIZE);
-    if (code != AFS_PAGESIZE) {
-	if (code < 0)
+    nBytes = FDH_PREAD(fdP, data, (afs_fsize_t) AFS_PAGESIZE,
+                       ((afs_foff_t)block) * AFS_PAGESIZE);
+    if (nBytes != AFS_PAGESIZE) {
+	if (nBytes < 0)
 	    code = errno;
 	else
 	    code = EIO;
@@ -78,8 +67,8 @@ int
 ReallyWrite(DirHandle * file, int block, char *data)
 {
     FdHandle_t *fdP;
-    extern int VolumeChanged;
     int code;
+    ssize_t nBytes;
 
     errno = 0;
 
@@ -88,14 +77,10 @@ ReallyWrite(DirHandle * file, int block, char *data)
 	code = errno;
 	return code;
     }
-    if (FDH_SEEK(fdP, block * AFS_PAGESIZE, SEEK_SET) < 0) {
-	code = errno;
-	FDH_REALLYCLOSE(fdP);
-	return code;
-    }
-    code = FDH_WRITE(fdP, data, (afs_fsize_t) AFS_PAGESIZE);
-    if (code != AFS_PAGESIZE) {
-	if (code < 0)
+    nBytes = FDH_PWRITE(fdP, data, (afs_fsize_t) AFS_PAGESIZE,
+                        ((afs_foff_t)block) * AFS_PAGESIZE);
+    if (nBytes != AFS_PAGESIZE) {
+	if (nBytes < 0)
 	    code = errno;
 	else
 	    code = EIO;
@@ -103,7 +88,7 @@ ReallyWrite(DirHandle * file, int block, char *data)
 	return code;
     }
     FDH_CLOSE(fdP);
-    VolumeChanged = 1;
+    *(file->volumeChanged) = 1;
     return 0;
 }
 
@@ -113,7 +98,7 @@ ReallyWrite(DirHandle * file, int block, char *data)
  */
 void
 SetSalvageDirHandle(DirHandle * dir, afs_int32 volume, Device device,
-		    Inode inode)
+		    Inode inode, int *volumeChanged)
 {
     static int SalvageCacheCheck = 1;
     memset(dir, 0, sizeof(DirHandle));
@@ -124,6 +109,7 @@ SetSalvageDirHandle(DirHandle * dir, afs_int32 volume, Device device,
     IH_INIT(dir->dirh_handle, device, volume, inode);
     /* Always re-read for a new dirhandle */
     dir->dirh_cacheCheck = SalvageCacheCheck++;
+    dir->volumeChanged = volumeChanged;
 }
 
 void
@@ -169,8 +155,8 @@ FidCpy(DirHandle * tofile, DirHandle * fromfile)
 }
 
 void
-Die(char *msg)
+Die(const char *msg)
 {
     printf("%s\n", msg);
-    assert(1 == 2);
+    osi_Panic("%s\n", msg);
 }
