@@ -211,7 +211,7 @@ extern void DeleteExtraVolumeHeaderFile(struct SalvInfo *salvinfo,
                                         struct VolumeSummary *vsp);
 extern void DistilVnodeEssence(struct SalvInfo *salvinfo, VolumeId vid,
                                VnodeClass class, Inode ino, Unique * maxu);
-extern int GetInodeSummary(struct SalvInfo *salvinfo, FILE *inodeFile,
+extern int GetInodeSummary(struct SalvInfo *salvinfo, FD_t inodeFile,
                            VolumeId singleVolumeNumber);
 extern int GetVolumeSummary(struct SalvInfo *salvinfo,
 			    VolumeId singleVolumeNumber);
@@ -234,7 +234,7 @@ extern void SalvageFileSysParallel(struct DiskPartition64 *partP);
 extern void SalvageFileSys(struct DiskPartition64 *partP, VolumeId singleVolumeNumber);
 extern void SalvageFileSys1(struct DiskPartition64 *partP,
 			    VolumeId singleVolumeNumber);
-extern int SalvageHeader(struct SalvInfo *salvinfo, struct stuff *sp,
+extern int SalvageHeader(struct SalvInfo *salvinfo, struct afs_inode_info *sp,
                         struct InodeSummary *isp, int check, int *deleteMe);
 extern int SalvageIndex(struct SalvInfo *salvinfo, Inode ino, VnodeClass class,
                         int RW, struct ViceInodeInfo *ip, int nInodes,
@@ -247,9 +247,7 @@ extern int SalvageVolume(struct SalvInfo *salvinfo, struct InodeSummary *rwIsp,
 extern void DoSalvageVolumeGroup(struct SalvInfo *salvinfo,
                                  struct InodeSummary *isp, int nVols);
 #ifdef AFS_NT40_ENV
-extern void SalvageVolumeGroup(struct SalvInfo *salvinfo, struct InodeSummary *isp, int nVols);
-#else
-#define SalvageVolumeGroup DoSalvageVolumeGroup
+extern void nt_SalvageVolumeGroup(struct SalvInfo *salvinfo, struct InodeSummary *isp, int nVols);
 #endif
 extern int SalvageVolumeHeaderFile(struct SalvInfo *salvinfo,
                                    struct InodeSummary *isp,
