@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,8 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <roken.h>
 
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
@@ -20,7 +19,7 @@
 #include <afs/kautils.h>
 #include <rx/rxkad.h>
 #include <afs/auth.h>
-#include <afs/cellconfig.h>+
+#include <afs/cellconfig.h>
 #include <afs/cmd.h>
 #include <afs/vlserver.h>
 #include "volser.h"
